;
; DragonROM.asm	Disasembily of Dragon 32/64 Basic ROM.
;
; Disasembled 2006-08-14, P.Harvey-Smith.
;
; From this file you can assemble any of the Dragon basic ROMS.
;
; Define DRAGON64 	to assemble for Dragon 64, 32K rom
; Define DRAGON64RAM	to assemble for Dragon 64, 64K rom
;
; If neither are defined, a Dragon32 ROM will be generated.
;
; Changes :
;
; 2006-08-15, P.Harvey-Smith.
;	Merged Dragon 32, 64 and 64 ram mode disassemblies, this file
;	can now build to any of the Dragon ROMS.
;
;	Re-wrote all CMPX, BRN, LDA, LDX, TST instruction hiding instructions
;	using defines from romdefs.asm.
;
;	Began systematic replacement of system variable reffrences in the 
;	$0000-$03FF reigon with symbolic names from romdefs.asm, this should 
;	make understanding  the code easier.
;


        ifne	dragon64ram
	ORG	$C000				; Dragon 64 ram mode
	else
	ORG     $8000				; Dragon 32/64 ROM mode
	endc
	
	use	romdefs.asm
	use	dgndefs.asm

;
; Basic vector table
;

BasicHWInit:
L8000   JMP     >DoHWInit			; Hardware init, routine to chain to in Y

BasicSWInit:
L8003   JMP     >DoSWInit			; Software Init

BasicKbdIn:
L8006   JMP     >TextScanKbd 			; Scan keyboard, if a key pressed return in A

BasicCursorB:
L8009   JMP     >TextUpdateCurs 		; Update cursor (blink)	

BasicScreenOut:
L800C   JMP     >TextVDUOut 			; Output char in A (to screen)

BasicPrintOut:
L800F   JMP     >PrinterOut 			; Output char in A to printer

BasicJoyIn:
L8012   JMP     >SysReadJoystick 		; Read joysticks

BasicCassOn:
L8015   JMP     >CasMotorOn 			; Turn on cassette motor

BasicCassOff:
L8018   JMP     >CasMotorOff 			; Turn off cassette motor

CasWriteLeader:
L801B   JMP     >LBE68				; Write leader to tape

BasicCassByOut:
L801E   JMP     >CasByteOut 			; Output a byte in A to tape 

BasicCassOnRd:
L8021   JMP     >CasReadLeader 			; Read leader and get ready top read data

BasicCassByIn:
L8024   JMP     >CasByteIn 			; Read a byte in from tape

BasicCassBitIn:
L8027   JMP     >CasBitIn 			; Read a bit from tape

BasicSerIn:
L802A   JMP     >DoSerialIn				; Read a byte from serial port (D64 only)

BasicSerOut:
L802D   JMP     >DoSerialOut				; Send a byte to serial port (D64 only)

BasicSetBaud:
L8030   JMP     >DoSetBaud				; Set baud rate (D64 only)

;
; Basic keyword table (commands).
;

BasCommandWords
	FCS	/FOR/
        FCS     /GO/
        FCS     /REM/
	FCS	/'/
        FCS     /ELSE/
        FCS     /IF/
        FCS     /DATA/
        FCS     /PRINT/
        FCS     /ON/
        FCS     /INPUT/
        FCS     /END/
        FCS     /NEXT/
        FCS     /DIM/
        FCS     /READ/
        FCS     /LET/
        FCS     /RUN/
        FCS     /RESTORE/
        FCS     /RETURN/
        FCS     /STOP/
        FCS     /POKE/
        FCS     /CONT/
        FCS     /LIST/
        FCS     /CLEAR/
        FCS     /NEW/
        FCS     /DEF/
        FCS     /CLOAD/
        FCS     /CSAVE/
        FCS     /OPEN/
        FCS     /CLOSE/
        FCS     /LLIST/
        FCS     /SET/
        FCS     /RESET/
        FCS     /CLS/
        FCS     /MOTOR/
        FCS     /SOUND/
        FCS     /AUDIO/
        FCS     /EXEC/
        FCS     /SKIPF/
        FCS     /DEL/
        FCS     /EDIT/
        FCS     /TRON/
        FCS     /TROFF/
        FCS     /LINE/
        FCS     /PCLS/
        FCS     /PSET/
        FCS     /PRESET/
        FCS     /SCREEN/
        FCS     /PCLEAR/
        FCS     /COLOR/
        FCS     /CIRCLE/
        FCS     /PAINT/
        FCS     /GET/
        FCS     /PUT/
        FCS     /DRAW/
        FCS     /PCOPY/
        FCS     /PMODE/
        FCS     /PLAY/
        FCS     /DLOAD/
        FCS     /RENUM/
        FCS     /TAB(/
        FCS     /TO/
        FCS     /SUB/
        FCS     /FN/
        FCS     /THEN/
        FCS     /NOT/
        FCS     /STEP/
        FCS     /OFF/
	FCS	/+/
	FCS	/-/
	FCS	/*/
	FCS	"/"
	FCS	/^/
        FCS     /AND/
        FCS     /OR/
	FCS	/>/
	FCS	/=/
	FCS	/</
        FCS     /USING/

;
; Keyword dispatch table (commands).
;
BasCommandDisp
        FDB     CmdFor
        FDB     CmdGo
        FDB     CmdREM
        FDB     CmdREM
        FDB     CmdREM
        FDB     CmdIF
        FDB     CmdData
        FDB     CmdPrint
        FDB     CmdON
        FDB     CmdInput
        FDB     CmdEnd
        FDB     CmdNext
        FDB     CmdDim
        FDB     CmdRead
        FDB     CmdLet
        FDB     CmdRun
        FDB     CmdRestore
        FDB     CmdReturn
        FDB     CmdStop
        FDB     CmdPoke
        FDB     CmdCont
        FDB     CmdList
        FDB     CmdClear
        FDB     CmdNew
        FDB     CmdDef
        FDB     LB6D4
        FDB     LB682
        FDB     LB828
        FDB     LB64C
        FDB     CmdLList
        FDB     LB9D2
        FDB     LBA03
        FDB     LBA5F
        FDB     LB981
        FDB     LBA9A
        FDB     CmdAudio
        FDB     LB770
        FDB     LB81E
        FDB     CmdDelete
        FDB     CmdEdit
        FDB     CmdTron
        FDB     CmdTroff
        FDB     CmdLine
        FDB     CmdPCls
        FDB     CmdPset
        FDB     CmdPReset
        FDB     CmdScreen
        FDB     CmdPClear
        FDB     CmdColor
        FDB     CmdCircle
        FDB     CmdPaint
        FDB     CmdGet
        FDB     CmdPut
        FDB     GrDraw
        FDB     CmdPcopy
        FDB     CmdPmode
        FDB     CmdPlay
        FDB     CmdDload
        FDB     CmdRenum

;
; Basic keyword table (functions).
;

BasFunctionWords
        FCS     /SGN/
        FCS     /INT/
        FCS     /ABS/
        FCS     /POS/
        FCS     /RND/
        FCS     /SQR/
        FCS     /LOG/
        FCS     /EXP/
        FCS     /SIN/
        FCS     /COS/
        FCS     /TAN/
        FCS     /ATN/
        FCS     /PEEK/
        FCS     /LEN/
        FCS     /STR$/
        FCS     /VAL/
        FCS     /ASC/
        FCS     /CHR$/
        FCS     /EOF/
        FCS     /JOYSTK/
        FCS     /FIX/
        FCS     /HEX$/
        FCS     /LEFT$/
        FCS     /RIGHT$/
        FCS     /MID$/
        FCS     /POINT/
        FCS     /INKEY$/
        FCS     /MEM/
        FCS     /VARPTR/
        FCS     /INSTR/
        FCS     /TIMER/
        FCS     /PPOINT/
        FCS     /STRING$/
        FCS     /USR/
        
;
; Keyword dispatch table (functions).
;
BasFunctionDisp
	FDB     CmdSGN
        FDB     CmdINT
        FDB     CmdABS
        FDB     CmdPOS
        FDB     CmdRND
        FDB     CmdSQR
        FDB     CmdLOG
        FDB     CmdEXP
        FDB     CmdSIN
        FDB     CmdCOS
        FDB     CmdTAN
        FDB     CmdATN
        FDB     CmdPeek
        FDB     CmdLEN
        FDB     CmdSTRS
        FDB     CmdVAL
        FDB     CmdASC
        FDB     CmdCHRS
        FDB     LB800
        FDB     CmdJoystk
        FDB     CmdFIX
        FDB     CmdHexS
        FDB     CmdLeftS
        FDB     CmdRightS
        FDB     CmdMidS
        FDB     LBA44
        FDB     LB796
        FDB     CmdMEM
        FDB     CmdVarptr
        FDB     CmdInstr
        FDB     CmdTimer
        FDB     CmdPPoint
        FDB     CmdStringS
        FDB     CmdUSR

D8294   FCB     $79
        FDB     CmdPlus
	FCB	$79
        FDB     CmdMinus
        FCB     $7B
        FDB     CmdMultiply
	FCB	$7B
        FDB     CmdDivide
        FCB     $7F
	FDB	CmdExponet
        FCB     $50
        FDB     CmdAND
        FCB     $46
	FDB	CmdOR

BasErrorCodeTable:
D82A9   FCC     "NFSNRGODFCOVOMULBSDD/0IDTMOSLSSTCNUFFDAODNIOFMNOIEDSNE"

MessError
	FCC	/ ERROR/
	FCB     $00
MessIn
        FCC     / IN /
	FCB     $00

MessOK
	FCB     $0D
        FCC     /OK/
        FCB     $0D,$00

MessBreak
        FCB     $0D
        FCC     /BREAK/
        FCB     $00

L82F7   LEAX    4,S
L82F9   LDB     #$12
        STX     <BasTempPtr
        LDA     ,X
        SUBA    #$80
        BNE     L8318
        LDX     1,X
        STX     <BasTempPtr1
        LDX     <BasTempVarDesc
        BEQ     L8314
        CMPX    <BasTempPtr1
        BEQ     L8318
        LDX     <BasTempPtr
        ABX
        BRA     L82F9

L8314   LDX     <BasTempPtr1
        STX     <BasTempVarDesc
L8318   LDX     <BasTempPtr
        TSTA
        RTS

BasChkArrSpaceMv:
L831C   BSR     L8335
L831E   LDU     <$41
        LEAU    1,U
        LDX     <$43
        LEAX    1,X
L8326   LDA     ,-X
        PSHU    A
        CMPX    <$47
        BNE     L8326
        STU     <BasVarFPAcc4
L8330   RTS

BasChkB2Free:
L8331   CLRA
        ASLB
        ADDD    <BasVarEnd
L8335   ADDD    #$003A
        BCS     BasOMError L8342
        STS     <BasBotStack
        CMPD    <BasBotStack
        BCS     L8330

BasOMError:
L8342   LDB     #$0C
SysErr:
L8344   JSR     VectUserError L018E
        JSR     VectSysError L0191
        JSR     >BasicCassOff L8018
        JSR     >SndDisable LBAC3
        JSR     >BasResetStack L8434
        CLR     <TextDevN
        JSR     >L90A5
        JSR     >TextOutQuestion L90F8
        LDX     #BasErrorCodeTable D82A9

SysErr2:
        ABX
        BSR     L839E
        BSR     L839E
        LDX     #MessError-1
L8366   JSR     >TextOutString L90E5
        LDA     <BasCurrentLine
        INCA
        BEQ     BasCmdMode L8371
        JSR     >L9573

BasCmdMode:
L8371   JSR     >L90A5
        LDX     #MessOK
        JSR     >TextOutString L90E5
L837A   JSR     >LB5C6
        LDU     #$FFFF
        STU     <BasCurrentLine
        BCS     L837A
        TST     <CasEOFFlag
        LBNE    LB6FD
        STX     <BasAddrSigByte
        JSR     <BasChrGet
        BEQ     L837A
        BCS     L83A3
        LDB     #$32
        TST     <TextDevN
        BNE     SysErr  L8344
        JSR     >L8F67
        JMP     >L84D6

L839E   LDA     ,X+
        JMP     >L90FA

L83A3   JSR     >BasGetLineNo L869A
L83A6   LDX     <$2B
        STX     BasLinInpHead L02DA
        JSR     >L8F67
        STB     <BasGenCount
        BSR     BasFindLineNo L83FF
        BCS     L83C6
        LDD     <$47
        SUBD    ,X
        ADDD    <BasVarSimpleAddr
        STD     <BasVarSimpleAddr
        LDU     ,X
L83BE   PULU    A
        STA     ,X+
        CMPX    <BasVarSimpleAddr
        BNE     L83BE
L83C6   LDA     BasLinInpBuff L02DC
        BEQ     L83E7
        LDD     <BasVarSimpleAddr
        STD     <$43
        ADDB    <BasGenCount
        ADCA    #$00
        STD     <$41
        JSR     >BasChkArrSpaceMv L831C
        LDU     #$02D8
L83DB   PULU    A
        STA     ,X+
        CMPX    <BasVarFPAcc4
        BNE     L83DB
        LDX     <$41
        STX     <BasVarSimpleAddr
L83E7   BSR     BasVect1 L841F
        BSR     BasVect2 L83ED
        BRA     L837A

BasVect2:
L83ED   LDX     <BasStartProg
L83EF   LDD     ,X
        BEQ     L8414
        LEAU    4,X
L83F5   LDA     ,U+
        BNE     L83F5
        STU     ,X
        LDX     ,X
        BRA     L83EF

BasFindLineNo:
L83FF   LDD     <$2B
        LDX     <BasStartProg
L8403   LDU     ,X
        BEQ     L8410
        CMPD    2,X
        BLS     L8412
        LDX     ,X
        BRA     L8403

L8410   ORCC    #$01
L8412   STX     <$47
L8414   RTS

CmdNew:
L8415   BNE     L8412
BasNew:
L8417   LDX     <BasStartProg
        CLR     ,X+
        CLR     ,X+
        STX     <BasVarSimpleAddr
BasVect1:
L841F   LDX     <BasStartProg
        JSR     >BasSetProgPtrX L85EE
L8424   JSR     VectResetBasMem L0197
        LDX     <AddrFWareRamTop
        STX     <BasVarStrTop
        JSR     >CmdRestore L8514
        LDX     <BasVarSimpleAddr
        STX     <BasVarArrayAddr
        STX     <BasVarEnd
BasResetStack:
L8434   LDX     #BasStrDescStack L01A9
        STX     <BasStrFirstFreeTemp
        LDX     ,S
        LDS     <AddrStack
        CLR     ,-S
        CLR     <BasOldInputPtr
        CLR     <BasOldInputPtr+1
        CLR     <BasDisArraySearch
        JMP     ,X

CmdFor:
L8448   LDA     #$80
        STA     <BasDisArraySearch
        JSR     >CmdLet L86BC
        JSR     >L82F7
        LEAS    2,S
        BNE     L845A
        LDX     <BasTempPtr
        LEAS    B,X
L845A   LDB     #$09
        JSR     >BasChkB2Free L8331
        JSR     >L861B
        LDD     <BasCurrentLine
        PSHS    D,X
        LDB     #$BC
        JSR     >VarCKChar L89AC
        JSR     >VarGetExprCC L8874
        JSR     >L8872
        LDB     <$54
        ORB     #$7F
        ANDB    <$50
        STB     <$50
        LDY     #L8480
        JMP     >L891B

L8480   LDX     #D920E
        JSR     >L93BF
        JSR     <BasChrGetCurr
        CMPA    #$C1
        BNE     L8491
        JSR     <BasChrGet
        JSR     >L8872
L8491   JSR     >L9418
        JSR     >L8917
        LDD     <BasTempVarDesc
        PSHS    D
        LDA     #$80
        PSHS    A
BasRun:
L849F   JSR     VectGetNextCmd L019A
        ANDCC   #$AF
        BSR     BasPollKeyboard L851B
        LDX     <BasAddrSigByte
        STX     <BasDirectTextPtr
        LDA     ,X+
        BEQ     L84B5
        CMPA    #$3A
        BEQ     L84D6
L84B2   JMP     >BasSNError L89B4

L84B5   LDA     ,X++
        STA     <BasBreakFlag
        LBEQ    L8545
        LDD     ,X+
        STD     <BasCurrentLine
        STX     <BasAddrSigByte
        LDA     <BasTronFlag
        BEQ     L84D6
        LDA     #$5B
        JSR     >TextOutChar LB54A
        LDA     <BasCurrentLine
        JSR     >TextOutNum16 L957A
        LDA     #$5D
        JSR     >TextOutChar LB54A
L84D6   JSR     <BasChrGet
        BSR     L84DC
BasBRARun:
        BRA     BasRun  L849F

L84DC   BEQ     L851A
        JSR     VectCmdInterp L0179
        TSTA
        LBPL    CmdLet  L86BC
        CMPA    #$BA
        BHI     L84F5
        LDX     BasAddrCmdDisp L0123
BasDoDipatch:
        ASLA
        TFR     A,B
        ABX
        JSR     <BasChrGet
        JMP     [,X]

L84F5   CMPA    #$FF
        BEQ     L8501
        CMPA    #$CD
        BLS     L84B2
        JMP     [BasAddrDskCmdDisp] L012D

L8501   JSR     <BasChrGet
        CMPA    #$98
        LBEQ    L9B0C
        CMPA    #$9E
        LBEQ    L9D51
        JSR     VectAccessScreen L01A0
        BRA     L84B2

CmdRestore:
L8514   LDX     <BasStartProg
        LEAX    -1,X
L8518   STX     <BasVarDataAddr
L851A   RTS

BasPollKeyboard:
L851B   JSR     >BasicKbdIn L8006
        BEQ     L852A
L8520   CMPA    #$03
        BEQ     CmdStop L8539
        CMPA    #$13
        BEQ     TextWaitKey L852B
        STA     <TextLastKey
L852A   RTS

TextWaitKey:
L852B   JSR     >BasicKbdIn L8006
        BEQ     TextWaitKey L852B
        BRA     L8520

CmdEnd:
L8532   JSR     >LB65C
        JSR     <BasChrGetCurr
        BRA     L853B

CmdStop:
L8539   ORCC    #$01
L853B   BNE     L8570
        LDX     <BasAddrSigByte
        STX     <BasDirectTextPtr
L8541   ROR     <BasBreakFlag
        LEAS    2,S
L8545   LDX     <BasCurrentLine
        CMPX    #$FFFF
        BEQ     L8552
        STX     <BasContLine
        LDX     <BasDirectTextPtr
        STX     <BasOldInputPtr
L8552   CLR     <TextDevN
        LDX     #MessBreak-1
        TST     <BasBreakFlag
        LBPL    BasCmdMode L8371
        JMP     >L8366

CmdCont:
L8560   BNE     L8570
        LDB     #$20
        LDX     <BasOldInputPtr
        LBEQ    SysErr  L8344
        STX     <BasAddrSigByte
        LDX     <BasContLine
        STX     <BasCurrentLine
L8570   RTS

CmdClear:
L8571   BEQ     L859F
        JSR     >L8B23
        PSHS    D
        LDX     <AddrFWareRamTop
        JSR     <BasChrGetCurr
        BEQ     L858A
        JSR     >VarCKComma L89AA
        JSR     >VarGet16Bit L8E83
        LEAX    -1,X
        CMPX    <AddrRamTop
        BHI     L85A2
L858A   TFR     X,D
        SUBD    ,S++
        BCS     L85A2
        TFR     D,U
        SUBD    #$003A
        BCS     L85A2
        SUBD    <BasVarSimpleAddr
        BCS     L85A2
        STU     <AddrStack
        STX     <AddrFWareRamTop
L859F   JMP     >L8424

L85A2   JMP     >BasOMError L8342

CmdRun:
L85A5   JSR     VectRunLink L0194
        JSR     >L98E3
        JSR     >LB65C
        JSR     <BasChrGetCurr
        LBEQ    BasVect1 L841F
        JSR     >L8424
        BRA     L85D2

CmdGo:
L85B9   TFR     A,B
L85BB   JSR     <BasChrGet
        CMPB    #$BC
        BEQ     L85D7
        CMPB    #$BD
        BNE     L860A
        LDB     #$03
        JSR     >BasChkB2Free L8331
        LDU     <BasAddrSigByte
        LDX     <BasCurrentLine
        LDA     #$BD
        PSHS    A,X,U
L85D2   BSR     L85D7
        JMP     >BasRun L849F

L85D7   JSR     <BasChrGetCurr
        JSR     >BasGetLineNo L869A
        BSR     L861E
        LEAX    1,X
        LDD     <$2B
        CMPD    <BasCurrentLine
        BHI     L85E9
BasSkipLineNo:
        LDX     <BasStartProg
L85E9   JSR     >L8403
        BCS     L8605
BasSetProgPtrX:
L85EE   LEAX    -1,X
        STX     <BasAddrSigByte
L85F2   RTS

CmdReturn:
L85F3   BNE     L85F2
        LDA     #$FF
        STA     <BasTempVarDesc
        JSR     >L82F7
        TFR     X,S
        CMPA    #$3D
        BEQ     L860D
D8602   LDB     #$04

        FCB     $8C

L8605   LDB     #$0E
        JMP     >SysErr L8344

L860A   JMP     >BasSNError L89B4

L860D   PULS    A,X,U
        STX     <BasCurrentLine
        STU     <BasAddrSigByte
CmdData:
L8613   BSR     L861B
        FCB	Skip2
CmdREM
L8616	BSR	L861E	;#D8D06
        STX     <BasAddrSigByte
L861A   RTS

L861B   LDB     #$3A

        FCB     Skip1LD

L861E   CLRB
        STB     <BasDelim1
D8621   CLRB
        LDX     <BasAddrSigByte
L8624   TFR     B,A
        LDB     <BasDelim1
        STA     <BasDelim1
L862A   LDA     ,X
        BEQ     L861A
        PSHS    B
        CMPA    ,S+
        BEQ     L861A
        LEAX    1,X
        CMPA    #$22
        BEQ     L8624
        INCA
        BNE     L863F
        LEAX    1,X
L863F   CMPA    #$86
        BNE     L862A
        INC     <BasIfCount
        BRA     L862A

CmdIF:
L8647   JSR     >L8872
        JSR     <BasChrGetCurr
        CMPA    #$81
        BEQ     L8655
        LDB     #$BF
        JSR     >VarCKChar L89AC
L8655   LDA     <BasVarFPAcc1
        BNE     L866C
        CLR     <BasIfCount
L865B   BSR     CmdData L8613
        TSTA
        BEQ     L861A
        JSR     <BasChrGet
        CMPA    #$84
        BNE     L865B
        DEC     <BasIfCount
        BPL     L865B
        JSR     <BasChrGet
L866C   JSR     <BasChrGetCurr
        LBCS    L85D7
        JMP     >L84DC

CmdON:
L8675   JSR     >VarGet8Bit L8E51
        LDB     #$81
        JSR     >VarCKChar L89AC
        PSHS    A
        CMPA    #$BD
        BEQ     L8687
        CMPA    #$BC
L8685   BNE     L860A
L8687   DEC     <$53
        BNE     L8690
        PULS    B
        JMP     >L85BB

L8690   JSR     <BasChrGet
        BSR     BasGetLineNo L869A
        CMPA    #$2C
        BEQ     L8687
        PULS    B,PC

BasGetLineNo:
L869A   LDX     <Misc16BitScratch
        STX     <$2B
L869E   BCC     L8704
        SUBA    #$30
        STA     <BasDelim1
        LDD     <$2B
        CMPA    #$18
        BHI     L8685
        ASLB
        ROLA
        ASLB
        ROLA
        ADDD    <$2B
        ASLB
        ROLA
        ADDB    <BasDelim1
        ADCA    #$00
        STD     <$2B
        JSR     <BasChrGet
        BRA     L869E

CmdLet:
L86BC   JSR     >VarGetVar L8A94
        STX     <BasTempVarDesc
        LDB     #$CB
        JSR     >VarCKChar L89AC
        LDA     <BasVarType
        PSHS    A
        JSR     >VarGetStr L8887
        PULS    A
        RORA
        JSR     >L8879
        LBEQ    L93DE
L86D7   JSR     VectAssignStr L019D
        LDX     <BasVarAssign16
        LDD     <AddrStack
        CMPD    2,X
        BCC     L86F4
        CMPX    <BasVarSimpleAddr
        BCS     L86F4
        LDB     ,X
        JSR     >L8C50
        LDX     <$4D
        JSR     >L8D89
        LDX     #$0056
L86F4   STX     <$4D
        JSR     >L8DBB
        LDU     <$4D
        LDX     <BasTempVarDesc
        PULU    D,Y
        STA     ,X
        STY     2,X
L8704   RTS

        FCC     /?REDO/
        FCB     $0D
        FCB     $00

L870C   LDB     #$24
        TST     <TextDevN
        BEQ     L8715
L8712   JMP     >SysErr L8344

L8715   LDA     <BasInputFlag
        BEQ     L8720
        LDX     <BasVarDataLine
        STX     <BasCurrentLine
        JMP     >BasSNError L89B4

L8720   LDX     #L8704
        JSR     >TextOutString L90E5
        LDX     <BasDirectTextPtr
        STX     <BasAddrSigByte
        RTS

CmdInput:
L872B   JSR     >BasChkDirect L9C76
        BSR     L8733
        CLR     <TextDevN
        RTS

L8733   CMPA    #$23
        BNE     L8740
        JSR     >LB7D7
        JSR     >LB623
        JSR     >VarCKComma L89AA
L8740   CMPA    #$22
        BNE     L874F
        JSR     >L8975
        LDB     #$3B
        JSR     >VarCKChar L89AC
        JSR     >L90E8
L874F   LDX     #BasLinInpBuff L02DC
        CLR     ,X
        TST     <TextDevN
        BNE     CmdReadFromX L877A
        BSR     L8760
        LDB     #$2C
        STB     ,X
        BRA     CmdReadFromX L877A

L8760   JSR     >TextOutQuestion L90F8
        JSR     >TextOutSpace L90F5
L8766   JSR     >LB5C6
        BCC     L8770
        LEAS    4,S
        JMP     >L8541

L8770   LDB     #$30
        TST     <CasEOFFlag
        BNE     L8712
        RTS

CmdRead:
L8777   LDX     <BasVarDataAddr
        
	FCB	Skip1LD			; LDA     #$4F
CmdReadFromX:
L877A   CLRA

        STA     <BasInputFlag
        STX     <TextKbdBuffAddr
L877F   JSR     >VarGetVar L8A94
        STX     <BasTempVarDesc
        LDX     <BasAddrSigByte
        STX     <$2B
        LDX     <TextKbdBuffAddr
        LDA     ,X
        BNE     L879A
        LDA     <BasInputFlag
        BNE     L87EA
        JSR     VectReReqestIn L017C
        JSR     >TextOutQuestion L90F8
        BSR     L8760
L879A   STX     <BasAddrSigByte
        JSR     <BasChrGet
        LDB     <BasVarType
        BEQ     L87C9
        LDX     <BasAddrSigByte
        STA     <BasDelim1
        CMPA    #$22
        BEQ     L87BC
        LEAX    -1,X
        CLRA
        STA     <BasDelim1
        JSR     >LB595
        TST     <CasIOFlag
        BNE     L87BC
        LDA     #$3A
        STA     <BasDelim1
        LDA     #$2C
L87BC   STA     <BasDelim2
        JSR     >L8C61
        JSR     >L897A
        JSR     >L86D7
        BRA     L87CF

L87C9   JSR     >L94BD
        JSR     >L93DE
L87CF   JSR     <BasChrGetCurr
        BEQ     L87D9
        CMPA    #$2C
        LBNE    L870C
L87D9   LDX     <BasAddrSigByte
        STX     <TextKbdBuffAddr
        LDX     <$2B
        STX     <BasAddrSigByte
        JSR     <BasChrGetCurr
        BEQ     L8806
        JSR     >VarCKComma L89AA
        BRA     L877F

L87EA   STX     <BasAddrSigByte
        JSR     >L861B
        LEAX    1,X
        TSTA
        BNE     L87FE
        LDB     #$06
        LDU     ,X++
        BEQ     L883B
        LDD     ,X++
        STD     <BasVarDataLine
L87FE   LDA     ,X
        CMPA    #$86
        BNE     L87EA
        BRA     L879A

L8806   LDX     <TextKbdBuffAddr
        LDB     <BasInputFlag
        LBNE    L8518
        LDA     ,X
        BEQ     L8818
        LDX     #L8818
        JMP     >TextOutString L90E5

L8818   RTS

        FCC     /?EXTRA IGNORED/
        FCB     $0D
        FCB     $00

CmdNext:
L8829   BNE     L882F
        LDX     <Misc16BitScratch
        BRA     L8832

L882F   JSR     >VarGetVar L8A94
L8832   STX     <BasTempVarDesc
        JSR     >L82F7
        BEQ     L883D
        LDB     #$00
L883B   BRA     L8884

L883D   TFR     X,S
        LEAX    3,X
        JSR     >L93BF
        LDA     8,S
        STA     <$54
        LDX     <BasTempVarDesc
        JSR     >L910B
        JSR     >L93DE
        LEAX    9,S
        JSR     >L9441
        SUBB    8,S
        BEQ     L8865
        LDX     14,S
        STX     <BasCurrentLine
        LDX     $10,S
        STX     <BasAddrSigByte
L8862   JMP     >BasRun L849F

L8865   LEAS    $12,S
        JSR     <BasChrGetCurr
        CMPA    #$2C
        BNE     L8862
        JSR     <BasChrGet
        BSR     L882F
L8872   BSR     VarGetStr L8887
VarGetExprCC:
L8874   ANDCC   #$FE

        FCB	Skip2TST		; TST     $1A01
VarGetExpr:	
L8877   ORCC	#$01

L8879   TST     <BasVarType
        BCS     L8880
        BPL     L8818

        FCB	Skip2
L8880   BMI	L8818

BasTMError:
        LDB     #$18
L8884   JMP     >SysErr L8344

VarGetStr:
L8887   BSR     L88F7
L8889   CLRA
     
	FCB	Skip2
L888B	PSHS	B   

        PSHS    A
        LDB     #$01
        JSR     >BasChkB2Free L8331
        JSR     >L8954
        CLR     <BasTempRelateFlag
L8899   JSR     <BasChrGetCurr
L889B   SUBA    #$CA
        BCS     L88B2
        CMPA    #$03
        BCC     L88B2
        CMPA    #$01
        ROLA
        EORA    <BasTempRelateFlag
        CMPA    <BasTempRelateFlag
        BCS     L8910
        STA     <BasTempRelateFlag
        JSR     <BasChrGet
        BRA     L889B

L88B2   LDB     <BasTempRelateFlag
        BNE     L88E9
        LBCC    L8925
        ADDA    #$07
        BCC     L8925
        ADCA    <BasVarType
        LBEQ    L8D55
        ADCA    #$FF
        PSHS    A
        ASLA
        ADDA    ,S+
        LDX     #D8294
        LEAX    A,X
L88D0   PULS    A
        CMPA    ,X
        BCC     L892B
        BSR     VarGetExprCC L8874
L88D8   PSHS    A
        BSR     L8905
        LDX     <$3D
        PULS    A
        BNE     L88FF
        TSTA
        LBEQ    L8951
        BRA     L8934

L88E9   ASL     <BasVarType
        ROLB
        BSR     L88F7
        LDX     #D88FC
        STB     <BasTempRelateFlag
        CLR     <BasVarType
        BRA     L88D0

L88F7   LDX     <BasAddrSigByte
        JMP     >BasSetProgPtrX L85EE

D88FC   FCB     $64
	FDB	L8A31
;        FCB     $8A
;        FCB     $31

L88FF   CMPA    ,X
        BCC     L8934
        BRA     L88D8

L8905   LDD     1,X
        PSHS    D
        BSR     L8913
        LDB     <BasTempRelateFlag
        LBRA    L888B

L8910   JMP     >BasSNError L89B4

L8913   LDB     <$54
        LDA     ,X
L8917   PULS    Y
        PSHS    B
L891B   LDB     <BasVarFPAcc1
        LDX     <$50
        LDU     <BasVarAssign16
        PSHS    B,X,U
        JMP     ,Y

L8925   LDX     <Misc16BitScratch
        LDA     ,S+
        BEQ     L8951
L892B   CMPA    #$64
        BEQ     L8932
        JSR     >VarGetExprCC L8874
L8932   STX     <$3D
L8934   PULS    B
        CMPA    #$5A
        BEQ     L8953
        CMPA    #$7D
        BEQ     L8953
        LSRB
        STB     <BasRelateFlag
        PULS    A,X,U
        STA     <BasVarFPAcc2
        STX     <$5D
        STU     <$5F
        PULS    B
        STB     <$61
        EORB    <$54
        STB     <$62
L8951   LDB     <BasVarFPAcc1
L8953   RTS

L8954   JSR     VectEvaluateExpr L018B
        CLR     <BasVarType
        JSR     <BasChrGet
        BCC     L8960
L895D   JMP     >L94BD

L8960   JSR     >L8ADF
        BCC     L89C1
        CMPA    #$2E
        BEQ     L895D
        CMPA    #$C4
        BEQ     L89B9
        CMPA    #$C3
        BEQ     L8954
        CMPA    #$22
        BNE     L897F
L8975   LDX     <BasAddrSigByte
        JSR     >L8C5B
L897A   LDX     <$64
        STX     <BasAddrSigByte
        RTS

L897F   CMPA    #$C0
        BNE     L8990
        LDA     #$5A
        JSR     >L888B
        JSR     >L8B2D
        COMA
        COMB
        JMP     >VarAssign16Bit2 L8C37

L8990   CMPA    #$BE
        LBEQ    L9CC4
        CMPA    #$26
        LBEQ    L9C1B
        INCA
        BEQ     L89CD
L899F   BSR     VarCKOpBrac L89A7
        JSR     >VarGetStr L8887
VarCKClBrac:
L89A4   LDB     #$29


        FCB	Skip2		;CMPX    #$C628
VarCKOpBrac:
L89A7   LDB	#$28

        FCB	Skip2		; CMPX    #$C62C

VarCKComma:
L89AA   LDB	#$2C

VarCKChar:
L89AC   CMPB    [BasAddrSigByte] L00A6
        BNE     BasSNError L89B4
        JMP     <BasChrGet

BasSNError:
L89B4   LDB     #$02
        JMP     >SysErr L8344

L89B9   LDA     #$7D
        JSR     >L888B
        JMP     >L96DE

L89C1   JSR     >VarGetVar L8A94
D89C4   STX     <BasVarAssign16
        LDA     <BasVarType
        BNE     L8953
        JMP     >L93BF

L89CD   JSR     <BasChrGet
        TFR     A,B
        ASLB
        JSR     <BasChrGet
        CMPB    #$42
        BLS     L89DC
        JMP     [BasAddrDskFuncDisp] L0132

L89DC   PSHS    B
        CMPB    #$2C
        BCS     L8A04
        CMPB    #$34
        BCC     L8A06
        BSR     VarCKOpBrac L89A7
        LDA     ,S
        CMPA    #$32
        BCC     L8A06
        JSR     >VarGetStr L8887
        BSR     VarCKComma L89AA
        JSR     >VarGetExpr L8877
        PULS    A
        LDU     <BasVarAssign16
        PSHS    A,U
        JSR     >VarGet8Bit L8E51
        PULS    A
        PSHS    D
        
	FCB	Skip2LDX
L8A04   
	BSR	L899F
L8A06   PULS    B
        LDX     BasAddrFuncDisp L0128
        ABX
        JSR     [,X]
        JMP     >VarGetExprCC L8874

CmdOR:
L8A11   FCB	Skip1LD		; LDA     #$4F
CmdAND:
L8A12	CLRA
        STA     <BasGenCount
        JSR     >L8B2D
        STD     <BasDelim1
        JSR     >L93F5
        JSR     >L8B2D
        TST     <BasGenCount
        BNE     L8A2A
        ANDA    <BasDelim1
        ANDB    <BasDelim2
        BRA     L8A2E

L8A2A   ORA     <BasDelim1
        ORB     <BasDelim2
L8A2E   JMP     >VarAssign16Bit2 L8C37

L8A31   JSR     >L8879
        BNE     L8A46
        LDA     <$61
        ORA     #$7F
        ANDA    <$5D
        STA     <$5D
        LDX     #BasVarFPAcc2 L005C
        JSR     >L9441
        BRA     L8A7C

L8A46   CLR     <BasVarType
        DEC     <BasTempRelateFlag
        JSR     >L8D9D
        STB     <$56
        STX     <$58
        LDX     <$5F
        JSR     >VarDelVar L8D9F
        LDA     <$56
        PSHS    B
        SUBA    ,S+
        BEQ     L8A65
        LDA     #$01
        BCC     L8A65
        LDB     <$56
        NEGA
L8A65   STA     <$54
        LDU     <$58
        INCB
L8A6A   DECB
        BNE     L8A71
        LDB     <$54
        BRA     L8A7C

L8A71   LDA     ,X+
        CMPA    ,U+
        BEQ     L8A6A
        LDB     #$FF
        BCC     L8A7C
        NEGB
L8A7C   ADDB    #$01
        ROLB
        ANDB    <BasRelateFlag
        BEQ     L8A85
        LDB     #$FF
L8A85   JMP     >L9427

L8A88   JSR     >VarCKComma L89AA
CmdDim:
L8A8B   LDB     #$01
        BSR     L8A97
        JSR     <BasChrGetCurr
        BNE     L8A88
        RTS

VarGetVar:
L8A94   CLRB
        JSR     <BasChrGetCurr
L8A97   STB     <BasArrayEval
L8A99   STA     <BasVarLastInUse
        JSR     <BasChrGetCurr
        BSR     L8ADF
        LBCS    BasSNError L89B4
        CLRB
        STB     <BasVarType
        JSR     <BasChrGet
        BCS     L8AAE
        BSR     L8ADF
        BCS     L8AB8
L8AAE   TFR     A,B
L8AB0   JSR     <BasChrGet
        BCS     L8AB0
        BSR     L8ADF
        BCC     L8AB0
L8AB8   CMPA    #$24
        BNE     L8AC2
        COM     <BasVarType
        ADDB    #$80
        JSR     <BasChrGet
L8AC2   STB     <$38
        ORA     <BasDisArraySearch
        SUBA    #$28
        LBEQ    L8B44
        CLR     <BasDisArraySearch
        LDX     <BasVarSimpleAddr
        LDD     <BasVarLastInUse
L8AD2   CMPX    <BasVarArrayAddr
        BEQ     L8AE8
        CMPD    ,X++
        BEQ     L8B19
        LEAX    5,X
        BRA     L8AD2

L8ADF   CMPA    #$41
        BCS     L8AE7
        SUBA    #$5B
        SUBA    #$A5
L8AE7   RTS

L8AE8   LDX     #Misc16BitScratch L008A
        LDU     ,S
        CMPU    #D89C4
        BEQ     L8B1B
        LDD     <BasVarEnd
        STD     <$43
        ADDD    #$0007
        STD     <$41
        LDX     <BasVarArrayAddr
        STX     <$47
        JSR     >BasChkArrSpaceMv L831C
        LDX     <$41
        STX     <BasVarEnd
        LDX     <BasVarFPAcc4
        STX     <BasVarArrayAddr
        LDX     <$47
        LDD     <BasVarLastInUse
        STD     ,X++
        CLRA
        CLRB
        STD     ,X
        STD     2,X
        STA     4,X
L8B19   STX     <BasVarPtrLast
L8B1B   RTS

D8B1C   FCB     $90
        FCB     $80
        FCB     $00
        FCB     $00
        FCB     $00

L8B21   JSR     <BasChrGet
L8B23   JSR     >L8872
L8B26   JSR     >VarGetExprCC L8874
VarGetUsr:
L8B29   LDA     <$54
        BMI     BasFCError L8B8D
L8B2D   JSR     >VarGetExprCC L8874
        LDA     <BasVarFPAcc1
        CMPA    #$90
        BCS     L8B3E
        LDX     #D8B1C
        JSR     >L9441
        BNE     BasFCError L8B8D
L8B3E   JSR     >L9473
        LDD     <BasVarAssign16
        RTS

L8B44   LDB     <BasArrayEval
        LDA     <BasVarType
        PSHS    D
        CLRB
L8B4B   LDX     <BasVarLastInUse
        PSHS    B,X
        BSR     L8B21
        PULS    B,X,Y
        STX     <BasVarLastInUse
        LDU     <BasVarAssign16
        PSHS    Y,U
        INCB
        JSR     <BasChrGetCurr
        CMPA    #$2C
        BEQ     L8B4B
        STB     <BasGenCount
        JSR     >VarCKClBrac L89A4
        PULS    D
        STA     <BasVarType
        STB     <BasArrayEval
        LDX     <BasVarArrayAddr
L8B6D   CMPX    <BasVarEnd
        BEQ     L8B92
        LDD     <BasVarLastInUse
        CMPD    ,X
        BEQ     L8B7E
        LDD     2,X
        LEAX    D,X
        BRA     L8B6D

L8B7E   LDB     #$12
        LDA     <BasArrayEval
        BNE     L8B8F
        LDB     <BasGenCount
        CMPB    4,X
        BEQ     L8BE3
L8B8A   LDB     #$10

        FCB     $8C

BasFCError:
L8B8D   LDB     #$08
L8B8F   JMP     >SysErr L8344

L8B92   LDD     #$0005
        STD     <$64
        LDD     <BasVarLastInUse
        STD     ,X
        LDB     <BasGenCount
        STB     4,X
        JSR     >BasChkB2Free L8331
        STX     <$41
L8BA4   LDB     #$0B
        CLRA
        TST     <BasArrayEval
        BEQ     L8BB0
        PULS    D
        ADDD    #$0001
L8BB0   STD     5,X
        BSR     L8C11
        STD     <$64
        LEAX    2,X
        DEC     <BasGenCount
        BNE     L8BA4
        STX     <BasTempPtr
        ADDD    <BasTempPtr
        LBCS    BasOMError L8342
        TFR     D,X
        JSR     >L8335
        SUBD    #$0035
        STD     <BasVarEnd
        CLRA
L8BCF   LEAX    -1,X
        STA     5,X
        CMPX    <BasTempPtr
        BNE     L8BCF
        LDX     <$41
        LDA     <BasVarEnd
        SUBD    <$41
        STD     2,X
        LDA     <BasArrayEval
        BNE     L8C10
L8BE3   LDB     4,X
        STB     <BasGenCount
        CLRA
        CLRB
L8BE9   STD     <$64
        PULS    D
        STD     <BasVarAssign16
        CMPD    5,X
        BCC     L8C2E
        LDU     <$64
        BEQ     L8BFC
        BSR     L8C11
        ADDD    <BasVarAssign16
L8BFC   LEAX    2,X
        DEC     <BasGenCount
        BNE     L8BE9
        STD     ,--S
        ASLB
        ROLA
        ASLB
        ROLA
        ADDD    ,S++
        LEAX    D,X
        LEAX    5,X
        STX     <BasVarPtrLast
L8C10   RTS

L8C11   LDA     #$10
        STA     <BasVarFPAcc4
        LDD     5,X
        STD     <BasBotStack
        CLRA
        CLRB
L8C1B   ASLB
        ROLA
        BCS     L8C2E
        ASL     <$65
        ROL     <$64
        BCC     L8C29
        ADDD    <BasBotStack
        BCS     L8C2E
L8C29   DEC     <BasVarFPAcc4
        BNE     L8C1B
        RTS

L8C2E   JMP     >L8B8A

CmdMEM:
L8C31   TFR     S,D

	ifne	DRAGON64
	JMP     >L9FCE
	else
        SUBD    <BasVarEnd
VarAssign16Bit:
L8C35   FCB     $21
	endc

VarAssign8Bit:
L8C36   CLRA
VarAssign16Bit2:
L8C37   CLR     <BasVarType
        STD     <$50
        LDB     #$90
        JMP     >L942D

CmdSTRS:
L8C40   JSR     >VarGetExprCC L8874
        LDU     #$03D9
        JSR     >L958A
        LEAS    2,S
        LDX     #$03D8
        BRA     L8C5B

L8C50   STX     <$4D
BasResStr:
L8C52   BSR     BasResStr2 L8CB3
L8C54   STX     <$58
        STB     <$56
        RTS

L8C59   LEAX    -1,X
L8C5B   LDA     #$22
L8C5D   STA     <BasDelim1
        STA     <BasDelim2
L8C61   LEAX    1,X
        STX     <$62
        STX     <$58
        LDB     #$FF
L8C69   INCB
        LDA     ,X+
        BEQ     L8C7A
        CMPA    <BasDelim1
        BEQ     L8C76
        CMPA    <BasDelim2
        BNE     L8C69
L8C76   CMPA    #$22
        BEQ     L8C7C
L8C7A   LEAX    -1,X
L8C7C   STX     <$64
        STB     <$56
        JSR     VectResetBasMem L0197
        LDU     <$62
        CMPU    #$03D9
        BHI     L8C92
        BSR     L8C50
        LDX     <$62
        JSR     >L8D8B
L8C92   LDX     <BasStrFirstFreeTemp
        CMPX    #CasFNameLen L01D1
        BNE     L8C9E
        LDB     #$1E
L8C9B   JMP     >SysErr L8344

L8C9E   LDA     <$56

	ifne	dragon64
	STA	,X
	else
        STA     0,X
	endc
	
        LDD     <$58
        STD     2,X
        LDA     #$FF
        STA     <BasVarType
        STX     <BasStrLastUsedTemp
        STX     <BasVarAssign16
        LEAX    5,X
        STX     <BasStrFirstFreeTemp
        RTS

BasResStr2:
L8CB3   CLR     <BasGarbageFlag
L8CB5   CLRA
        PSHS    D
        LDD     <BasVarStrTop
        SUBD    ,S+
        CMPD    <AddrStack
        BCS     L8CCB
        STD     <BasVarStrTop
        LDX     <BasVarStrTop
        LEAX    1,X
        STX     <BasStrUtil
        PULS    B,PC

L8CCB   LDB     #$1A
        COM     <BasGarbageFlag
        BEQ     L8C9B
        BSR     VarGarbageCollect L8CD7
        PULS    B
        BRA     L8CB5

VarGarbageCollect:
L8CD7   LDX     <AddrFWareRamTop
L8CD9   STX     <BasVarStrTop
        CLRA
        CLRB
        STD     <$4B
        LDX     <AddrStack
        STX     <$47
        LDX     #BasStrDescStack L01A9
L8CE6   CMPX    <BasStrFirstFreeTemp
        BEQ     L8CEE
        BSR     L8D1E
        BRA     L8CE6

L8CEE   LDX     <BasVarSimpleAddr
L8CF0   CMPX    <BasVarArrayAddr
        BEQ     L8CF8
        BSR     L8D18
        BRA     L8CF0

L8CF8   STX     <$41
L8CFA   LDX     <$41
L8CFC   CMPX    <BasVarEnd
        BEQ     L8D35
        LDD     2,X
        ADDD    <$41
        STD     <$41
D8D06   LDA     1,X
        BPL     L8CFA
        LDB     4,X
        ASLB
        ADDB    #$05
        ABX
L8D10   CMPX    <$41
        BEQ     L8CFC
        BSR     L8D1E
        BRA     L8D10

L8D18   LDA     1,X
        LEAX    2,X
        BPL     L8D32
L8D1E   LDB     ,X
        BEQ     L8D32
        LDD     2,X
        CMPD    <BasVarStrTop
        BHI     L8D32
        CMPD    <$47
        BLS     L8D32
        STX     <$4B
        STD     <$47
L8D32   LEAX    5,X
L8D34   RTS

L8D35   LDX     <$4B
        BEQ     L8D34
        CLRA
        LDB     ,X
        DECB
        ADDD    <$47
        STD     <$43
        LDX     <BasVarStrTop
        STX     <$41
        JSR     >L831E
        LDX     <$4B
        LDD     <BasVarFPAcc4
        STD     2,X
        LDX     <BasVarFPAcc4
        LEAX    -1,X
        JMP     >L8CD9

L8D55   LDD     <BasVarAssign16
        PSHS    D
        JSR     >L8954
        JSR     >VarGetExpr L8877
        PULS    X
        STX     <$62
        LDB     ,X
        LDX     <BasVarAssign16
        ADDB    ,X
        BCC     L8D70
        LDB     #$1C
        JMP     >SysErr L8344

L8D70   JSR     >L8C50
        LDX     <$62
        LDB     ,X
        BSR     L8D89
        LDX     <$4D
        BSR     VarDelVar L8D9F
        BSR     L8D8B
        LDX     <$62
        BSR     VarDelVar L8D9F
        JSR     >L8C92
        JMP     >L8899

L8D89   LDX     2,X
L8D8B   LDU     <BasStrUtil
        INCB
        BRA     L8D94

L8D90   LDA     ,X+
        STA     ,U+
L8D94   DECB
        BNE     L8D90
        STU     <BasStrUtil
D8D99   RTS

BasGetStrLenAddr:
L8D9A   JSR     >VarGetExpr L8877
L8D9D   LDX     <BasVarAssign16
VarDelVar:
L8D9F   LDB     ,X
        BSR     L8DBB
        BNE     L8DB8
        LDX     7,X
        LEAX    -1,X
        CMPX    <BasVarStrTop
        BNE     L8DB5
        PSHS    B
        ADDD    <BasVarStrTop
        STD     <BasVarStrTop
        PULS    B
L8DB5   LEAX    1,X
        RTS

L8DB8   LDX     2,X
        RTS

L8DBB   CMPX    <BasStrLastUsedTemp
        BNE     L8DC6
        STX     <BasStrFirstFreeTemp
        LEAX    -5,X
        STX     <BasStrLastUsedTemp
        CLRA
L8DC6   RTS

CmdLEN:
L8DC7   BSR     L8DCC
L8DC9   JMP     >VarAssign8Bit L8C36

L8DCC   BSR     BasGetStrLenAddr L8D9A
        CLR     <BasVarType
        TSTB
        RTS

CmdCHRS:
L8DD2   JSR     >L8E54
L8DD5   LDB     #$01
        JSR     >BasResStr2 L8CB3
        LDA     <$53
        JSR     >L8C54
        STA     ,X
L8DE1   LEAS    2,S
L8DE3   JMP     >L8C92

CmdASC:
L8DE6   BSR     BasGetStrFirst L8DEA
        BRA     L8DC9

BasGetStrFirst:
L8DEA   BSR     L8DCC
        BEQ     L8E4C
        LDB     ,X
        RTS

CmdLeftS:
L8DF1   BSR     L8E3B
L8DF3   CLRA
L8DF4   CMPB    ,X
        BLS     L8DFB
        LDB     ,X
        CLRA
L8DFB   PSHS    D
        JSR     >BasResStr L8C52
        LDX     <$4D
        BSR     VarDelVar L8D9F
        PULS    B
        ABX
        PULS    B
        JSR     >L8D8B
        BRA     L8DE3

CmdRightS:
L8E0E   BSR     L8E3B
        SUBA    ,X
        NEGA
        BRA     L8DF4

CmdMidS:
L8E15   LDB     #$FF
        STB     <$53
        JSR     <BasChrGetCurr
        CMPA    #$29
        BEQ     L8E24
        JSR     >VarCKComma L89AA
        BSR     VarGet8Bit L8E51
L8E24   BSR     L8E3B
        BEQ     L8E4C
        CLRB
        DECA
        CMPA    ,X
        BCC     L8DFB
        TFR     A,B
        SUBB    ,X
        NEGB
        CMPB    <$53
        BLS     L8DFB
        LDB     <$53
        BRA     L8DFB

L8E3B   JSR     >VarCKClBrac L89A4
        LDU     ,S
        LDX     5,S
        STX     <$4D
        LDA     4,S
        LDB     4,S
        LEAS    7,S
        TFR     U,PC

L8E4C   JMP     >BasFCError L8B8D

L8E4F   JSR     <BasChrGet
VarGet8Bit:
L8E51   JSR     >L8872
L8E54   JSR     >L8B26
        TSTA
        BNE     L8E4C
        JMP     <BasChrGetCurr

CmdVAL:
L8E5C   JSR     >L8DCC
        LBEQ    L9182
        LDU     <BasAddrSigByte
        STX     <BasAddrSigByte
        ABX
        LDA     ,X
        PSHS    A,X,U
        CLR     ,X
        JSR     <BasChrGetCurr
        JSR     >L94BD
        PULS    A,X,U
        STA     ,X
        STU     <BasAddrSigByte
        RTS

L8E7A   BSR     VarGet16Bit L8E83
        STX     <$2B
VarGetComma8:
L8E7E   JSR     >VarCKComma L89AA
        BRA     VarGet8Bit L8E51

VarGet16Bit:
L8E83   JSR     >L8872
L8E86   LDA     <$54
        BMI     L8E4C
        LDA     <BasVarFPAcc1
        CMPA    #$90
        BHI     L8E4C
        JSR     >L9473
        LDX     <BasVarAssign16
        RTS

CmdPeek:
L8E96   BSR     L8E86
        LDB     ,X
        JMP     >VarAssign8Bit L8C36

CmdPoke:
L8E9D   BSR     L8E7A
        LDX     <$2B
        STB     ,X
        RTS

CmdLList:
L8EA4   LDB     #$FE
        STB     <TextDevN
        JSR     <BasChrGetCurr
CmdList:
L8EAA   PSHS    CC
        JSR     >BasGetLineNo L869A
        JSR     >BasFindLineNo L83FF
        STX     <BasListLine
        PULS    CC
        BEQ     L8ECA
        JSR     <BasChrGetCurr
        BEQ     L8ECF
        CMPA    #$C4
        BNE     L8EC9
        JSR     <BasChrGet
        BEQ     L8ECA
        JSR     >BasGetLineNo L869A
        BEQ     L8ECF
L8EC9   RTS

L8ECA   LDU     #$FFFF
        STU     <$2B
L8ECF   LEAS    2,S
        LDX     <BasListLine
L8ED3   JSR     >L90A5
        JSR     >LB77B
        LDD     ,X
        BNE     L8EE5
L8EDD   JSR     >LB663
        CLR     <TextDevN
        JMP     >BasCmdMode L8371

L8EE5   STX     <BasListLine
        LDD     2,X
        CMPD    <$2B
        BHI     L8EDD
        JSR     >TextOutNum16 L957A
        JSR     >TextOutSpace L90F5
        LDX     <BasListLine
        BSR     L8F08
        LDX     [BasListLine] L0066
        LDU     #$02DD
L8EFF   LDA     ,U+
        BEQ     L8ED3
        JSR     >L90FA
        BRA     L8EFF

L8F08   JSR     VectDeTokenize L01A6
        LEAX    4,X
        LDY     #$02DD
L8F11   LDA     ,X+
        BEQ     L8F66
        BMI     L8F2C
        CMPA    #$3A
        BNE     L8F28
        LDB     ,X
        CMPB    #$84
        BEQ     L8F11
        CMPB    #$83
        BEQ     L8F11

        FCB	Skip2 
	
L8F26   LDA	#$21
L8F28   BSR     L8F5A
        BRA     L8F11

L8F2C   LDU     #$0116
        CMPA    #$FF
        BNE     L8F37
        LDA     ,X+
        LEAU    5,U
L8F37   ANDA    #$7F
L8F39   LEAU    10,U
        TST     ,U
        BEQ     L8F26
        SUBA    ,U
        BPL     L8F39
        ADDA    ,U
        LDU     1,U
L8F47   DECA
        BMI     L8F50
L8F4A   TST     ,U+
        BPL     L8F4A
        BRA     L8F47

L8F50   LDA     ,U
        BSR     L8F5A
        TST     ,U+
        BPL     L8F50
        BRA     L8F11

L8F5A   CMPY    #$03D6
        BCC     L8F66
        ANDA    #$7F
        STA     ,Y+
        CLR     ,Y
L8F66   RTS

L8F67   JSR     VectTokenize L01A3
        LDX     <BasAddrSigByte
        LDU     #BasLinInpBuff L02DC
L8F6F   CLR     <$43
        CLR     <$44
L8F73   LDA     ,X+
        BEQ     L8F98
        TST     <$43
        BEQ     L8F8A
        JSR     >L8ADF
        BCC     L8F98
        CMPA    #$30
        BCS     L8F88
        CMPA    #$39
        BLS     L8F98
L8F88   CLR     <$43
L8F8A   CMPA    #$20
        BEQ     L8F98
        STA     <$42
        CMPA    #$22
        BEQ     L8FCC
        TST     <$44
        BEQ     L8FB1
L8F98   STA     ,U+
        BEQ     L8FA2
        CMPA    #$3A
        BEQ     L8F6F
L8FA0   BRA     L8F73

L8FA2   CLR     ,U+
        CLR     ,U+
        TFR     U,D
        SUBD    #$02DA
        LDX     #$02DB
        STX     <BasAddrSigByte
        RTS

L8FB1   CMPA    #$3F
        BNE     L8FB9
        LDA     #$87
        BRA     L8F98

L8FB9   CMPA    #$27
        BNE     L8FD0
        LDD     #$3A83
        STD     ,U++
L8FC2   CLR     <$42
L8FC4   LDA     ,X+
        BEQ     L8F98
        CMPA    <$42
        BEQ     L8F98
L8FCC   STA     ,U+
        BRA     L8FC4

L8FD0   CMPA    #$30
        BCS     L8FD8
        CMPA    #$3C
        BCS     L8F98
L8FD8   LEAX    -1,X
        PSHS    X,U
        CLR     <$41
        LDU     #$0116
L8FE1   CLR     <$42
L8FE3   LEAU    10,U
        LDA     ,U
        BEQ     L901A
        LDY     1,U
L8FEC   LDX     ,S
L8FEE   LDB     ,Y+
        SUBB    ,X+
        BEQ     L8FEE
        CMPB    #$80
        BNE     L9030
        LEAS    2,S
        PULS    U
        ORB     <$42
        LDA     <$41
        BNE     L9008
        CMPB    #$84
        BNE     L900C
        LDA     #$3A
L9008   STD     ,U++
        BRA     L8FA0

L900C   STB     ,U+
        CMPB    #$86
        BNE     L9014
        INC     <$44
L9014   CMPB    #$82
        BEQ     L8FC2
L9018   BRA     L8FA0

L901A   LDU     #$011B
        COM     <$41
        BNE     L8FE1
        PULS    X,U
        LDA     ,X+
        STA     ,U+
        JSR     >L8ADF
        BCS     L9018
        COM     <$43
        BRA     L9018

L9030   INC     <$42
        DECA
        BEQ     L8FE3
        LEAY    -1,Y
L9037   LDB     ,Y+
        BPL     L9037
        BRA     L8FEC

CmdPrint:
L903D   BEQ     TextOutCRLF L90A1
        BSR     L9044
        CLR     <TextDevN
        RTS

L9044   CMPA    #$40
        BNE     L904D
        JSR     >LB786
        BRA     L9057

L904D   CMPA    #$23
        BNE     L905E
        JSR     >LB7D7
        JSR     >LB63C
L9057   JSR     <BasChrGetCurr
        BEQ     TextOutCRLF L90A1
        JSR     >VarCKComma L89AA
L905E   CMPA    #$CD
        LBEQ    LA225
L9064   BEQ     L90AE
L9066   CMPA    #$BB
        BEQ     L90C7
        CMPA    #$2C
        BEQ     L90AF
        CMPA    #$3B
        BEQ     L90E0
        JSR     >VarGetStr L8887
        LDA     <BasVarType
        PSHS    A
        BNE     L9081
        JSR     >L9587
        JSR     >L8C59
L9081   BSR     L90E8
        PULS    B
        JSR     >LB595
        TST     <CasIOFlag
        BEQ     L9092
        BSR     TextOutCRLF L90A1
        JSR     <BasChrGetCurr
        BRA     L9064

L9092   TSTB
        BNE     L909D
        JSR     <BasChrGetCurr
        CMPA    #$2C
        BEQ     L90AF
        BSR     TextOutSpace L90F5
L909D   JSR     <BasChrGetCurr
        BNE     L9066
TextOutCRLF:
L90A1   LDA     #$0D
        BRA     L90FA

L90A5   JSR     >LB595
        BEQ     TextOutCRLF L90A1
        LDA     <TextVDUCurrCol
        BNE     TextOutCRLF L90A1
L90AE   RTS

L90AF   JSR     >LB595
        BEQ     L90BE
        LDB     <TextVDUCurrCol
        CMPB    <TextVDULastComma
        BCS     L90C0
        BSR     TextOutCRLF L90A1
        BRA     L90E0

L90BE   LDB     <TextVDUCurrCol
L90C0   SUBB    <TextVDUCommaW
        BCC     L90C0
        NEGB
        BRA     L90D7

L90C7   JSR     >L8E4F
        CMPA    #$29
        LBNE    BasSNError L89B4
        JSR     >LB595
        SUBB    <TextVDUCurrCol
        BLS     L90E0
L90D7   TST     <CasIOFlag
        BNE     L90E0
L90DB   BSR     TextOutSpace L90F5
        DECB
        BNE     L90DB
L90E0   JSR     <BasChrGet
        JMP     >L9064

TextOutString:
L90E5   JSR     >L8C5B
L90E8   JSR     >L8D9D
        INCB
L90EC   DECB
        BEQ     L90AE
        LDA     ,X+
        BSR     L90FA
        BRA     L90EC

TextOutSpace:
L90F5   LDA     #$20

        FCB     $8C

TextOutQuestion:
L90F8   LDA     #$3F
L90FA   JMP     >TextOutChar LB54A

L90FD   LDX     #D966E
        BRA     L910B

L9102   JSR     >L92DA
CmdMinus:
L9105   COM     <$54
        COM     <$62
        BRA     CmdPlus L910E

L910B   JSR     >L92DA
CmdPlus:
L910E   TSTB
        LBEQ    L93F5
        LDX     #BasVarFPAcc2 L005C
L9116   TFR     A,B
        TSTB
        BEQ     L9187
        SUBB    <BasVarFPAcc1
        BEQ     L9188
        BCS     L912B
        STA     <BasVarFPAcc1
        LDA     <$61
        STA     <$54
        LDX     #BasVarFPAcc1 L004F
        NEGB
L912B   CMPB    #$F8
        BLE     L9188
        CLRA
        LSR     1,X
        JSR     >L9203
L9135   LDB     <$62
        BPL     L9144
        COM     1,X
        COM     2,X
        COM     3,X
        COM     4,X
        COMA
        ADCA    #$00
L9144   STA     <$63
        LDA     <$53
        ADCA    <$60
        STA     <$53
        LDA     <BasVarAssign16
        ADCA    <$5F
        STA     <BasVarAssign16
        LDA     <$51
        ADCA    <$5E
        STA     <$51
        LDA     <$50
        ADCA    <$5D
        STA     <$50
        TSTB
        BPL     L91A5
L9161   BCS     VarNormFPA0 L9165
        BSR     L91C2
VarNormFPA0:
L9165   CLRB
L9166   LDA     <$50
        BNE     L9198
        LDA     <$51
        STA     <$50
        LDA     <BasVarAssign16
        STA     <$51
        LDA     <$53
        STA     <BasVarAssign16
        LDA     <$63
        STA     <$53
        CLR     <$63
        ADDB    #$08
        CMPB    #$28
        BLT     L9166
L9182   CLRA
L9183   STA     <BasVarFPAcc1
        STA     <$54
L9187   RTS

L9188   BSR     L91F7
        CLRB
        BRA     L9135

L918D   INCB
        ASL     <$63
        ROL     <$53
        ROL     <BasVarAssign16
        ROL     <$51
        ROL     <$50
L9198   BPL     L918D
        LDA     <BasVarFPAcc1
        PSHS    B
        SUBA    ,S+
        STA     <BasVarFPAcc1
        BLS     L9182
	
	FCB	Skip2
L91A5   BCS	L91AF

        ASL     <$63
        LDA     #$00
        STA     <$63
        BRA     L91BB

L91AF   INC     <BasVarFPAcc1
        BEQ     L91DB
        ROR     <$50
        ROR     <$51
        ROR     <BasVarAssign16
        ROR     <$53
L91BB   BCC     L91C1
        BSR     L91CC
        BEQ     L91AF
L91C1   RTS

L91C2   COM     <$54
L91C4   COM     <$50
        COM     <$51
        COM     <BasVarAssign16
        COM     <$53
L91CC   LDX     <BasVarAssign16
        LEAX    1,X
        STX     <BasVarAssign16
        BNE     L91DA
        LDX     <$50
        LEAX    1,X
        STX     <$50
L91DA   RTS

L91DB   LDB     #$0A
        JMP     >SysErr L8344

L91E0   LDX     #$0012
L91E3   LDA     4,X
        STA     <$63
        LDA     3,X
        STA     4,X
        LDA     2,X
        STA     3,X
        LDA     1,X
        STA     2,X
        LDA     <$5B
        STA     1,X
L91F7   ADDB    #$08
        BLE     L91E3
        LDA     <$63
        SUBB    #$08
        BEQ     L920D
L9201   ASR     1,X
L9203   ROR     2,X
        ROR     3,X
        ROR     4,X
        RORA
        INCB
        BNE     L9201
L920D   RTS

D920E   FCB     $81
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
D9213   FCB     $03
        FCB     $7F
        FCB     $5E
        FCB     $56
        FCB     $CB
        FCB     $79
        FCB     $80
        FCB     $13
        FCB     $9B
        FCB     $0B
        FCB     $64
        FCB     $80
        FCB     $76
        FCB     $38
        FCB     $93
        FCB     $16
        FCB     $82
        FCB     $38
        FCB     $AA
        FCB     $3B
        FCB     $20
D9228   FCB     $80
        FCB     $35
        FCB     $04
        FCB     $F3
        FCB     $34
D922D   FCB     $81
        FCB     $35
        FCB     $04
        FCB     $F3
        FCB     $34
D9232   FCB     $80
        FCB     $80
        FCB     $00
        FCB     $00
        FCB     $00
D9237   FCB     $80
        FCB     $31
        FCB     $72
        FCB     $17
        FCB     $F8

CmdLOG:
L923C   JSR     >L9418
        LBLE    BasFCError L8B8D
        LDX     #D9228
        LDA     <BasVarFPAcc1
        SUBA    #$80
        PSHS    A
        LDA     #$80
        STA     <BasVarFPAcc1
        JSR     >L910B
        LDX     #D922D
        JSR     >L933A
        LDX     #D920E
        JSR     >L9102
        LDX     #D9213
        JSR     >L9743
        LDX     #D9232
        JSR     >L910B
        PULS    B
        JSR     >L9547
        LDX     #D9237
L9273   BSR     L92DA

CmdMultiply:
L9275   BEQ     L92D9
        BSR     L92F3
L9279   LDA     #$00
        STA     <BasTempFPA2
        STA     <$14
        STA     <$15
        STA     <$16
        LDB     <$53
        BSR     L92A9
        LDB     <$63
        STB     <$AE
        LDB     <BasVarAssign16
        BSR     L92A9
        LDB     <$63
        STB     <$AD
        LDB     <$51
        BSR     L92A9
        LDB     <$63
        STB     <$AC
        LDB     <$50
        BSR     L92AD
        LDB     <$63
        STB     <BasRndData
        JSR     >L93B6
        JMP     >VarNormFPA0 L9165

L92A9   LBEQ    L91E0
L92AD   COMA
L92AE   LDA     <BasTempFPA2
        RORB
        BEQ     L92D9
        BCC     L92CB
        LDA     <$16
        ADDA    <$60
        STA     <$16
        LDA     <$15
        ADCA    <$5F
        STA     <$15
        LDA     <$14
        ADCA    <$5E
        STA     <$14
        LDA     <BasTempFPA2
        ADCA    <$5D
L92CB   RORA
        STA     <BasTempFPA2
        ROR     <$14
        ROR     <$15
        ROR     <$16
        ROR     <$63
        CLRA
        BRA     L92AE

L92D9   RTS

L92DA   LDD     1,X
        STA     <$61
        ORA     #$80
        STD     <$5D
        LDB     <$61
        EORB    <$54
        STB     <$62
        LDD     3,X
        STD     <$5F
        LDA     ,X
        STA     <BasVarFPAcc2
        LDB     <BasVarFPAcc1
        RTS

L92F3   TSTA
        BEQ     L930C
        ADDA    <BasVarFPAcc1
        RORA
        ROLA
        BVC     L930C
        ADDA    #$80
        STA     <BasVarFPAcc1
        BEQ     L930E
        LDA     <$62
        STA     <$54
        RTS

L9307   LDA     <$54
        COMA
        BRA     L930E

L930C   LEAS    2,S
L930E   LBPL    L9182
L9312   JMP     >L91DB

L9315   JSR     >L940A
        BEQ     L9327
        ADDA    #$02
        BCS     L9312
        CLR     <$62
        JSR     >L9116
        INC     <BasVarFPAcc1
        BEQ     L9312
L9327   RTS

D9328   FCB     $84
        FCB     $20
        FCB     $00
        FCB     $00
        FCB     $00

L932D   JSR     >L940A
        LDX     #D9328
        CLRB
L9334   STB     <$62
        JSR     >L93BF
        
	FCB	Skip2
L933A   BSR	L92DA
	
CmdDivide:
L933C   BEQ     L93B1
        NEG     <BasVarFPAcc1
        BSR     L92F3
        INC     <BasVarFPAcc1
        BEQ     L9312
        LDX     #BasTempFPA2 L0013
        LDB     #$04
        STB     <BasGenCount
        LDB     #$01
L934F   LDA     <$50
        CMPA    <$5D
        BNE     L9368
        LDA     <$51
        CMPA    <$5E
        BNE     L9368
        LDA     <BasVarAssign16
        CMPA    <$5F
        BNE     L9368
        LDA     <$53
        CMPA    <$60
        BNE     L9368
        COMA
L9368   TFR     CC,A
        ROLB
        BCC     L9377
        STB     ,X+
        DEC     <BasGenCount
        BMI     L93A7
        BEQ     L93A3
        LDB     #$01
L9377   TFR     A,CC
        BCS     L9389
L937B   ASL     <$60
        ROL     <$5F
        ROL     <$5E
        ROL     <$5D
        BCS     L9368
        BMI     L934F
        BRA     L9368

L9389   LDA     <$60
        SUBA    <$53
        STA     <$60
        LDA     <$5F
        SBCA    <BasVarAssign16
        STA     <$5F
        LDA     <$5E
        SBCA    <$51
        STA     <$5E
        LDA     <$5D
        SBCA    <$50
        STA     <$5D
        BRA     L937B

L93A3   LDB     #$40
        BRA     L9377

L93A7   RORB
        RORB
        RORB
        STB     <$63
        BSR     L93B6
        JMP     >VarNormFPA0 L9165

L93B1   LDB     #$14
        JMP     >SysErr L8344

L93B6   LDX     <BasTempFPA2
        STX     <$50
        LDX     <$15
        STX     <BasVarAssign16
        RTS

L93BF   PSHS    A
        LDD     1,X
        STA     <$54
        ORA     #$80
        STD     <$50
        CLR     <$63
        LDB     ,X
        LDX     3,X
        STX     <BasVarAssign16
        STB     <BasVarFPAcc1
        PULS    A,PC

L93D5   LDX     #BasVarFPAcc4 L0045
        BRA     L93E0

L93DA   LDX     #BasVarFPAcc3 L0040

        FCB	Skip2
L93DE   LDX	<BasTempVarDesc

L93E0   LDA     <BasVarFPAcc1
        STA     ,X
        LDA     <$54
        ORA     #$7F
        ANDA    <$50
        STA     1,X
        LDA     <$51
        STA     2,X
        LDU     <BasVarAssign16
        STU     3,X
        RTS

L93F5   LDA     <$61
L93F7   STA     <$54
        LDX     <BasVarFPAcc2
        STX     <BasVarFPAcc1
        CLR     <$63
        LDA     <$5E
        STA     <$51
        LDA     <$54
        LDX     <$5F
        STX     <BasVarAssign16
        RTS

L940A   LDD     <BasVarFPAcc1
        STD     <BasVarFPAcc2
        LDX     <$51
        STX     <$5E
        LDX     <$53
        STX     <$60
        TSTA
        RTS

L9418   LDB     <BasVarFPAcc1
        BEQ     L9424
L941C   LDB     <$54
L941E   ROLB
        LDB     #$FF
        BCS     L9424
        NEGB
L9424   RTS

CmdSGN:
L9425   BSR     L9418
L9427   STB     <$50
        CLR     <$51
        LDB     #$88
L942D   LDA     <$50
        SUBA    #$80
L9431   STB     <BasVarFPAcc1
        LDD     <Misc16BitScratch
        STD     <BasVarAssign16
        STA     <$63
        STA     <$54
        JMP     >L9161

CmdABS:
L943E   CLR     <$54
        RTS

L9441   LDB     ,X
        BEQ     L9418
        LDB     1,X
        EORB    <$54
        BMI     L941C
L944B   LDB     <BasVarFPAcc1
        CMPB    ,X
        BNE     L946E
        LDB     1,X
        ORB     #$7F
        ANDB    <$50
        CMPB    1,X
        BNE     L946E
        LDB     <$51
        CMPB    2,X
        BNE     L946E
        LDB     <BasVarAssign16
        CMPB    3,X
        BNE     L946E
        LDB     <$53
        SUBB    4,X
        BNE     L946E
        RTS

L946E   RORB
        EORB    <$54
        BRA     L941E

L9473   LDB     <BasVarFPAcc1
        BEQ     L94B4
        SUBB    #$A0
        LDA     <$54
        BPL     L9482
        COM     <$5B
        JSR     >L91C4
L9482   LDX     #BasVarFPAcc1 L004F
        CMPB    #$F8
        BGT     L948F
        JSR     >L91F7
        CLR     <$5B
        RTS

L948F   CLR     <$5B
        LDA     <$54
        ROLA
        ROR     <$50
        JMP     >L9203

CmdINT:
L9499   LDB     <BasVarFPAcc1
        CMPB    #$A0
        BCC     L94BC
        BSR     L9473
        STB     <$63
        LDA     <$54
        STB     <$54
        SUBA    #$80
        LDA     #$A0
        STA     <BasVarFPAcc1
        LDA     <$53
        STA     <BasDelim1
        JMP     >L9161

L94B4   STB     <$50
        STB     <$51
        STB     <BasVarAssign16
        STB     <$53
L94BC   RTS

L94BD   LDX     <Misc16BitScratch
        STX     <$54
        STX     <BasVarFPAcc1
        STX     <$51
        STX     <BasVarAssign16
        STX     <$47
        STX     <BasVarFPAcc4
        BCS     L9534
        CMPA    #$26
        LBEQ    L9C1B
        CMPA    #$2D
        BNE     L94DB
        COM     <$55
        BRA     L94DF

L94DB   CMPA    #$2B
        BNE     L94E3
L94DF   JSR     <BasChrGet
        BCS     L9534
L94E3   CMPA    #$2E
        BEQ     L950F
        CMPA    #$45
        BNE     L9513
        JSR     <BasChrGet
        BCS     L9553
        CMPA    #$C4
        BEQ     L9501
        CMPA    #$2D
        BEQ     L9501
        CMPA    #$C3
        BEQ     L9503
        CMPA    #$2B
        BEQ     L9503
        BRA     L9507

L9501   COM     <$48
L9503   JSR     <BasChrGet
        BCS     L9553
L9507   TST     <$48
        BEQ     L9513
        NEG     <$47
        BRA     L9513

L950F   COM     <$46
        BNE     L94DF
L9513   LDA     <$47
        SUBA    <BasVarFPAcc4
        STA     <$47
        BEQ     L952D
        BPL     L9526
L951D   JSR     >L932D
        INC     <$47
        BNE     L951D
        BRA     L952D

L9526   JSR     >L9315
        DEC     <$47
        BNE     L9526
L952D   LDA     <$55
        BPL     L94BC
        JMP     >L96DE

L9534   LDB     <BasVarFPAcc4
        SUBB    <$46
        STB     <BasVarFPAcc4
        PSHS    A
        JSR     >L9315
        PULS    B
        SUBB    #$30
        BSR     L9547
        BRA     L94DF

L9547   JSR     >L93DA
        JSR     >L9427
        LDX     #BasVarFPAcc3 L0040
        JMP     >L910B

L9553   LDB     <$47
        ASLB
        ASLB
        ADDB    <$47
        ASLB
        SUBA    #$30
        PSHS    B
        ADDA    ,S+
        STA     <$47
        BRA     L9503

D9564   FCB     $9B
        FCB     $3E
        FCB     $BC
        FCB     $1F
        FCB     $FD
D9569   FCB     $9E
        FCB     $6E
        FCB     $6B
        FCB     $27
        FCB     $FD
D956E   FCB     $9E
        FCB     $6E
        FCB     $6B
        FCB     $28
        FCB     $00

L9573   LDX     #MessIn-1
        BSR     L9584
        LDD     <BasCurrentLine
TextOutNum16:
L957A   STD     <$50
        LDB     #$90
        COMA
        JSR     >L9431
TextOutNumFPA0:
        BSR     L9587
L9584   JMP     >TextOutString L90E5

L9587   LDU     #$03DA
L958A   LDA     #$20
        LDB     <$54
        BPL     L9592
        LDA     #$2D
L9592   STA     ,U+
        STU     <$64
        STA     <$54
        LDA     #$30
        LDB     <BasVarFPAcc1
        LBEQ    L9666
        CLRA
        CMPB    #$80
        BHI     L95AD
        LDX     #D956E
        JSR     >L9273
        LDA     #$F7
L95AD   STA     <BasVarFPAcc4
L95AF   LDX     #D9569
        JSR     >L944B
        BGT     L95C6
L95B7   LDX     #D9564
        JSR     >L944B
        BGT     L95CD
        JSR     >L9315
        DEC     <BasVarFPAcc4
        BRA     L95B7

L95C6   JSR     >L932D
        INC     <BasVarFPAcc4
        BRA     L95AF

L95CD   JSR     >L90FD
        JSR     >L9473
        LDB     #$01
        LDA     <BasVarFPAcc4
        ADDA    #$0A
        BMI     L95E4
        CMPA    #$0B
        BCC     L95E4
        DECA
        TFR     A,B
        LDA     #$02
L95E4   DECA
        DECA
        STA     <$47
        STB     <BasVarFPAcc4
        BGT     L95F9
        LDU     <$64
        LDA     #$2E
        STA     ,U+
        TSTB
        BEQ     L95F9
        LDA     #$30
        STA     ,U+
L95F9   LDX     #D9673
        LDB     #$80
L95FE   LDA     <$53
        ADDA    3,X
        STA     <$53
        LDA     <BasVarAssign16
        ADCA    2,X
        STA     <BasVarAssign16
        LDA     <$51
        ADCA    1,X
        STA     <$51
        LDA     <$50
        ADCA    ,X
        STA     <$50
        INCB
        RORB
        ROLB
        BVC     L95FE
        BCC     L9620
        SUBB    #$0B
        NEGB
L9620   ADDB    #$2F
        LEAX    4,X
        TFR     B,A
        ANDA    #$7F
        STA     ,U+
        DEC     <BasVarFPAcc4
        BNE     L9632
        LDA     #$2E
        STA     ,U+
L9632   COMB
        ANDB    #$80
        CMPX    #CmdSQR L9697
        BNE     L95FE
L963A   LDA     ,-U
        CMPA    #$30
        BEQ     L963A
        CMPA    #$2E
        BNE     L9646
        LEAU    -1,U
L9646   LDA     #$2B
        LDB     <$47
        BEQ     L9668
        BPL     L9651
        LDA     #$2D
        NEGB
L9651   STA     2,U
        LDA     #$45
        STA     1,U
        LDA     #$2F
L9659   INCA
        SUBB    #$0A
        BCC     L9659
        ADDB    #$3A
        STD     3,U
        CLR     5,U
        BRA     L966A

L9666   STA     ,U
L9668   CLR     1,U
L966A   LDX     #$03DA
L966D   RTS

D966E   FCB     $80
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
D9673   FCB     $FA
        FCB     $0A
        FCB     $1F
        FCB     $00
        FCB     $00
        FCB     $98
        FCB     $96
        FCB     $80
        FCB     $FF
        FCB     $F0
        FCB     $BD
        FCB     $C0
        FCB     $00
        FCB     $01
        FCB     $86
        FCB     $A0
        FCB     $FF
        FCB     $FF
        FCB     $D8
        FCB     $F0
        FCB     $00
        FCB     $00
        FCB     $03
        FCB     $E8
        FCB     $FF
        FCB     $FF
        FCB     $FF
        FCB     $9C
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $0A
        FCB     $FF
        FCB     $FF
        FCB     $FF
        FCB     $FF

CmdSQR:
L9697   JSR     >L940A
        LDX     #D966E
        JSR     >L93BF
CmdExponet:
L96A0   BEQ     CmdEXP  L9713
        TSTA
        BNE     L96AE
        LDA     <$54
        LBMI    L93B1
        JMP     >L9183

L96AE   LDX     #BasVarFPAcc5 L004A
        JSR     >L93E0
        CLRB
        LDA     <$61
        BPL     L96C9
        JSR     >CmdINT L9499
        LDX     #BasVarFPAcc5 L004A
        LDA     <$61
        JSR     >L944B
        BNE     L96C9
        COMA
        LDB     <BasDelim1
L96C9   JSR     >L93F7
        PSHS    B
        JSR     >CmdLOG L923C
        LDX     #BasVarFPAcc5 L004A
        JSR     >L9273
        BSR     CmdEXP  L9713
        PULS    A
        RORA
        BCC     L966D
L96DE   LDA     <BasVarFPAcc1
        BEQ     L96E4
        COM     <$54
L96E4   RTS

D96E5   FCB     $81
        FCB     $38
        FCB     $AA
        FCB     $3B
        FCB     $29
D96EA   FCB     $07
        FCB     $71
        FCB     $34
        FCB     $58
        FCB     $3E
        FCB     $56
        FCB     $74
        FCB     $16
        FCB     $7E
        FCB     $B3
        FCB     $1B
        FCB     $77
        FCB     $2F
        FCB     $EE
        FCB     $E3
        FCB     $85
        FCB     $7A
        FCB     $1D
        FCB     $84
        FCB     $1C
        FCB     $2A
        FCB     $7C
        FCB     $63
        FCB     $59
        FCB     $58
        FCB     $0A
        FCB     $7E
        FCB     $75
        FCB     $FD
        FCB     $E7
        FCB     $C6
        FCB     $80
        FCB     $31
        FCB     $72
        FCB     $18
        FCB     $10
        FCB     $81
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00

CmdEXP:
L9713   LDX     #D96E5
        BSR     L974F
        JSR     >L93DA
        LDA     <BasVarFPAcc1
        CMPA    #$88
        BCS     L9724
L9721   JMP     >L9307

L9724   JSR     >CmdINT L9499
        LDA     <BasDelim1
        ADDA    #$81
        BEQ     L9721
        DECA
        PSHS    A
        LDX     #BasVarFPAcc3 L0040
        JSR     >L9102
        LDX     #D96EA
        BSR     L9752
        CLR     <$62
        PULS    A
        JSR     >L92F3
        RTS

L9743   STX     <$64
        JSR     >L93DA
        BSR     L974F
        BSR     L9754
        LDX     #BasVarFPAcc3 L0040
L974F   JMP     >L9273

L9752   STX     <$64
L9754   JSR     >L93D5
        LDX     <$64
        LDB     ,X+
        STB     <$55
        STX     <$64
L975F   BSR     L974F
        LDX     <$64
        LEAX    5,X
        STX     <$64
        JSR     >L910B
        LDX     #BasVarFPAcc4 L0045
        DEC     <$55
        BNE     L975F
        RTS

CmdRND:
L9772   JSR     >L9418
        BMI     L9798
        BEQ     BasRandom8 L978E
        BSR     L978B
        JSR     >L93DA
        BSR     BasRandom8 L978E
        LDX     #BasVarFPAcc3 L0040
        BSR     L974F
        LDX     #D920E
        JSR     >L910B
L978B   JMP     >CmdINT L9499

BasRandom8:
L978E   LDX     BasRandomSeed+1		; $0116
        STX     <$50
        LDX     BasRandomSeed++3	; $0118
        STX     <BasVarAssign16
L9798   LDX     >D97C7
        STX     <$5D
        LDX     >D97C9
        STX     <$5F
        JSR     >L9279
        LDD     <$AD
        ADDD    #$658B
        STD     BasRandomSeed+3		; $0118
        STD     <BasVarAssign16
        LDD     <BasRndData
        ADCB    #$B0
        ADCA    #$05
        STD     BasRandomSeed+1		; $0116
        STD     <$50
        CLR     <$54
        LDA     #$80
        STA     <BasVarFPAcc1
        LDA     <$15
        STA     <$63
        JMP     >VarNormFPA0 L9165

D97C7   FCB     $40
        FCB     $E6
D97C9   FCB     $4D
        FCB     $AB

CmdCOS:
L97CB   LDX     #D983F
        JSR     >L910B
CmdSIN:
L97D1   JSR     >L940A
        LDX     #D9844
        LDB     <$61
        JSR     >L9334
        JSR     >L940A
        BSR     L978B
        CLR     <$62
        LDA     <BasVarFPAcc2
        LDB     <BasVarFPAcc1
        JSR     >CmdMinus L9105
        LDX     #D9849
        JSR     >L9102
        LDA     <$54
        PSHS    A
        BPL     L97FF
        JSR     >L90FD
        LDA     <$54
        BMI     L9802
        COM     <BasRelateFlag
L97FF   JSR     >L96DE
L9802   LDX     #D9849
        JSR     >L910B
        PULS    A
        TSTA
        BPL     L9810
        JSR     >L96DE
L9810   LDX     #D984E
L9813   JMP     >L9743

CmdTAN:
L9816   JSR     >L93DA
        CLR     <BasRelateFlag
        BSR     CmdSIN  L97D1
        LDX     #BasVarFPAcc5 L004A
        JSR     >L93E0
        LDX     #BasVarFPAcc3 L0040
        JSR     >L93BF
        CLR     <$54
        LDA     <BasRelateFlag
        BSR     L983B
        TST     <BasVarFPAcc1
        LBEQ    L91DB
        LDX     #BasVarFPAcc5 L004A
L9838   JMP     >L933A

L983B   PSHS    A
        BRA     L97FF

D983F   FCB     $81
        FCB     $49
        FCB     $0F
        FCB     $DA
        FCB     $A2
D9844   FCB     $83
        FCB     $49
        FCB     $0F
        FCB     $DA
        FCB     $A2
D9849   FCB     $7F
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
D984E   FCB     $05
        FCB     $84
        FCB     $E6
        FCB     $1A
        FCB     $2D
        FCB     $1B
        FCB     $86
        FCB     $28
        FCB     $07
        FCB     $FB
        FCB     $F8
        FCB     $87
        FCB     $99
        FCB     $68
        FCB     $89
        FCB     $01
        FCB     $87
        FCB     $23
        FCB     $35
        FCB     $DF
        FCB     $E1
        FCB     $86
        FCB     $A5
        FCB     $5D
        FCB     $E7
        FCB     $28
        FCB     $83
        FCB     $49
        FCB     $0F
        FCB     $DA
        FCB     $A2
        FCB     $A1
        FCB     $54
        FCB     $46
        FCB     $8F
        FCB     $13
        FCB     $8F
        FCB     $52
        FCB     $43
        FCB     $89
        FCB     $CD

CmdATN:
L9877   LDA     <$54
        PSHS    A
        BPL     L987F
        BSR     L98A2
L987F   LDA     <BasVarFPAcc1
        PSHS    A
        CMPA    #$81
        BCS     L988C
        LDX     #D920E
        BSR     L9838
L988C   LDX     #D98A6
        BSR     L9813
        PULS    A
        CMPA    #$81
        BCS     L989D
        LDX     #D983F
        JSR     >L9102
L989D   PULS    A
        TSTA
        BPL     L98A5
L98A2   JMP     >L96DE

L98A5   RTS

D98A6   FCB     $0B
        FCB     $76
        FCB     $B3
        FCB     $83
        FCB     $BD
        FCB     $D3
        FCB     $79
        FCB     $1E
        FCB     $F4
        FCB     $A6
        FCB     $F5
        FCB     $7B
        FCB     $83
        FCB     $FC
        FCB     $B0
        FCB     $10
        FCB     $7C
        FCB     $0C
        FCB     $1F
        FCB     $67
        FCB     $CA
        FCB     $7C
        FCB     $DE
        FCB     $53
        FCB     $CB
        FCB     $C1
        FCB     $7D
        FCB     $14
        FCC     /dpL}/
        FCB     $B7
        FCB     $EA
        FCC     /Qz}c0/
        FCB     $88
        FCB     $7E
        FCB     $7E
        FCB     $92
        FCB     $44
        FCB     $99
        FCC     /:~L/
        FCB     $CC
        FCB     $91
        FCB     $C7
        FCB     $7F
        FCB     $AA
        FCB     $AA
        FCB     $AA
        FCB     $13
        FCB     $81
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00

L98E3   
	ifne	dragon64
	LDD	#$B844
	else
	LDD     #$BA42
	endc
        STD     <SndVolume
        LDA     #$02
        STA     <SndTempo
        STA     <SndOctave
        ASLA
        STA     <SndNoteLen
        CLR     <$E5
        LDD     <Misc16BitScratch
        STD     <$E8
        LDB     #$80
        STD     <GrCurrXCo
        LDB     #$60
        STD     <GrCurrYCo
L98FF   RTS

L9900   JSR     <BasChrGet
        JSR     >LB7AA
        BSR     L994A
        STX     CasLoadAddr L01E7
        BSR     L994A
        CMPX    2,S
        LBCS    BasFCError L8B8D
        BSR     L994A
        STX     CasEntryAddr L01E5
        JSR     <BasChrGetCurr
        BNE     L98FF
CasWriteBin:
L991B   LDA     #$02
        LDX     <Misc16BitScratch
        JSR     >LB891
        CLR     <CasStatus
        INC     <CasBlockType
        JSR     >CasWriteLeader L801B
        LDX     4,S
L992B   STX     <CasIOBuffAddr
        LDA     #$FF
        STA     <CasBlockLen
        LDD     2,S
        SUBD    <CasIOBuffAddr
        BCC     L993C
        LEAS    6,S
        JMP     >LB6CD

L993C   CMPD    #$00FF
        BCC     L9945
        INCB
        STB     <CasBlockLen
L9945   JSR     >CasBlockOut LB999
        BRA     L992B

L994A   JSR     >VarCKComma L89AA
        JSR     >VarGet16Bit L8E83
        LDU     ,S
        STX     ,S
        TFR     U,PC

CmdFIX:
L9956   JSR     >L9418
        BMI     L995E
L995B   JMP     >CmdINT L9499

L995E   COM     <$54
        BSR     L995B
        JMP     >L96DE

CmdEdit:
L9965   JSR     >L9D9F
        LEAS    2,S
L996A   LDA     #$01
        STA     <$D8
        JSR     >BasFindLineNo L83FF
        LBCS    L8605
        JSR     >L8F08
        TFR     Y,D
        SUBD    #$02DE
        STB     <BasEditorLineLen
L997F   LDD     <$2B
        JSR     >TextOutNum16 L957A
        JSR     >TextOutSpace L90F5
        LDX     #$02DD
        LDB     <$D8
        BNE     L99B3
L998E   CLRB
L998F   JSR     >L9AB9
        JSR     >LA438
        BCS     L99A2
        SUBA    #$30
        PSHS    A
        LDA     #$0A
        MUL
        ADDB    ,S+
        BRA     L998F

L99A2   SUBB    #$01
        ADCB    #$01
        CMPA    #$41
        BNE     L99AF
        JSR     >TextOutCRLF L90A1
        BRA     L996A

L99AF   CMPA    #$4C
        BNE     L99BE
L99B3   BSR     L99E6
        CLR     <$D8
        JSR     >TextOutCRLF L90A1
        BRA     L997F

L99BC   LEAS    2,S
L99BE   CMPA    #$0D
        BNE     L99CF
        BSR     L99E6
L99C4   JSR     >TextOutCRLF L90A1
        LDX     #$02DD
        STX     <BasAddrSigByte
        JMP     >L83A6

L99CF   CMPA    #$45
        BEQ     L99C4
        CMPA    #$51
        BNE     L99DD
        JSR     >TextOutCRLF L90A1
        JMP     >BasCmdMode L8371

L99DD   BSR     L99E1
        BRA     L998E

L99E1   CMPA    #$20
        BNE     L99F5

        FCB     $8C

L99E6   LDB     #$F9
L99E8   LDA     ,X
        BEQ     L99F4
        JSR     >TextOutChar LB54A
        LEAX    1,X
        DECB
        BNE     L99E8
L99F4   RTS

L99F5   CMPA    #$44
        BNE     L9A41
L99F9   TST     ,X
        BEQ     L99F4
        BSR     L9A03
        DECB
        BNE     L99F9
        RTS

L9A03   DEC     <BasEditorLineLen
        LEAY    -1,X
L9A07   LEAY    1,Y
        LDA     1,Y
        STA     ,Y
        BNE     L9A07
        RTS

L9A10   CMPA    #$49
        BEQ     L9A27
        CMPA    #$58
        BEQ     L9A25
        CMPA    #$48
        BNE     L9A78
        CLR     ,X
        TFR     X,D
        SUBD    #$02DE
        STB     <BasEditorLineLen
L9A25   BSR     L99E6
L9A27   JSR     >L9AB9
        CMPA    #$0D
        BEQ     L99BC
        CMPA    #$1B
        BEQ     L9A57
        CMPA    #$08
        BNE     L9A58
        CMPX    #$02DD
        BEQ     L9A27
        BSR     L9A82
        BSR     L9A03
        BRA     L9A27

L9A41   CMPA    #$43
        BNE     L9A10
L9A45   TST     ,X
        BEQ     L9A57
        JSR     >L9AB9
        BCS     L9A50
        BRA     L9A45

L9A50   STA     ,X+
        BSR     L9A8B
        DECB
        BNE     L9A45
L9A57   RTS

L9A58   LDB     <BasEditorLineLen
        CMPB    #$F9
        BNE     L9A60
        BRA     L9A27

L9A60   PSHS    X
L9A62   TST     ,X+
        BNE     L9A62
L9A66   LDB     ,-X
        STB     1,X
        CMPX    ,S
        BNE     L9A66
        LEAS    2,S
        STA     ,X+
        BSR     L9A8B
        INC     <BasEditorLineLen
        BRA     L9A27

L9A78   CMPA    #$08
        BNE     L9A8E
L9A7C   BSR     L9A82
        DECB
        BNE     L9A7C
        RTS

L9A82   CMPX    #$02DD
        BEQ     L9A57
        LEAX    -1,X
        LDA     #$08
L9A8B   JMP     >TextOutChar LB54A

L9A8E   CMPA    #$4B
        BEQ     L9A97
        SUBA    #$53
        BEQ     L9A97
        RTS

L9A97   PSHS    A
        BSR     L9AB9
        PSHS    A
L9A9D   LDA     ,X
        BEQ     L9AB7
        TST     1,S
        BNE     L9AAB
        BSR     L9A8B
        LEAX    1,X
        BRA     L9AAE

L9AAB   JSR     >L9A03
L9AAE   LDA     ,X
        CMPA    ,S
        BNE     L9A9D
        DECB
        BNE     L9A9D
L9AB7   PULS    Y,PC

L9AB9   JSR     >TextWaitKeyCurs2 LB505
        CMPA    #$7F
        BCC     L9AB9
        CMPA    #$5F
        BNE     L9AC6
        LDA     #$1B
L9AC6   CMPA    #$0D
        BEQ     L9AD8
        CMPA    #$1B
        BEQ     L9AD8
        CMPA    #$08
        BEQ     L9AD8
        CMPA    #$20
        BCS     L9AB9
        ORCC    #$01
L9AD8   RTS

CmdTron:
L9AD9   FCB     $86

CmdTroff:
L9ADA   CLRA
        STA     <BasTronFlag
        RTS

CmdPOS:
L9ADE   LDA     <TextDevN
        PSHS    A
        JSR     >LB7E0
        JSR     >LB63C
        JSR     >LB595
        LDB     <TextVDUCurrCol
        PULS    A
        STA     <TextDevN
        JMP     >VarAssign8Bit L8C36

CmdVarptr:
L9AF4   JSR     >VarCKOpBrac L89A7
        LDD     <BasVarEnd
        PSHS    D
        JSR     >VarGetVar L8A94
        JSR     >VarCKClBrac L89A4
        PULS    D
        EXG     X,D
        CMPX    <BasVarEnd
        BNE     L9B5A
	
	ifne	dragon64
	JMP     >L9FD0
	else
        JMP     >VarAssign16Bit2 L8C37
	endc
	
L9B0C   JSR     <BasChrGet
        JSR     >VarCKOpBrac L89A7
        JSR     >VarGetVar L8A94
        PSHS    X
        LDD     2,X
        CMPD    <AddrStack
        BLS     L9B21
        SUBD    <AddrFWareRamTop
        BLS     L9B33
L9B21   LDB     ,X
        JSR     >BasResStr2 L8CB3
        PSHS    X
        LDX     2,S
        JSR     >L8D89
        PULS    X,U
        STX     2,U
        PSHS    U
L9B33   JSR     >VarGetComma8 L8E7E
        PSHS    B
        TSTB
        BEQ     L9B5A
        LDB     #$FF
        CMPA    #$29
        BEQ     L9B44
        JSR     >VarGetComma8 L8E7E
L9B44   PSHS    B
        JSR     >VarCKClBrac L89A4
        LDB     #$CB
        JSR     >VarCKChar L89AC
        BSR     L9B7E
        TFR     X,U
        LDX     2,S
        LDA     ,X
        SUBA    1,S
        BCC     L9B5D
L9B5A   JMP     >BasFCError L8B8D

L9B5D   INCA
        CMPA    ,S
        BCC     L9B64
        STA     ,S
L9B64   LDA     1,S
        EXG     A,B
        LDX     2,X
        DECB
        ABX
        TSTA
        BEQ     L9B7C
        CMPA    ,S
        BLS     L9B75
        LDA     ,S
L9B75   TFR     A,B
        EXG     U,X
        JSR     >UtilCopyBXtoU LB7CC
L9B7C   PULS    D,X,PC

L9B7E   JSR     >VarGetStr L8887
        JMP     >BasGetStrLenAddr L8D9A

CmdStringS:
L9B84   JSR     >VarCKOpBrac L89A7
        JSR     >VarGet8Bit L8E51
        PSHS    B
        JSR     >VarCKComma L89AA
        JSR     >VarGetStr L8887
        JSR     >VarCKClBrac L89A4
        LDA     <BasVarType
        BNE     L9B9E
        JSR     >L8E54
        BRA     L9BA1

L9B9E   JSR     >BasGetStrFirst L8DEA
L9BA1   PSHS    B
        LDB     1,S
        JSR     >BasResStr L8C52
        PULS    D
        BEQ     L9BB1
L9BAC   STA     ,X+
        DECB
        BNE     L9BAC
L9BB1   JMP     >L8DE1

CmdInstr:
L9BB4   JSR     >VarCKOpBrac L89A7
        JSR     >VarGetStr L8887
        LDB     #$01
        PSHS    B
        LDA     <BasVarType
        BNE     L9BD2
        JSR     >L8E54
        STB     ,S
        BEQ     L9B5A
        JSR     >VarCKComma L89AA
        JSR     >VarGetStr L8887
        JSR     >VarGetExpr L8877
L9BD2   LDX     <BasVarAssign16
        PSHS    X
        JSR     >VarCKComma L89AA
        JSR     >L9B7E
        PSHS    B,X
        JSR     >VarCKClBrac L89A4
        LDX     3,S
        JSR     >VarDelVar L8D9F
        PSHS    B
        CMPB    6,S
        BCS     L9C0F
        LDA     1,S
        BEQ     L9C0C
        LDB     6,S
        DECB
        ABX
L9BF4   LEAY    ,X
        LDU     2,S
        LDB     1,S
        LDA     ,S
        SUBA    6,S
        INCA
        CMPA    1,S
        BCS     L9C0F
L9C03   LDA     ,X+
        CMPA    ,U+
        BNE     L9C15
        DECB
        BNE     L9C03
L9C0C   LDB     6,S
        
	FCB	Skip1
L9C0F   CLRB
        LEAS    7,S
        JMP     >VarAssign8Bit L8C36

L9C15   INC     6,S
        LEAX    1,Y
        BRA     L9BF4

L9C1B   CLR     <BasVarAssign16
        CLR     <$53
        LDX     #BasVarAssign16 L0052
        JSR     <BasChrGet
        CMPA    #$4F
        BEQ     L9C3A
        CMPA    #$48
        BEQ     L9C4F
        JSR     <BasChrGetCurr
        BRA     L9C3C

L9C30   CMPA    #$38
        LBHI    BasSNError L89B4
        LDB     #$03
        BSR     L9C64
L9C3A   JSR     <BasChrGet
L9C3C   BCS     L9C30
VarAssign16BitB:
L9C3E   CLR     <$50
        CLR     <$51
        CLR     <BasVarType
        CLR     <$63
        CLR     <$54
        LDB     #$A0
        STB     <BasVarFPAcc1
        JMP     >VarNormFPA0 L9165

L9C4F   JSR     <BasChrGet
        BCS     L9C5E
        JSR     >L8ADF
        BCS     VarAssign16BitB L9C3E
        CMPA    #$47
        BCC     VarAssign16BitB L9C3E
        SUBA    #$07
L9C5E   LDB     #$04
        BSR     L9C64
        BRA     L9C4F

L9C64   ASL     1,X
        ROL     ,X
        LBCS    L91DB
        DECB
        BNE     L9C64
D9C6F   SUBA    #$30
        ADDA    1,X
        STA     1,X
L9C75   RTS

BasChkDirect:
L9C76   LDX     <BasCurrentLine
        LEAX    1,X
        BNE     L9C75
        LDB     #$16
L9C7E   JMP     >SysErr L8344

CmdDef:
L9C81   LDX     [BasAddrSigByte] L00A6
        CMPX    #$FFA1
        LBEQ    L9D00
        BSR     L9CB1
        BSR     BasChkDirect L9C76
        JSR     >VarCKOpBrac L89A7
        LDB     #$80
        STB     <BasDisArraySearch
        JSR     >VarGetVar L8A94
        BSR     L9CC1
        JSR     >VarCKClBrac L89A4
        LDB     #$CB
        JSR     >VarCKChar L89AC
        LDX     <$4B
        LDD     <BasAddrSigByte
        STD     ,X
        LDD     <BasVarPtrLast
        STD     2,X
        JMP     >CmdData L8613

L9CB1   LDB     #$BE
        JSR     >VarCKChar L89AC
        LDB     #$80
        STB     <BasDisArraySearch
        ORA     #$80
        JSR     >L8A99
        STX     <$4B
L9CC1   JMP     >VarGetExprCC L8874

L9CC4   BSR     L9CB1
        PSHS    X
        JSR     >L899F
        BSR     L9CC1
        PULS    U
        LDB     #$22
        LDX     2,U
        BEQ     L9C7E
        LDY     <BasAddrSigByte
        LDU     ,U
        STU     <BasAddrSigByte
        LDA     4,X
        PSHS    A
        LDD     ,X
        LDU     2,X
        PSHS    D,X,Y,U
        JSR     >L93E0
        JSR     >L8872
        PULS    D,X,Y,U
        STD     ,X
        STU     2,X
        PULS    A
        STA     4,X
        JSR     <BasChrGetCurr
        LBNE    BasSNError L89B4
        STY     <BasAddrSigByte
        RTS

L9D00   JSR     <BasChrGet
	
	ifne	dragon64
	JSR     <BasChrGet
	endc
	
        BSR     L9D0D
        PSHS    X
        BSR     L9D35
	
	ifne	dragon64
	STX     [,S++]
        else
	PULS    U
        STX     ,U
	endc
	
        RTS

L9D0D   CLRB
	
	ifne	dragon64
	JSR	<BasChrGetCurr
	else
        JSR     <BasChrGet
	endc
	
        BCC     L9D18
        SUBA    #$30
        TFR     A,B
        JSR     <BasChrGet
L9D18   LDX     <BasUSRTableAddr
        ASLB
        ABX
        RTS

CmdUSR:
L9D1D   BSR     L9D0D
        LDX     ,X
        PSHS    X
        JSR     >L899F
        LDX     #BasVarFPAcc1 L004F
        LDA     <BasVarType
        BEQ     L9D34
        JSR     >L8D9D
        LDX     <BasVarAssign16
        LDA     <BasVarType
L9D34   RTS

L9D35   LDB     #$CB
        JSR     >VarCKChar L89AC
        JMP     >VarGet16Bit L8E83

BasIRQVec:
L9D3D   LDA     $FF03
        BMI     L9D43
        RTI
L9D43   LDA     $FF02
        LDX     SysTimeVal L0112
        LEAX    1,X
        STX     SysTimeVal L0112
        JMP     >LAFD9

L9D51   JSR     <BasChrGet
        BSR     L9D35
        STX     SysTimeVal L0112
        RTS

CmdTimer:
L9D59   LDX     SysTimeVal L0112
        STX     <BasVarAssign16
        JMP     >VarAssign16BitB L9C3E

CmdDelete:
L9D61   LBEQ    BasFCError L8B8D
        JSR     >BasGetLineNo L869A
        JSR     >BasFindLineNo L83FF
        STX     <BasCloadMOffs
        JSR     <BasChrGetCurr
        BEQ     L9D81
        CMPA    #$C4
        BNE     L9DB0
        JSR     <BasChrGet
        BEQ     L9D7D
        BSR     L9D9F
        BRA     L9D81

L9D7D   LDA     #$FF
        STA     <$2B
L9D81   LDU     <BasCloadMOffs

	FCB	Skip2
L9D84   LDU	,U
        LDD     ,U
        BEQ     L9D90
        LDD     2,U
        SUBD    <$2B
        BLS     L9D84
L9D90   LDX     <BasCloadMOffs
        BSR     L9DA9
        JSR     >BasVect1 L841F
        LDX     <BasCloadMOffs
        JSR     >L83EF
        JMP     >BasCmdMode L8371

L9D9F   JSR     >BasGetLineNo L869A
        JMP     >LB7F9

L9DA5   LDA     ,U+
        STA     ,X+
L9DA9   CMPU    <BasVarSimpleAddr
        BNE     L9DA5
        STX     <BasVarSimpleAddr
L9DB0   RTS

CmdLineInput:
L9DB1   JSR     >BasChkDirect L9C76
        JSR     <BasChrGet
        CMPA    #$23
        BNE     L9DC3
        JSR     >LB7D7
        JSR     >LB623
        JSR     >VarCKComma L89AA
L9DC3   CMPA    #$22
        BNE     L9DD2
        JSR     >L8975
        LDB     #$3B
        JSR     >VarCKChar L89AC
        JSR     >L90E8
L9DD2   LEAS    -2,S
        JSR     >L8766
        LEAS    2,S
BasLineInputEntry:
        CLR     <TextDevN
        JSR     >VarGetVar L8A94
        STX     <BasTempVarDesc
        JSR     >VarGetExpr L8877
        LDX     #BasLinInpBuff L02DC
        CLRA
        JSR     >L8C5D
        JMP     >L86D7

L9DED   JSR     >BasGetLineNo L869A
        LDX     <$2B
        RTS

L9DF3   LDX     <BasRenumStart
L9DF5   STX     <$2B
        JMP     >BasFindLineNo L83FF

CmdRenum:
L9DFA   JSR     >L8424
        LDD     #$000A
        STD     <BasRenumStartLine
        STD     <BasRenumVal
        CLRB
        STD     <BasRenumStart
        JSR     <BasChrGetCurr
        BCC     L9E11
        BSR     L9DED
        STX     <BasRenumStartLine
        JSR     <BasChrGetCurr
L9E11   BEQ     L9E2E
        JSR     >VarCKComma L89AA
        BCC     L9E1E
        BSR     L9DED
        STX     <BasRenumStart
        JSR     <BasChrGetCurr
L9E1E   BEQ     L9E2E
        JSR     >VarCKComma L89AA
        BCC     L9E2B
        BSR     L9DED
        STX     <BasRenumVal
        BEQ     L9E74
L9E2B   JSR     >LB7F9
L9E2E   BSR     L9DF3
        STX     <BasCloadMOffs
        LDX     <BasRenumStartLine
        BSR     L9DF5
        CMPX    <BasCloadMOffs
        BCS     L9E74
        BSR     L9E58
        JSR     >L9ECE
        JSR     >BasVect2 L83ED
        BSR     L9DF3
        STX     <BasCloadMOffs
        BSR     L9E82
        BSR     L9E59
        BSR     L9E82
        JSR     >L9F6C
        JSR     >L8424
        JSR     >BasVect2 L83ED
        JMP     >BasCmdMode L8371

L9E58   FCB     $86

L9E59   CLRA
        STA     <$D8
        LDX     <BasCloadMOffs
        LDD     <BasRenumStartLine
        BSR     L9E77
L9E62   TST     <$D8
        BNE     L9E68
        STD     2,X
L9E68   LDX     ,X
        BSR     L9E77
        ADDD    <BasRenumVal
        BCS     L9E74
        CMPA    #$FA
        BCS     L9E62
L9E74   JMP     >BasFCError L8B8D

L9E77   PSHS    D
        LDD     ,X
        PULS    D
        BNE     L9E81
        LEAS    2,S
L9E81   RTS

L9E82   LDX     <BasStartProg
        LEAX    -1,X
L9E86   LEAX    1,X
        BSR     L9E77
L9E8A   LEAX    3,X
L9E8C   LEAX    1,X
        LDA     ,X
        BEQ     L9E86
        STX     <BasTempPtr
        DECA
        BEQ     L9EA3
        DECA
        BEQ     L9EC4
        DECA
        BNE     L9E8C
L9E9D   LDA     #$03
        STA     ,X+
        BRA     L9E8A

L9EA3   LDD     1,X
        DEC     2,X
        BEQ     L9EAA
        CLRA
L9EAA   LDB     3,X
        DEC     4,X
        BEQ     L9EB1
        CLRB
L9EB1   STD     1,X
        STD     <$2B
        JSR     >BasFindLineNo L83FF
L9EB8   LDX     <BasTempPtr
        BCS     L9E9D
        LDD     <$47
        INC     ,X+
        STD     ,X
        BRA     L9E8A

L9EC4   CLR     ,X
        LDX     1,X
        LDX     2,X
        STX     <$47
        BRA     L9EB8

L9ECE   LDX     <BasStartProg
        BRA     L9ED6

L9ED2   LDX     <BasAddrSigByte
        LEAX    1,X
L9ED6   BSR     L9E77
        LEAX    2,X
L9EDA   LEAX    1,X
L9EDC   STX     <BasAddrSigByte
L9EDE   JSR     <BasChrGet
L9EE0   TSTA
        BEQ     L9ED2
        BPL     L9EDE
        LDX     <BasAddrSigByte
        CMPA    #$FF
        BEQ     L9EDA
        JSR     VectAccessScreen L01A0
        CMPA    #$BF
        BEQ     L9F04
        CMPA    #$84
        BEQ     L9F04
        CMPA    #$81
        BNE     L9EDE
        JSR     <BasChrGet
        CMPA    #$BC
        BEQ     L9F04
        CMPA    #$BD
        BNE     L9EDC
L9F04   JSR     <BasChrGet
        BCS     L9F0C
L9F08   JSR     <BasChrGetCurr
        BRA     L9EE0

L9F0C   LDX     <BasAddrSigByte
        PSHS    X
        JSR     >BasGetLineNo L869A
        LDX     <BasAddrSigByte
L9F15   LDA     ,-X
        JSR     >LA438
        BCS     L9F15
        LEAX    1,X
        TFR     X,D
        SUBB    1,S
        SUBB    #$05
        BEQ     L9F46
        BCS     L9F32
        LEAU    ,X
        NEGB
        LEAX    B,X
        JSR     >L9DA9
        BRA     L9F46

L9F32   STX     <$47
        LDX     <BasVarSimpleAddr
        STX     <$43
        NEGB
        LEAX    B,X
        STX     <$41
        STX     <BasVarSimpleAddr
        JSR     >BasChkArrSpaceMv L831C
        LDX     <BasVarFPAcc4
        STX     <BasAddrSigByte
L9F46   PULS    X
        LDA     #$01
        STA     ,X
        STA     2,X
        STA     4,X
        LDB     <$2B
        BNE     L9F58
        LDB     #$01
        INC     2,X
L9F58   STB     1,X
        LDB     <$2C
        BNE     L9F62
        LDB     #$01
        INC     4,X
L9F62   STB     3,X
        JSR     <BasChrGetCurr
        CMPA    #$2C
        BEQ     L9F04
        BRA     L9F08

L9F6C   LDX     <BasStartProg
        LEAX    -1,X
L9F70   LEAX    1,X
        LDD     2,X
        STD     <BasCurrentLine
        JSR     >L9E77
        LEAX    3,X
L9F7B   LEAX    1,X
L9F7D   LDA     ,X
        BEQ     L9F70
        DECA
        BEQ     L9F9F
        SUBA    #$02
        BNE     L9F7B
        PSHS    X
        LDX     #ULMess-1
        JSR     >TextOutString L90E5
        LDX     ,S
        LDD     1,X
        JSR     >TextOutNum16 L957A
        JSR     >L9573
        JSR     >TextOutCRLF L90A1
        PULS    X
L9F9F   PSHS    X
        LDD     1,X
        STD     <BasVarAssign16
        JSR     >VarAssign16BitB L9C3E
        JSR     >L9587
        PULS    U
        LDB     #$05
L9FAF   LEAX    1,X
        LDA     ,X
        BEQ     L9FBA
        DECB
        STA     ,U+
        BRA     L9FAF

L9FBA   LEAX    ,U
        TSTB
        BEQ     L9F7D
        LEAY    ,U
        LEAU    B,U
        JSR     >L9DA9
        LEAX    ,Y
        BRA     L9F7D

ULMess
        FCC     /UL /
        FCB     $00

	ifne	dragon64
;
; Fixup for MEM command to allow for mem > 32K
; assigns an unsigned 16bit no	

L9FCE   SUBD    <BasVarEnd
L9FD0   CLR     <BasVarType
        STD     <BasVarAssign16
        LDB     #$90
        JMP     >VarAssign16BitB L9C3E
	else	
	FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
	endc

        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
IndKeyInput:
        FDB     BasicKbdIn
IndCharOutput:
        FDB     TextOutChar
IndCasOnRead:
        FDB     BasicCassOnRd
IndCasBlockIn:
        FDB     CasBlockIn
IndCasBlockOut:
        FDB     CasBlockOut
IndJoystickIn:
        FDB     BasicJoyIn
IndCasWriteLead:
        FDB     CasWriteLeader

CmdHexS:
LA00E   JSR     >L8E86
        LDX     #$03D9
        LDB     #$04
LA016   PSHS    B
        CLRB
        LDA     #$04
LA01B   ASL     <$53
        ROL     <BasVarAssign16
        ROLB
        DECA
        BNE     LA01B
        TSTB
        BNE     LA030
        LDA     ,S
        DECA
        BEQ     LA030
        CMPX    #$03D9
        BEQ     LA03C
LA030   ADDB    #$30
        CMPB    #$39
        BLS     LA038
        ADDB    #$07
LA038   STB     ,X+
        CLR     ,X
LA03C   PULS    B
        DECB
        BNE     LA016
        LEAS    2,S
        LDX     #$03D8
        JMP     >L8C5B

	ifne	resettest
NewSysReset
	JMP	>SysReset
	endc

CmdDload:
LA049   JSR     >CasClosFiles LB65F
        CLR     ,-S
        JSR     <BasChrGetCurr
        CMPA    #$4D
        BNE     LA058
        STA     ,S
        JSR     <BasChrGet
LA058   JSR     >LB7AA
        JSR     <BasChrGetCurr
        BEQ     LA070
        JSR     >VarCKComma L89AA
        CMPA    #$2C
        BEQ     LA070
        JSR     >VarGet8Bit L8E51
        JSR     >BasicSetBaud L8030
        LBCS    BasFCError L8B8D
LA070   JSR     >LA0F4
        PSHS    A
        LDA     #$FD
        STA     <TextDevN
        PULS    A
        TST     ,S+
        BNE     LA0A9
        JSR     >LB7F9
        TSTB
        BEQ     LA08B
        JSR     >BasNew L8417
        JMP     >L837A

LA08B   JMP     >BasFMError LB848

LA08E   CLR     <CasStatus
        JSR     <BasChrGet
        JSR     >LB7AA
        JSR     >LB87A
        TST     CasGapFlag L01E4
        LBEQ    LB73C
        LDU     CasFType L01E2
        DEC     <TextDevN
        JSR     >LB867
        TFR     U,D
LA0A9   SUBD    #$0200
        BNE     LA08B
        LDX     <Misc16BitScratch
        JSR     <BasChrGetCurr
        BEQ     LA0BA
        JSR     >VarCKComma L89AA
        JSR     >VarGet16Bit L8E83
LA0BA   STX     <BasCloadMOffs
        JSR     >LB7F9
LA0BF   BSR     TextWaitKeyCurs LA0EA
        PSHS    A
        BSR     LA0E3
        TFR     D,Y
        BSR     LA0E3
        ADDD    <BasCloadMOffs
        STD     <BasExecAddr
        TFR     D,X
        LDA     ,S+
        LBNE    LB663
LA0D5   BSR     TextWaitKeyCurs LA0EA
        STA     ,X
        CMPA    ,X+
        BNE     LA0F1
        LEAY    -1,Y
        BNE     LA0D5
        BRA     LA0BF

LA0E3   BSR     LA0E5
LA0E5   BSR     TextWaitKeyCurs LA0EA
        EXG     A,B
LA0E9   RTS

TextWaitKeyCurs:
LA0EA   JSR     >LB50A
        TST     <CasEOFFlag
        BEQ     LA0E9
LA0F1   JMP     >LB84B

LA0F4   BSR     LA115
        PSHS    D
        INCA
        BEQ     LA101
        LDU     <Misc16BitScratch
        BSR     LA108
        PULS    D,PC

LA101   LDB     #$34
        JMP     >SysErr L8344

LA106   LDU     <CasIOBuffAddr
LA108   LEAX    1,U
        STX     <CasIOBuffAddr
        LDX     #CasIOBuff L01DA
        JSR     >LA17E
        JMP     >LB876

LA115   CLRA
        PSHS    D,X
        LEAY    ,S
        BRA     LA11E

LA11C   BSR     LA149
LA11E   LDA     #$8A
        BSR     LA159
        BNE     LA11C
        LDX     #CasFName L01D2
LA127   LDA     ,X+
        JSR     >LA1C1
        CMPX    #CasIOBuff L01DA
        BNE     LA127
        BSR     LA163
        BNE     LA11C
        BSR     LA173
        BNE     LA11C
        STA     2,Y
        BSR     LA173
        BNE     LA11C
        STA     3,Y
        BSR     LA16C
        BNE     LA11C
        LEAS    2,S
        PULS    D,PC

LA149   INC     ,Y
        LDA     ,Y
        CMPA    #$05
        BCS     LA16B
        LDA     #$BC
        JSR     >BasicSerOut L802D
        JMP     >LB84B

LA159   PSHS    A
        BSR     LA1BA
        BNE     LA161
        CMPA    ,S
LA161   PULS    A,PC

LA163   LDA     1,Y
        BSR     LA1BA
        BNE     LA16B
        CMPA    #$C8
LA16B   RTS

LA16C   BSR     LA173
        BNE     LA16B
        LDA     1,Y
        RTS

LA173   JSR     >BasicSerIn L802A
        PSHS    CC,A
        EORA    1,Y
        STA     1,Y
        PULS    CC,A,PC

LA17E   CLRA
        PSHS    D,X,Y,U
        ASL     7,S
        ROL     6,S
        LSR     7,S
        LEAY    ,S
        BRA     LA18D

LA18B   BSR     LA149
LA18D   LDA     #$97
        BSR     LA159
        BNE     LA18B
        LDA     6,Y
        BSR     LA1C1
        LDA     7,Y
        BSR     LA1C1
        BSR     LA163
        BNE     LA18B
        BSR     LA173
        BNE     LA18B
        STA     4,Y
        LDX     2,Y
        LDB     #$80
LA1A9   BSR     LA173
        BNE     LA18B
        STA     ,X+
        DECB
        BNE     LA1A9
        BSR     LA16C
        BNE     LA18B
        LEAS    4,S
        PULS    D,X,PC

LA1BA   CLR     1,Y
        BSR     LA1C9
        JMP     >BasicSerIn L802A

LA1C1   PSHS    A
        EORA    1,Y
        STA     1,Y
        PULS    A
LA1C9   JMP     >BasicSerOut L802D

LA1CC   LDA     #$01
        STA     <$D9
LA1D0   DECB
        JSR     >LA366
        JSR     <BasChrGetCurr
        LBEQ    LA266
        STB     <BasCloadMOffs
        JSR     >VarGetStr L8887
        JSR     >VarGetExpr L8877
        LDX     <BasVarAssign16
        STX     <$4D
        LDB     <$D9
        JSR     >L8DF3
        JSR     >L90E8
        LDX     <BasVarAssign16
        LDB     <$D9
        SUBB    ,X
LA1F4   DECB
        LBMI    LA341
        JSR     >TextOutSpace L90F5
        BRA     LA1F4

LA1FE   STB     <BasCloadMOffs
        STX     <BasTempPtr
        LDA     #$02
        STA     <$D9
LA206   LDA     ,X
        CMPA    #$25
        BEQ     LA1D0
        CMPA    #$20
        BNE     LA217
        INC     <$D9
        LEAX    1,X
        DECB
        BNE     LA206
LA217   LDX     <BasTempPtr
        LDB     <BasCloadMOffs
        LDA     #$25
LA21D   JSR     >LA366
        JSR     >TextOutChar LB54A
        BRA     LA247

LA225   JSR     >L8889
        JSR     >VarGetExpr L8877
        LDB     #$3B
        JSR     >VarCKChar L89AC
        LDX     <BasVarAssign16
        STX     <BasRenumStartLine
        BRA     LA23C

LA236   LDA     <BasEditorLineLen
        BEQ     LA242
        LDX     <BasRenumStartLine
LA23C   CLR     <BasEditorLineLen
        LDB     ,X
        BNE     LA245
LA242   JMP     >BasFCError L8B8D

LA245   LDX     2,X
LA247   CLR     <$DA
LA249   CLR     <$D9
        LDA     ,X+
        CMPA    #$21
        LBEQ    LA1CC
        CMPA    #$23
        BEQ     LA2B2
        DECB
        BNE     LA270
        JSR     >LA366
        JSR     >TextOutChar LB54A
LA260   JSR     <BasChrGetCurr
        BNE     LA236
        LDA     <BasEditorLineLen
LA266   BNE     LA26B
        JSR     >TextOutCRLF L90A1
LA26B   LDX     <BasRenumStartLine
        JMP     >VarDelVar L8D9F

LA270   CMPA    #$2B
        BNE     LA27D
        JSR     >LA366
        LDA     #$08
        STA     <$DA
        BRA     LA249

LA27D   CMPA    #$2E
        BEQ     LA2CF
        CMPA    #$25
        LBEQ    LA1FE
        CMPA    ,X
LA289   BNE     LA21D
        CMPA    #$24
        BEQ     LA2A8
        CMPA    #$2A
        BNE     LA289
        LDA     <$DA
        ORA     #$20
        STA     <$DA
        CMPB    #$02
        BCS     LA2AE
        LDA     1,X
        CMPA    #$24
        BNE     LA2AE
        DECB
        LEAX    1,X
        INC     <$D9
LA2A8   LDA     <$DA
        ORA     #$10
        STA     <$DA
LA2AE   LEAX    1,X
        INC     <$D9
LA2B2   CLR     <$D8
LA2B4   INC     <$D9
        DECB
        BEQ     LA302
        LDA     ,X+
        CMPA    #$2E
        BEQ     LA2DD
        CMPA    #$23
        BEQ     LA2B4
        CMPA    #$2C
        BNE     LA2E8
        LDA     <$DA
        ORA     #$40
        STA     <$DA
        BRA     LA2B4

LA2CF   LDA     ,X
        CMPA    #$23
        LBNE    LA21D
        LDA     #$01
        STA     <$D8
        LEAX    1,X
LA2DD   INC     <$D8
        DECB
        BEQ     LA302
        LDA     ,X+
        CMPA    #$23
        BEQ     LA2DD
LA2E8   CMPA    #$5E
        BNE     LA302
        CMPA    ,X
        BNE     LA302
        CMPA    1,X
        BNE     LA302
        CMPA    2,X
        BNE     LA302
        CMPB    #$04
        BCS     LA302
        SUBB    #$04
        LEAX    4,X
        INC     <$DA
LA302   LEAX    -1,X
        INC     <$D9
        LDA     <$DA
        BITA    #$08
        BNE     LA324
        DEC     <$D9
        TSTB
        BEQ     LA324
        LDA     ,X
        SUBA    #$2D
        BEQ     LA31D
        CMPA    #$FE
        BNE     LA324
        LDA     #$08
LA31D   ORA     #$04
        ORA     <$DA
        STA     <$DA
        DECB
LA324   JSR     <BasChrGetCurr
        LBEQ    LA266
        STB     <BasCloadMOffs
        JSR     >L8872
        LDA     <$D9
        ADDA    <$D8
        CMPA    #$11
        LBHI    BasFCError L8B8D
        JSR     >LA373
        LEAX    -1,X
        JSR     >TextOutString L90E5
LA341   CLR     <BasEditorLineLen
        JSR     <BasChrGetCurr
        BEQ     LA354
        STA     <BasEditorLineLen
        CMPA    #$3B
        BEQ     LA352
        JSR     >VarCKComma L89AA
        BRA     LA354

LA352   JSR     <BasChrGet
LA354   LDX     <BasRenumStartLine
        LDB     ,X
        SUBB    <BasCloadMOffs
        LDX     2,X
        ABX
        LDB     <BasCloadMOffs
        LBNE    LA247
        JMP     >LA260

LA366   PSHS    A
        LDA     #$2B
        TST     <$DA
        BEQ     LA371
        JSR     >TextOutChar LB54A
LA371   PULS    A,PC

LA373   LDU     #$03DB
        LDB     #$20
        LDA     <$DA
        BITA    #$08
        BEQ     LA380
        LDB     #$2B
LA380   TST     <$54
        BPL     LA388
        CLR     <$54
        LDB     #$2D
LA388   STB     ,U+
        LDB     #$30
        STB     ,U+
        ANDA    #$01
        LBNE    LA49B
        LDX     #D956E
        JSR     >L944B
        BMI     LA3B1
        JSR     >L9587
LA39F   LDA     ,X+
        BNE     LA39F
LA3A3   LDA     ,-X
        STA     1,X
        CMPX    #$03DA
        BNE     LA3A3
        LDA     #$25
        STA     ,X
        RTS

LA3B1   LDA     <BasVarFPAcc1
        STA     <$47
        BEQ     LA3BA
        JSR     >LA55B
LA3BA   LDA     <$47
        LBMI    LA441
        NEGA
        ADDA    <$D9
        SUBA    #$09
        JSR     >LA478
        JSR     >LA5F1
        JSR     >LA590
        LDA     <$47
        JSR     >LA60F
        LDA     <$47
        JSR     >LA5D7
        LDA     <$D8
        BNE     LA3DE
        LEAU    -1,U
LA3DE   DECA
        JSR     >LA478
LA3E2   JSR     >LA513
        TSTA
        BEQ     LA3EE
        CMPB    #$2A
        BEQ     LA3EE
        STB     ,U+
LA3EE   CLR     ,U
        LDX     #$03DA
LA3F3   LEAX    1,X
        STX     <BasTempPtr
        LDA     <$3A
        SUBA    <$10
        SUBA    <$D9
        BEQ     LA437
        LDA     ,X
        CMPA    #$20
        BEQ     LA3F3
        CMPA    #$2A
        BEQ     LA3F3
        CLRA
LA40A   PSHS    A
        LDA     ,X+
        CMPA    #$2D
        BEQ     LA40A
        CMPA    #$2B
        BEQ     LA40A
        CMPA    #$24
        BEQ     LA40A
        CMPA    #$30
        BNE     LA42C
        LDA     1,X
        BSR     LA438
        BCS     LA42C
LA424   PULS    A
        STA     ,-X
        BNE     LA424
        BRA     LA3F3

LA42C   PULS    A
        TSTA
        BNE     LA42C
        LDX     <BasTempPtr
        LDA     #$25
        STA     ,-X
LA437   RTS

LA438   CMPA    #$30
        BCS     LA440
        SUBA    #$3A
        SUBA    #$C6
LA440   RTS

LA441   LDA     <$D8
        BEQ     LA446
        DECA
LA446   ADDA    <$47
        BMI     LA44B
        CLRA
LA44B   PSHS    A
LA44D   BPL     LA459
        PSHS    A
        JSR     >L932D
        PULS    A
        INCA
        BRA     LA44D

LA459   LDA     <$47
        SUBA    ,S+
        STA     <$47
        ADDA    #$09
        BMI     LA47C
        LDA     <$D9
        SUBA    #$09
        SUBA    <$47
        BSR     LA478
        JSR     >LA5F1
        BRA     LA48D

LA470   PSHS    A
        LDA     #$30
        STA     ,U+
        PULS    A
LA478   DECA
        BPL     LA470
        RTS

LA47C   LDA     <$D9
        BSR     LA478
        JSR     >LA5DB
        LDA     #$F7
        SUBA    <$47
        BSR     LA478
        CLR     <BasVarFPAcc4
        CLR     <BasEditorLineLen
LA48D   JSR     >LA590
        LDA     <$D8
        BNE     LA496
        LDU     <BasVarPtrLast
LA496   ADDA    <$47
        LBRA    LA3DE

LA49B   LDA     <BasVarFPAcc1
        PSHS    A
        BEQ     LA4A4
        JSR     >LA55B
LA4A4   LDA     <$D8
        BEQ     LA4A9
        DECA
LA4A9   ADDA    <$D9
        CLR     $03DA
        LDB     <$DA
        ANDB    #$04
        BNE     LA4B7
        COM     $03DA
LA4B7   ADDA    $03DA
        SUBA    #$09
        PSHS    A
LA4BE   BPL     LA4CA
        PSHS    A
        JSR     >L932D
        PULS    A
        INCA
        BRA     LA4BE

LA4CA   LDA     ,S
        BMI     LA4CF
        CLRA
LA4CF   NEGA
        ADDA    <$D9
        INCA
        ADDA    $03DA
        STA     <BasVarFPAcc4
        CLR     <BasEditorLineLen
        JSR     >LA590
        PULS    A
        JSR     >LA60F
        LDA     <$D8
        BNE     LA4E8
        LEAU    -1,U
LA4E8   LDB     ,S+
        BEQ     LA4F5
        LDB     <$47
        ADDB    #$09
        SUBB    <$D9
        SUBB    $03DA
LA4F5   LDA     #$2B
        TSTB
        BPL     LA4FD
        LDA     #$2D
        NEGB
LA4FD   STA     1,U
        LDA     #$45
        STA     ,U++
        LDA     #$2F
LA505   INCA
        SUBB    #$0A
        BCC     LA505
        ADDB    #$3A
        STD     ,U++
        CLR     ,U
        JMP     >LA3E2

LA513   LDX     #$03DB
        LDB     ,X
        PSHS    B
        LDA     #$20
        LDB     <$DA
        BITB    #$20
        PULS    B
        BEQ     LA52C
        LDA     #$2A
        CMPB    #$20
        BNE     LA52C
        TFR     A,B
LA52C   PSHS    B
LA52E   STA     ,X+
        LDB     ,X
        BEQ     LA544
        CMPB    #$45
        BEQ     LA544
        CMPB    #$30
        BEQ     LA52E
        CMPB    #$2C
        BEQ     LA52E
        CMPB    #$2E
        BNE     LA548
LA544   LDA     #$30
        STA     ,-X
LA548   LDA     <$DA
        BITA    #$10
        BEQ     LA552
        LDB     #$24
        STB     ,-X
LA552   ANDA    #$04
        PULS    B
        BNE     LA55A
        STB     ,-X
LA55A   RTS

LA55B   PSHS    U
        CLRA
LA55E   STA     <$47
        LDB     <BasVarFPAcc1
        CMPB    #$80
        BHI     LA577
        LDX     #D956E
        JSR     >L9273
        LDA     <$47
        SUBA    #$09
        BRA     LA55E

LA572   JSR     >L932D
        INC     <$47
LA577   LDX     #D9569
        JSR     >L944B
        BGT     LA572
LA57F   LDX     #D9564
        JSR     >L944B
        BGT     LA58E
        JSR     >L9315
        DEC     <$47
        BRA     LA57F

LA58E   PULS    U,PC

LA590   PSHS    U
        JSR     >L90FD
        JSR     >L9473
        PULS    U
        LDX     #D9673
        LDB     #$80
LA59F   BSR     LA5D7
LA5A1   LDA     <$53
        ADDA    3,X
        STA     <$53
        LDA     <BasVarAssign16
        ADCA    2,X
        STA     <BasVarAssign16
        LDA     <$51
        ADCA    1,X
        STA     <$51
        LDA     <$50
        ADCA    ,X
        STA     <$50
        INCB
        RORB
        ROLB
        BVC     LA5A1
        BCC     LA5C3
        SUBB    #$0B
        NEGB
LA5C3   ADDB    #$2F
        LEAX    4,X
        TFR     B,A
        ANDA    #$7F
        STA     ,U+
        COMB
        ANDB    #$80
        CMPX    #CmdSQR L9697
        BNE     LA59F
        CLR     ,U
LA5D7   DEC     <BasVarFPAcc4
        BNE     LA5E4
LA5DB   STU     <BasVarPtrLast
        LDA     #$2E
        STA     ,U+
        CLR     <BasEditorLineLen
        RTS

LA5E4   DEC     <BasEditorLineLen
        BNE     LA5F0
        LDA     #$03
        STA     <BasEditorLineLen
        LDA     #$2C
        STA     ,U+
LA5F0   RTS

LA5F1   LDA     <$47
        ADDA    #$0A
        STA     <BasVarFPAcc4
        INCA
LA5F8   SUBA    #$03
        BCC     LA5F8
        ADDA    #$05
        STA     <BasEditorLineLen
        LDA     <$DA
        ANDA    #$40
        BNE     LA608
        STA     <BasEditorLineLen
LA608   RTS

LA609   PSHS    A
        BSR     LA5D7
        PULS    A
LA60F   DECA
        BMI     LA61C
        PSHS    A
        LDA     #$30
        STA     ,U+
        LDA     ,S+
        BNE     LA609
LA61C   RTS

LA61D   LDU     #DA62A
        LDA     <GrCurrPmode
        ASLA
        LDU     A,U
        RTS

LA626   BSR     LA61D
        JMP     ,U

DA62A   FDB     LA634
        FDB     LA650
        FDB     LA634
        FDB     LA650
        FDB     LA634

LA634   PSHS    B,U
        LDB     <GrBytesPerLine
        LDA     <$C0
        MUL
        ADDD    <GrDisplayStartAddr
        TFR     D,X
        LDB     <$BE
        LSRB
        LSRB
        LSRB
        ABX
        LDA     <$BE
        ANDA    #$07
        LDU     #PixMaskTable2Col LA66B
        LDA     A,U
        PULS    B,U,PC

LA650   PSHS    B,U
        LDB     <GrBytesPerLine
        LDA     <$C0
        MUL
        ADDD    <GrDisplayStartAddr
        TFR     D,X
        LDB     <$BE
        LSRB
        LSRB
        ABX
        LDA     <$BE
        ANDA    #$03
        LDU     #PixMaskTable4Col LA673
        LDA     A,U
        PULS    B,U,PC

PixMaskTable2Col:
LA66B   SUBA    #$40
        BRA     LA67F

        FCB     $08
        FCB     $04
        FCB     $02
        FCB     $01

PixMaskTable4Col:
LA673   SUBB    #$30
        INC     <BasGenCount
LA677   LDB     <GrBytesPerLine
        ABX
        RTS

LA67B   LSRA
        BCC     LA681
        RORA
LA67F   LEAX    1,X
LA681   RTS

LA682   LSRA
        BCC     LA67B
        LDA     #$C0
        LEAX    1,X
        RTS

LA68A   JSR     >L8E7A
        LDY     #GrCurrX L00BD
LA691   CMPB    #$C0
        BCS     LA697
        LDB     #$BF
LA697   CLRA
        STD     2,Y
        LDD     <$2B
        CMPD    #$0100
        BCS     LA6A5
        LDD     #$00FF
LA6A5   STD     ,Y
        RTS

LA6A8   JSR     >LA68A
LA6AB   LDU     #GrCurrX L00BD
LA6AE   LDA     <GrCurrPmode
        CMPA    #$02
        BCC     LA6BA
        LDD     2,U
        LSRA
        RORB
        STD     2,U
LA6BA   LDA     <GrCurrPmode
        CMPA    #$04
        BCC     LA6C6
        LDD     ,U
        LSRA
        RORB
        STD     ,U
LA6C6   RTS

CmdPPoint:
LA6C7   JSR     >LA740
        JSR     >LA6AB
        JSR     >LA626
        ANDA    ,X
        LDB     <GrCurrPmode
        RORB
        BCC     LA6E9
LA6D7   CMPA    #$04
        BCS     LA6DF
        RORA
        RORA
        BRA     LA6D7

LA6DF   INCA
        ASLA
        ADDA    <GrColourSet
        LSRA
LA6E4   TFR     A,B
        JMP     >VarAssign8Bit L8C36

LA6E9   TSTA
        BEQ     LA6E4
        CLRA
        BRA     LA6DF

CmdPset:
LA6EF   LDA     #$01
        BRA     LA6F4

CmdPReset:
LA6F3   CLRA
LA6F4   STA     <GrPlotFlag
        JSR     >VarCKOpBrac L89A7
        JSR     >LA6A8
        JSR     >LA90F
        JSR     >VarCKClBrac L89A4
        JSR     >LA626
LA705   LDB     ,X
        PSHS    B
        TFR     A,B
        COMA
        ANDA    ,X
        ANDB    <GrCurrColour
        PSHS    B
        ORA     ,S+
        STA     ,X
        SUBA    ,S+
        ORA     <GrDirtyFlag
        STA     <GrDirtyFlag
        RTS

LA71D   LDX     <GrCurrXCo
        STX     <GrCurrX
        LDX     <GrCurrYCo
        STX     <GrCurrY
        CMPA    #$C4
        BEQ     LA72C
        JSR     >LA740
LA72C   LDB     #$C4
        JSR     >VarCKChar L89AC
        JSR     >VarCKOpBrac L89A7
        JSR     >L8E7A
        LDY     #GrPixelNoX L00C3
        JSR     >LA691
        BRA     LA746

LA740   JSR     >VarCKOpBrac L89A7
        JSR     >LA68A
LA746   JMP     >VarCKClBrac L89A4

CmdLine:
LA749   CMPA    #$89
        LBEQ    CmdLineInput L9DB1
        CMPA    #$28
        BEQ     LA75C
        CMPA    #$C4
        BEQ     LA75C
        LDB     #$40
        JSR     >VarCKChar L89AC
LA75C   JSR     >LA71D
        LDX     <GrPixelNoX
        STX     <GrCurrXCo
        LDX     <GrPixelNoY
        STX     <GrCurrYCo
        JSR     >VarCKComma L89AA
        CMPA    #$AD
        BEQ     LA777
        CMPA    #$AC
        LBNE    BasSNError L89B4
        LDB     #$01

        FCB	Skip1LD
LA777   CLRB

        PSHS    B
        JSR     <BasChrGet
        JSR     >LA7AE
        PULS    B
        STB     <GrPlotFlag
        JSR     >GrSetColours LA928
        JSR     <BasChrGetCurr
        LBEQ    LA82F
        JSR     >VarCKComma L89AA
        LDB     #$42
        JSR     >VarCKChar L89AC
        BNE     LA7B7
        BSR     LA7D2
        BSR     LA7FC
        LDX     <GrCurrX
        PSHS    X
        LDX     <GrPixelNoX
        STX     <GrCurrX
        BSR     LA7FC
        PULS    X
        STX     <GrCurrX
        LDX     <GrPixelNoY
        STX     <GrCurrY
        BRA     LA7D2

LA7AE   JSR     >LA6AB
        LDU     #GrPixelNoX L00C3
        JMP     >LA6AE

LA7B7   LDB     #$46
        JSR     >VarCKChar L89AC
        BRA     LA7C2

LA7BE   LEAX    -1,X
LA7C0   STX     <GrCurrY
LA7C2   JSR     >LA7D2
        LDX     <GrCurrY
        CMPX    <GrPixelNoY
        BEQ     LA7D1
        BCC     LA7BE
        LEAX    1,X
        BRA     LA7C0

LA7D1   RTS

LA7D2   LDX     <GrCurrX
        PSHS    X
        JSR     >LAAB8
        BCC     LA7DF
        LDX     <GrPixelNoX
        STX     <GrCurrX
LA7DF   TFR     D,Y
        LEAY    1,Y
        JSR     >LA626
        PULS    U
        STU     <GrCurrX
        BSR     LA822
LA7EC   STA     <BasEditorLineLen
        JSR     >LA705
        LDA     <BasEditorLineLen
        JSR     ,U
        LEAY    -1,Y
        BNE     LA7EC
        RTS

LA7FA   PULS    D
LA7FC   LDD     <GrCurrY
        PSHS    D
        JSR     >LAAAB
        BCC     LA809
        LDX     <GrPixelNoY
        STX     <GrCurrY
LA809   TFR     D,Y
        LEAY    1,Y
        JSR     >LA626
        PULS    U
        STU     <GrCurrY
        BSR     LA82B
        BRA     LA7EC

DA818   FDB     LA67B
        FDB     LA682
        FDB     LA67B
        FDB     LA682
        FDB     LA67B

LA822   LDU     #DA818
        LDB     <GrCurrPmode
        ASLB
        LDU     B,U
        RTS

LA82B   LDU     #LA677
        RTS

LA82F   LDY     #DA89B
        JSR     >LAAAB
        LBEQ    LA7D2
        BCC     LA840
        LDY     #DA8A9
LA840   PSHS    D
        LDU     #LA894
        JSR     >LAAB8
        BEQ     LA7FA
        BCC     LA84F
        LDU     #LA8A2
LA84F   CMPD    ,S
        PULS    X
        BCC     LA85A
        EXG     U,Y
        EXG     D,X
LA85A   PSHS    D,U
        PSHS    D
        LSRA
        RORB
        BCS     LA86B
        CMPU    #DA89C
        BCS     LA86B
        SUBD    #$0001
LA86B   PSHS    D,X
        JSR     >LA61D
LA870   JSR     ,U
        JSR     >LA705
        LDX     6,S
        BEQ     LA890
        LEAX    -1,X
        STX     6,S
        JSR     [$08,S]
        LDD     ,S
        ADDD    2,S
        STD     ,S
        SUBD    4,S
        BCS     LA870
        STD     ,S
        JSR     ,Y
        BRA     LA870

LA890   PULS    X
        PULS    D,X,Y,U,PC

LA894   LDX     <GrCurrX
        LEAX    1,X
        STX     <GrCurrX
        RTS

DA89B   FCB     $9E
DA89C   FCB     $BF
        FCB     $30
        FCB     $01
        FCB     $9F
        FCB     $BF
        FCB     $39

LA8A2   LDX     <GrCurrX
        LEAX    -1,X
        STX     <GrCurrX
        RTS

DA8A9   FCB     $9E
        FCB     $BF
        FCB     $30
        FCB     $1F
        FCB     $9F
        FCB     $BF
        FCB     $39

LA8B0   LDU     #BasCloadMOffs L00D3
        LDX     #$00FF
        STX     ,U
        LDX     #GrCurrY L00BF
        STX     2,U
        JMP     >LA6AE

CmdPCls:
LA8C0   BEQ     LA8D0
        BSR     LA8E8
LA8C4   LDA     #$55
        MUL
GrClearGrScreen:
LA8C7   LDX     <GrDisplayStartAddr
LA8C9   STB     ,X+
        CMPX    <GrLastDisplayAddr
        BNE     LA8C9
        RTS

LA8D0   LDB     <GrBackground
        BRA     LA8C4

CmdColor:
LA8D4   CMPA    #$2C
        BEQ     LA8E0
        BSR     LA8E8
        STB     <GrForeground
        JSR     <BasChrGetCurr
        BEQ     LA8E7
LA8E0   JSR     >VarCKComma L89AA
        BSR     LA8E8
        STB     <GrBackground
LA8E7   RTS

LA8E8   JSR     >VarGet8Bit L8E51
LA8EB   CMPB    #$09
        LBCC    BasFCError L8B8D
        CLRA
        CMPB    #$05
        BCS     LA8FA
        LDA     #$08
        SUBB    #$04
LA8FA   PSHS    A
        LDA     <GrCurrPmode
        RORA
        BCC     LA909
        TSTB
        BNE     LA906
LA904   LDB     #$04
LA906   DECB
LA907   PULS    A,PC

LA909   RORB
        BCS     LA904
        CLRB
        BRA     LA907

LA90F   JSR     >GrSetColours LA928
        JSR     <BasChrGetCurr
        BEQ     LA926
        CMPA    #$29
        BEQ     LA926
        JSR     >VarCKComma L89AA
        CMPA    #$2C
        BEQ     LA926
        JSR     >LA8E8
        BSR     LA930
LA926   JMP     <BasChrGetCurr

GrSetColours:
LA928   LDB     <GrForeground
        TST     <GrPlotFlag
        BNE     LA930
        LDB     <GrBackground
LA930   STB     <GrColourTemp
        LDA     #$55
        MUL
        STB     <GrCurrColour
        RTS

GrSelectDisplay:
LA938   BNE     LA95D
TextResetVDU:
LA93A   PSHS    D,X
        LDX     #$FFC8
        STA     10,X
        STA     8,X
        STA     6,X
        STA     4,X
        STA     2,X
        STA     1,X
        STA     -2,X
        STA     -4,X
        STA     -6,X
        STA     -8,X
        LDA     $FF22
        ANDA    #$07
        STA     $FF22
        PULS    D,X,PC

LA95D   PSHS    D,X
        LDA     <GrCurrPmode
        ADDA    #$03
        LDB     #$10
        MUL
        ORB     #$80
        ORB     <GrColourSet
        LDA     $FF22
        ANDA    #$07
        PSHS    A
        ORB     ,S+
        STB     $FF22
        LDA     <GrDisplayStartAddr
        LSRA
        JSR     >GrSetVDGOffset LA99D
        LDA     <GrCurrPmode
        ADDA    #$03
        CMPA    #$07
        BNE     LA985
        DECA
LA985   BSR     GrSetVDGMode LA989
        PULS    D,X,PC

GrSetVDGMode:
LA989   LDB     #$03
        LDX     #$FFC0
LA98E   RORA
        BCC     LA995
        STA     1,X
        BRA     LA997

LA995   STA     ,X
LA997   LEAX    2,X
        DECB
        BNE     LA98E
        RTS

GrSetVDGOffset:
LA99D   LDB     #$07
        LDX     #$FFC6
        BRA     LA98E

GrSelectVDGColSet:
LA9A4   LDA     $FF22
        ANDA    #$F7
        ORA     <GrColourSet
        STA     $FF22
        RTS

CmdPmode:
LA9AF   CMPA    #$2C
        BEQ     LA9DE
        JSR     >VarGet8Bit L8E51
        CMPB    #$05
        BCC     LA9FB
        LDA     <GrStartPages
        STA     <GrDisplayStartAddr
        ASLB
        LDU     #DAAA2
        ADDA    B,U
        CMPA    <BasStartProg
        BHI     LA9FB
        STA     <GrLastDisplayAddr
        LEAU    -1,U
        LDA     B,U
        STA     <GrBytesPerLine
        LSRB
        STB     <GrCurrPmode
        CLRA
        STA     <GrBackground
        LDA     #$03
        STA     <GrForeground
        JSR     <BasChrGetCurr
        BEQ     LA9FA
LA9DE   JSR     >VarGetComma8 L8E7E
GrSelectPage:
LA9E1   TSTB
        BEQ     LA9FB
        DECB
        LDA     #$06
        MUL
        ADDB    <GrStartPages
        PSHS    B
        ADDB    <GrLastDisplayAddr
        SUBB    <GrDisplayStartAddr
        CMPB    <BasStartProg
        BHI     LA9FB
        STB     <GrLastDisplayAddr
        PULS    B
        STB     <GrDisplayStartAddr
LA9FA   RTS

LA9FB   JMP     >BasFCError L8B8D

CmdScreen:
LA9FE   CMPA    #$2C
        BEQ     LAA0D
        JSR     >VarGet8Bit L8E51
        TSTB
        JSR     >GrSelectDisplay LA938
        JSR     <BasChrGetCurr
        BEQ     LA9FA
LAA0D   JSR     >VarGetComma8 L8E7E
GrSelectColourSet:
LAA10   TSTB
        BEQ     LAA15
        LDB     #$08
LAA15   STB     <GrColourSet
        BRA     GrSelectVDGColSet LA9A4

CmdPClear:
LAA19   JSR     >VarGet8Bit L8E51
        TSTB
        BEQ     LA9FB
        CMPB    #$09
        BCC     LA9FB
GrReserveGrRam:
LAA23   LDA     #$06
        MUL
        ADDB    <GrStartPages
        TFR     B,A
        LDB     #$01
        TFR     D,Y
        CMPD    <GrLastDisplayAddr
        LBCS    BasFCError L8B8D
        SUBD    <BasStartProg
        ADDD    <BasVarSimpleAddr
        TFR     D,X
        ADDD    #$00C8
        SUBD    <AddrStack
        BCC     LA9FB
        LDA     <BasCurrentLine
        INCA
        BEQ     LAA4F
        TFR     Y,D
        SUBD    <BasStartProg
        ADDD    <BasAddrSigByte
        STD     <BasAddrSigByte
LAA4F   LDU     <BasVarSimpleAddr
        STX     <BasVarSimpleAddr
        CMPU    <BasVarSimpleAddr
        BCC     LAA6F
LAA58   LDA     ,-U
        STA     ,-X
        CMPU    <BasStartProg
        BNE     LAA58
        STY     <BasStartProg
        CLR     -1,Y
LAA66   JSR     >BasVect2 L83ED
        JSR     >L8424
        JMP     >BasRun L849F

LAA6F   LDU     <BasStartProg
        STY     <BasStartProg
        CLR     -1,Y
LAA76   LDA     ,U+
        STA     ,Y+
        CMPY    <BasVarSimpleAddr
        BNE     LAA76
        BRA     LAA66

LAA81   LDB     #$1E
        STB     <BasStartProg
        LDA     #$06
BasLocateScreen:
        STA     <GrStartPages
        STA     <GrDisplayStartAddr
        CLRA
        STA     <GrCurrPmode
        LDA     #$10
        STA     <GrBytesPerLine
        LDA     #$03
        STA     <GrForeground
        LDA     #$0C
        STA     <GrLastDisplayAddr
        LDX     <BasStartProg
        CLR     -1,X
        JMP     >BasNew L8417

        FCB     $10
DAAA2   FCB     $06
        FCB     $20
        FCB     $0C
        FCB     $10
        FCB     $0C
        FCB     $20
        FCB     $18
        FCB     $20
        FCB     $18

LAAAB   LDD     <GrPixelNoY
        SUBD    <GrCurrY
LAAAF   BCC     LAAEC
        PSHS    CC
        JSR     >LB15E
        PULS    CC,PC

LAAB8   LDD     <GrPixelNoX
        SUBD    <GrCurrX
        BRA     LAAAF

CmdPcopy:
LAABE   BSR     LAADA
        PSHS    D
        LDB     #$BC
        JSR     >VarCKChar L89AC
        BSR     LAADA
        PULS    X
        TFR     D,U
        LDY     #$0300
LAAD1   LDD     ,X++
        STD     ,U++
        LEAY    -1,Y
        BNE     LAAD1
        RTS

LAADA   JSR     >VarGet8Bit L8E51
        TSTB
        BEQ     LAAED

	ifeq	dragon64
        CMPB    <BasStartProg
        BHI     LAAED
        endc
	
	DECB
        LDA     #$06
        MUL
        ADDB    <GrStartPages
        
	ifne	dragon64
        CMPB    <BasStartProg
        BCC     LAAED
	endc
	
	EXG     A,B
LAAEC   RTS

LAAED   JMP     >BasFCError L8B8D

CmdGet:
LAAF0   CLRB
        BRA     LAAF5

CmdPut:
LAAF3   LDB     #$01
LAAF5   STB     <$D8
        JSR     VectAccessScreen L01A0
        CMPA    #$40
        BNE     LAB00
        JSR     <BasChrGet
LAB00   JSR     >LA71D
        JSR     >VarCKComma L89AA
        JSR     >LAC67
        TFR     X,D
        LDU     ,X
        LEAU    -2,U
        LEAU    D,U
        STU     <BasRenumStart
        LEAX    2,X
        LDB     ,X
        ASLB
        ABX
        STX     <BasRenumVal
        LDA     <BasVarType
        BNE     LAAED
        CLR     <$D4
        JSR     <BasChrGetCurr
        BEQ     LAB52
        COM     <$D4
        JSR     >VarCKComma L89AA
        TST     <$D8
        BNE     LAB35
        LDB     #$47
        JSR     >VarCKChar L89AC
        BRA     LAB65

LAB35   LDB     #$05
        LDX     #DABD4
LAB3A   LDU     ,X++
        LDY     ,X++
        CMPA    ,X+
        BEQ     LAB49
        DECB
        BNE     LAB3A
        JMP     >BasSNError L89B4

LAB49   STY     <BasRenumStartLine
        STU     <$D9
        JSR     <BasChrGet
        BRA     LAB65

LAB52   LDB     #$F8
        LDA     <GrCurrPmode
        RORA
        BCC     LAB5B
        LDB     #$FC
LAB5B   TFR     B,A
        ANDB    <$BE
        STB     <$BE
        ANDA    <$C4
        STA     <$C4
LAB65   JSR     >LAAB8
        BCC     LAB6E
        LDX     <GrPixelNoX
        STX     <GrCurrX
LAB6E   STD     <GrPixelNoX
        JSR     >LAAAB
        BCC     LAB79
        LDX     <GrPixelNoY
        STX     <GrCurrY
LAB79   STD     <GrPixelNoY
        LDA     <GrCurrPmode
        RORA
        LDD     <GrPixelNoX
        BCC     LAB86
        ADDD    <GrPixelNoX
        STD     <GrPixelNoX
LAB86   JSR     >LA7AE
        LDD     <GrPixelNoX
        LDX     <GrPixelNoY
        LEAX    1,X
        STX     <GrPixelNoY
        TST     <$D4
        BNE     LABED
        LSRA
        RORB
        LSRA
        RORB
        LSRA
        RORB
        ADDD    #$0001
        STD     <GrPixelNoX
        JSR     >LA626
LABA3   LDB     <$C4
        PSHS    X
LABA7   TST     <$D8
        BEQ     LABCC
        BSR     LABBE
        LDA     ,U
        STA     ,X+
LABB1   DECB
        BNE     LABA7
        PULS    X
        JSR     >LA677
        DEC     <$C6
        BNE     LABA3
LABBD   RTS

LABBE   LDU     <BasRenumVal
        LEAU    1,U
        STU     <BasRenumVal
        CMPU    <BasRenumStart
        BNE     LABBD
LABC9   JMP     >BasFCError L8B8D

LABCC   LDA     ,X+
        BSR     LABBE
        STA     ,U
        BRA     LABB1

DABD4   FDB	LAC2F
	FDB	LAC36
	FCB     $AC
	        
        FDB     LAC36
	FDB	LAC2F
	FCB	$AD
	
	FDB	LAC4C
	FDB	LAC36
	FCB	$C9

	FDB	LAC2F
	FDB	LAC4C
	FCB	$C8

	FDB	LAC3C
	FDB	LAC3C
	FCB	$C0


LABED   ADDD    #$0001
        STD     <GrPixelNoX
        LDA     <$D8
        BNE     LABFF
        LDU     <BasRenumStart
LABF8   STA     ,-U
        CMPU    <BasRenumVal
        BHI     LABF8
LABFF   JSR     >LA626
        LDB     <GrCurrPmode
        RORB
        BCC     LAC09
        ANDA    #$AA
LAC09   LDB     #$01
        LDY     <BasRenumVal
LAC0E   PSHS    A,X
        LDU     <GrPixelNoX
LAC12   PSHS    A,U
        LSRB
        BCC     LAC1F
        RORB
        LEAY    1,Y
        CMPY    <BasRenumStart
        BEQ     LABC9
LAC1F   TST     <$D8
        BEQ     LAC42
        BITB    ,Y
        BEQ     LAC2B
        JMP     [BasRenumStartLine] L00D5

LAC2B   JMP     [$00D9]

LAC2F	COMA	
	ANDA	,X
	STA	,X
	BRA	LAC4C

LAC36	ORA	,X
	STA	,X
	BRA	LAC4C

LAC3C	EORA	,X
	STA	,X
	BRA	LAC4C
	
LAC42   BITA    ,X
        BEQ     LAC4C
        TFR     B,A
        ORA     ,Y
        STA     ,Y
LAC4C   PULS    A,U
        JSR     >LA67B
        LEAU    -1,U
        CMPU    <Misc16BitScratch
        BNE     LAC12
        LDX     1,S
        LDA     <GrBytesPerLine
        LEAX    A,X
        PULS    A
        LEAS    2,S
        DEC     <$C6
        BNE     LAC0E
        RTS

LAC67   JSR     >VarGetVar L8A94
        LDB     ,-X
        LDA     ,-X
        TFR     D,U
        LDX     <BasVarArrayAddr
LAC72   CMPX    <BasVarEnd
        LBEQ    BasFCError L8B8D
        CMPU    ,X
        BEQ     LAC83
        LDD     2,X
        LEAX    D,X
        BRA     LAC72

LAC83   LEAX    2,X
        RTS

LAC86   RTS

CmdPaint:
LAC87   CMPA    #$40
        BNE     LAC8D
        JSR     <BasChrGet
LAC8D   JSR     >LA740
        JSR     >LA6AB
        LDA     #$01
        STA     <GrPlotFlag
        JSR     >LA90F
        LDD     <GrColourTemp
        PSHS    D
        JSR     <BasChrGetCurr
        BEQ     LACA5
        JSR     >LA90F
LACA5   LDA     <GrCurrColour
        STA     <$D8
        PULS    D
        STD     <GrColourTemp
        CLRA
        PSHS    D,X,U
        JSR     >LA8B0
        JSR     >LA61D
        STU     <$D9
        JSR     >LAD7A
        BEQ     LACCC
        JSR     >LAD66
        LDA     #$01
        STA     <BasEditorLineLen
        JSR     >LAD55
        NEG     <BasEditorLineLen
        JSR     >LAD55
LACCC   STS     <$DC
LACCF   TST     <GrDirtyFlag
        BNE     LACD6
        LDS     <$DC
LACD6   PULS    D,X,U
        CLR     <GrDirtyFlag
        STS     <$DC
        LEAX    1,X
        STX     <GrCurrX
        STU     <BasRenumStart
        STA     <BasEditorLineLen
        BEQ     LAC86
        BMI     LACEF
        INCB
        CMPB    <$D6
        BLS     LACF3
        CLRB
LACEF   TSTB
        BEQ     LACCF
        DECB
LACF3   STB     <$C0
        JSR     >LAD7A
        BEQ     LAD09
        CMPD    #$0003
        BCS     LAD04
        LEAX    -2,X
        BSR     LAD3C
LAD04   JSR     >LAD66
LAD07   BSR     LAD55
LAD09   COMA
        COMB
LAD0B   ADDD    <BasRenumStart
        STD     <BasRenumStart
        BLE     LAD27
        JSR     >LA894
        JSR     >LADAD
        BNE     LAD1E
        LDD     #$FFFF
        BRA     LAD0B

LAD1E   JSR     >LA8A2
        BSR     LAD61
        BSR     LAD83
        BRA     LAD07

LAD27   JSR     >LA894
        LEAX    D,X
        STX     <GrCurrX
        COMA
        COMB
        SUBD    #$0001
        BLE     LAD39
        TFR     D,X
        BSR     LAD3C
LAD39   JMP     >LACCF

LAD3C   STD     <GrCircleXCo
        PULS    Y
        LDD     <GrCurrX
        PSHS    D,X
        LDA     <BasEditorLineLen
        NEGA
LAD47   LDB     <$C0
        PSHS    D
        PSHS    Y
        LDB     #$02
        JSR     >BasChkB2Free L8331
        LDD     <GrCircleXCo
        RTS

LAD55   STD     <GrCircleXCo
        PULS    Y
        LDD     <GrPixelNoX
        PSHS    D,X
        LDA     <BasEditorLineLen
        BRA     LAD47

LAD61   LDX     <GrCurrX
        STX     <GrPixelNoX
        RTS

LAD66   STD     <GrCircleYCo
        LDY     <GrPixelNoX
        BSR     LAD61
        STY     <GrCurrX
        BSR     LAD83
        LDX     <GrCircleYCo
        LEAX    D,X
        ADDD    #$0001
        RTS

LAD7A   JSR     >LAD61
        LDY     #LA8A2
        BRA     LAD89

LAD83   LDY     #LA894
        JSR     ,Y
LAD89   LDU     <Misc16BitScratch
        LDX     <GrCurrX
LAD8D   BMI     LADA6
        CMPX    <BasCloadMOffs
        BHI     LADA6
        PSHS    Y,U
        BSR     LADAD
        BEQ     LADA4
        JSR     >LA705
        PULS    Y,U
        LEAU    1,U
        JSR     ,Y
        BRA     LAD8D

LADA4   PULS    Y,U
LADA6   TFR     U,D
        TFR     D,X
        SUBD    <Misc16BitScratch
        RTS

LADAD   JSR     [$00D9]
        TFR     A,B
        ANDB    <$D8
        PSHS    D
        ANDA    ,X
        CMPA    1,S
        PULS    D,PC

CmdPlay:
LADBD   LDX     <Misc16BitScratch
        LDB     #$01
        PSHS    B,X
        JSR     >VarGetStr L8887
        CLRB
        JSR     >LBAF1
        JSR     >SndEnable LBAC5
LADCD   JSR     >BasGetStrLenAddr L8D9A
        BRA     LADD4

LADD2   PULS    B,X
LADD4   STB     <$D8
        BEQ     LADD2
        STX     <$D9
        LBEQ    SndDisable LBAC3
LADDE   TST     <$D8
        BEQ     LADD2
        JSR     >LAF33
        CMPA    #$3B
        BEQ     LADDE
        CMPA    #$27
        BEQ     LADDE
        CMPA    #$58
        LBEQ    LAFA5
        BSR     LADF7
        BRA     LADDE

LADF7   CMPA    #$4F
        BNE     LAE08
        LDB     <SndOctave
        INCB
        BSR     LAE5B
        DECB
        CMPB    #$04
        BHI     LAE68
        STB     <SndOctave
        RTS

LAE08   CMPA    #$56
        BNE     LAE26
        LDB     <SndVolume
        LSRB
        LSRB
        SUBB    #$1F
        BSR     LAE5B
        CMPB    #$1F
        BHI     LAE68
        ASLB
        ASLB
        PSHS    B
	
	ifne	dragon64
        LDD     #$7E7C
	else
        LDD     #$7E7E
	endc
	
        ADDA    ,S
        SUBB    ,S+
        STD     <SndVolume
        RTS

LAE26   CMPA    #$4C
        BNE     LAE4D
        LDB     <SndNoteLen
        BSR     LAE5B
        TSTB
        BEQ     LAE68
        STB     <SndNoteLen
        CLR     <$E5
LAE35   BSR     LAE3A
        BCC     LAE35
        RTS

LAE3A   TST     <$D8
        BEQ     LAE48
        JSR     >LAF33
        CMPA    #$2E
        BEQ     LAE4A
        JSR     >LAF7D
LAE48   COMA
        RTS

LAE4A   INC     <$E5
        RTS

LAE4D   CMPA    #$54
        BNE     LAE5E
        LDB     <SndTempo
        BSR     LAE5B
        TSTB
        BEQ     LAE68
        STB     <SndTempo
        RTS

LAE5B   JMP     >LAF47

LAE5E   CMPA    #$50
        BNE     LAE86
        JSR     >LB066
        TSTB
        BNE     LAE6B
LAE68   JMP     >BasFCError L8B8D

LAE6B   LDA     <$E5
        LDX     <SndVolume
        PSHS    A,X
	
	ifne	dragon64
	LDA	#$7C
	else
        LDA     #$7E
	endc
	
        STA     <SndVolume
        STA     <$E0
        CLR     <$E5
        BSR     LAE82
        PULS    A,X
        STA     <$E5
        STX     <SndVolume
        RTS

LAE82   CLR     ,-S
        BRA     LAEC6

LAE86   CMPA    #$4E
        BNE     LAE8D
        JSR     >LAF33
LAE8D   CMPA    #$41
        BCS     LAE95
        CMPA    #$47
        BLS     SndPlayNote LAE9A
LAE95   JSR     >LAF59
        BRA     LAEBD

SndPlayNote:
LAE9A   SUBA    #$41
        LDX     #DAFF6
        LDB     A,X
        TST     <$D8
        BEQ     LAEBD
        JSR     >LAF33
        CMPA    #$23
        BEQ     LAEB0
        CMPA    #$2B
        BNE     LAEB3
LAEB0   INCB
        BRA     LAEBD

LAEB3   CMPA    #$2D
        BNE     LAEBA
        DECB
        BRA     LAEBD

LAEBA   JSR     >LAF7D
LAEBD   DECB
        CMPB    #$0B
        BHI     LAE68
        PSHS    B
        LDB     <SndNoteLen
LAEC6   LDA     <SndTempo
        MUL
        STD     <BasRenumStartLine
        LEAU    1,S
        LDA     <SndOctave
        CMPA    #$01
        BHI     LAEFF
        LDX     #DAFFD
        LDB     #$18
        MUL
        ABX
        PULS    B
        ASLB
        ABX
        LEAY    ,X
        BSR     LAF27
        STD     <SndTimerPlay
LAEE4   BSR     LAEF2
        LDA     <SndVolume
        BSR     LAEF5
        BSR     LAEF2
        LDA     <$E0
        BSR     LAEF5
        BRA     LAEE4

LAEF2 
	ifne	dragon64
	LDA	#$7C
	else
	LDA     #$7E
	endc
	
        NOP
LAEF5   STA     $FF20
        LDX     ,Y
LAEFA   LEAX    -1,X
        BNE     LAEFA
        RTS

LAEFF   LDX     #DB015
        LDB     #$0C
        MUL
        ABX
        PULS    B
        ABX
        BSR     LAF27
        STD     <SndTimerPlay
LAF0D   BSR     LAF1B
        LDA     <SndVolume
        BSR     LAF1E
        BSR     LAF1B
        LDA     <$E0
        BSR     LAF1E
        BRA     LAF0D

LAF1B   
	ifne	dragon64
	LDA	#$7C
	else
	LDA     #$7E
	endc

        NOP
LAF1E   STA     $FF20
        LDA     ,X
LAF23   DECA
        BNE     LAF23
        RTS

LAF27   LDB     #$FF
        LDA     <$E5
        BEQ     LAF32
        ADDA    #$02
        MUL
        LSRA
        RORB
LAF32   RTS

LAF33   PSHS    X
LAF35   TST     <$D8
        BEQ     LAF86
        LDX     <$D9
        LDA     ,X+
        STX     <$D9
        DEC     <$D8
        CMPA    #$20
        BEQ     LAF35
        PULS    X,PC

LAF47   BSR     LAF33
        CMPA    #$2B
        BEQ     LAF89
        CMPA    #$2D
        BEQ     LAF8D
        CMPA    #$3E
        BEQ     LAF97
        CMPA    #$3C
        BEQ     LAF92
LAF59   CMPA    #$3D
        BEQ     LAF9C
        JSR     >LA438
        BCS     LAF86
        CLRB
LAF63   SUBA    #$30
        STA     <BasEditorLineLen
        LDA     #$0A
        MUL
        TSTA
        BNE     LAF86
        ADDB    <BasEditorLineLen
        BCS     LAF86
        TST     <$D8
        BEQ     LAF8C
        JSR     >LAF33
        JSR     >LA438
        BCC     LAF63
LAF7D   INC     <$D8
        LDX     <$D9
        LEAX    -1,X
        STX     <$D9
        RTS

LAF86   JMP     >BasFCError L8B8D

LAF89   INCB
        BEQ     LAF86
LAF8C   RTS

LAF8D   TSTB
        BEQ     LAF86
        DECB
        RTS

LAF92   TSTB
        BEQ     LAF86
        LSRB
        RTS

LAF97   TSTB
        BMI     LAF86
        ASLB
        RTS

LAF9C   PSHS    Y,U
        BSR     LAFB6
        JSR     >L8E54
        PULS    Y,U,PC

LAFA5   JSR     >LAFB6
        LDB     #$02
        JSR     >BasChkB2Free L8331
        LDB     <$D8
        LDX     <$D9
        PSHS    B,X
        JMP     >LADCD

LAFB6   LDX     <$D9
        PSHS    X
        JSR     >LAF33
        JSR     >L8ADF
        BCS     LAF86
LAFC2   JSR     >LAF33
        CMPA    #$3B
        BNE     LAFC2
        PULS    X
        LDU     <BasAddrSigByte
        PSHS    U
        STX     <BasAddrSigByte
        JSR     >L89C1
        PULS    X
        STX     <BasAddrSigByte
        RTS

LAFD9   CLRA
        TFR     A,DP
        LDD     <SndTimerPlay
        LBEQ    LBB02
        SUBD    <BasRenumStartLine
        STD     <SndTimerPlay
        BHI     LAFF5
        CLR     <SndTimerPlay
        CLR     <SndTimerPlay+1
        PULS    A
        LDS     7,S
        ANDA    #$7F
        PSHS    A
LAFF5   RTI

DAFF6   FCB     $0A
        FCB     $0C
        FCB     $01
        FCB     $03
        FCB     $05
        FCB     $06
        FCB     $08
DAFFD   FCB     $01
        FCB     $A8
        FCB     $01
        FCB     $90
        FCB     $01
        FCB     $7A
        FCB     $01
        FCB     $64
        FCB     $01
        FCB     $50
        FCB     $01
        FCB     $3D
        FCB     $01
        FCB     $2B
        FCB     $01
        FCB     $1A
        FCB     $01
        FCB     $0A
        FCB     $00
        FCB     $FB
        FCB     $00
        FCB     $ED
        FCB     $00
        FCB     $DF
DB015   FCB     $00
        FCB     $D3
        FCB     $00
        FCB     $C7
        FCB     $00
        FCB     $BB
        FCB     $00
        FCB     $B1
        FCB     $00
        FCB     $A6
        FCB     $00
        FCB     $9D
        FCB     $00
        FCB     $94
        FCB     $00
        FCB     $8B
        FCB     $00
        FCB     $83
        FCB     $00
        FCB     $7C
        FCB     $00
        FCB     $75
        FCB     $00
        FCB     $6E
        FCB     $A6
        FCB     $9C
        FCB     $93
        FCB     $8B
        FCB     $83
        FCB     $7B
        FCB     $74
        FCB     $6D
        FCB     $67
        FCB     $61
        FCB     $5B
        FCB     $56
        FCB     $51
        FCB     $4C
        FCB     $47
        FCB     $43
        FCB     $3F
        FCB     $3B
        FCB     $37
        FCB     $34
        FCB     $31
        FCB     $2E
        FCB     $2B
        FCB     $28
        FCB     $26
        FCB     $23
        FCB     $21
        FCB     $1F
        FCB     $1D
        FCB     $1B
        FCB     $19
        FCB     $18
        FCB     $16
        FCB     $14
        FCB     $13
        FCB     $12

GrDraw:
LB051   LDX     <Misc16BitScratch
        LDB     #$01
        PSHS    B,X
        STB     <GrPlotFlag
        STX     <BasRenumStartLine
        JSR     >GrSetColours LA928
        JSR     >VarGetStr L8887
LB061   JSR     >BasGetStrLenAddr L8D9A
        BRA     LB06E

LB066   JSR     >LAF33
        JMP     >LAF59

LB06C   PULS    B,X
LB06E   STB     <$D8
        BEQ     LB06C
        STX     <$D9
        LBEQ    LB162
LB078   TST     <$D8
        BEQ     LB06C
        JSR     >LAF33
        CMPA    #$3B
        BEQ     LB078
        CMPA    #$27
        BEQ     LB078
        CMPA    #$4E
        BNE     LB08F
        COM     <BasRenumStartLine
        BRA     LB078

LB08F   CMPA    #$42
        BNE     LB097
        COM     <$D6
        BRA     LB078

LB097   CMPA    #$58
        LBEQ    LB133
        CMPA    #$4D
        LBEQ    LB1CD
        PSHS    A
        LDB     #$01
        TST     <$D8
        BEQ     LB0BC
        JSR     >LAF33
        JSR     >L8ADF
        PSHS    CC
        JSR     >LAF7D
        PULS    CC
        BCC     LB0BC
        BSR     LB066
LB0BC   PULS    A
        CMPA    #$43
        BEQ     LB0EA
        CMPA    #$41
        BEQ     LB0F4
        CMPA    #$53
        BEQ     LB0FC
        CMPA    #$55
        BEQ     LB12A
        CMPA    #$44
        BEQ     LB127
        CMPA    #$4C
        BEQ     LB122
        CMPA    #$52
        BEQ     LB11D
        SUBA    #$45
        BEQ     LB10D
        DECA
        BEQ     LB108
        DECA
        BEQ     LB116
        DECA
        BEQ     LB104
LB0E7   JMP     >BasFCError L8B8D

LB0EA   JSR     >LA8EB
        STB     <GrForeground
        JSR     >GrSetColours LA928
LB0F2   BRA     LB078

LB0F4   CMPB    #$04
        BCC     LB0E7
        STB     <$E8
        BRA     LB0F2

LB0FC   CMPB    #$3F
        BCC     LB0E7
        STB     <$E9
        BRA     LB0F2

LB104   CLRA
        BSR     LB15F
	
        FCB	Skip1
LB108   CLRA

        TFR     D,X
        BRA     LB166

LB10D   CLRA
        TFR     D,X
        BSR     LB15F
        EXG     D,X
        BRA     LB166

LB116   CLRA
        TFR     D,X
        BSR     LB15F
        BRA     LB166

LB11D   CLRA
LB11E   LDX     <Misc16BitScratch
        BRA     LB166

LB122   CLRA
        BSR     LB15F
        BRA     LB11E

LB127   CLRA
        BRA     LB12D

LB12A   CLRA
        BSR     LB15F
LB12D   LDX     <Misc16BitScratch
        EXG     X,D
        BRA     LB166

LB133   JSR     >LAFB6
        LDB     #$02
        JSR     >BasChkB2Free L8331
        LDB     <$D8
        LDX     <$D9
        PSHS    B,X
        JMP     >LB061

LB144   LDB     <$E9
        BEQ     LB163
        CLRA
        EXG     D,X
        STA     ,-S
        BPL     LB151
        BSR     LB15E
LB151   JSR     >LB350
        TFR     U,D
        LSRA
        RORB
DB158   LSRA
        RORB
        TST     ,S+
        BPL     LB162
LB15E   NEGA
LB15F   NEGB
        SBCA    #$00
LB162   RTS

LB163   TFR     X,D
        RTS

LB166   PSHS    D
        BSR     LB144
        PULS    X
        PSHS    D
        BSR     LB144
        PULS    X
        LDY     <$E8
        PSHS    Y
LB177   TST     ,S
        BEQ     LB183
        EXG     X,D
        BSR     LB15E
        DEC     ,S
        BRA     LB177

LB183   PULS    Y
        LDU     <Misc16BitScratch
        ADDD    <GrCurrXCo
        BMI     LB18D
        TFR     D,U
LB18D   TFR     X,D
        LDX     <Misc16BitScratch
        ADDD    <GrCurrYCo
        BMI     LB197
        TFR     D,X
LB197   CMPU    #SecVecSWI3 L0100
        BCS     LB1A0
        LDU     #$00FF
LB1A0   CMPX    #$00C0
        BCS     LB1A8
        LDX     #GrCurrY L00BF
LB1A8   LDD     <GrCurrXCo
        STD     <GrCurrX
        LDD     <GrCurrYCo
        STD     <GrCurrY
        STX     <GrPixelNoY
        STU     <GrPixelNoX
        TST     <BasRenumStartLine
        BNE     LB1BC
        STX     <GrCurrYCo
        STU     <GrCurrXCo
LB1BC   JSR     >LA7AE
        TST     <$D6
        BNE     LB1C6
        JSR     >LA82F
LB1C6   CLR     <BasRenumStartLine
        CLR     <$D6
        JMP     >LB078

LB1CD   JSR     >LAF33
        PSHS    A
        JSR     >LB1F9
        PSHS    D
        JSR     >LAF33
        CMPA    #$2C
        LBNE    LB0E7
        JSR     >LB1F6
        TFR     D,X
        PULS    U
        PULS    A
        CMPA    #$2B
        BEQ     LB1F1
        CMPA    #$2D
        BNE     LB197
LB1F1   TFR     U,D
        JMP     >LB166

LB1F6   JSR     >LAF33
LB1F9   CMPA    #$2B
        BEQ     LB204
        CMPA    #$2D
        BEQ     LB205
        JSR     >LAF7D
LB204   CLRA
LB205   PSHS    A
        JSR     >LB066
        PULS    A
        TSTA
        BEQ     LB213
        CLRA
        NEGB
        SBCA    #$00
LB213   RTS

        FCB     $00
        FCB     $00
DB216   FCB     $00
        FCB     $01
        FCB     $FE
        FCB     $C5
        FCB     $19
        FCB     $19
        FCB     $FB
        FCB     $16
        FCB     $31
        FCB     $F2
        FCB     $F4
        FCB     $FB
        FCB     $4A
        FCB     $51
        FCB     $EC
        FCB     $84
        FCB     $61
        FCB     $F9
        FCB     $E1
        FCB     $C7
        FCB     $78
        FCB     $AE
        FCB     $D4
        FCB     $DC
        FCB     $8E
        FCB     $3B
        FCB     $C5
        FCB     $E5
        FCB     $A2
        FCB     $69
        FCB     $B5
        FCB     $06
        FCB     $B5
        FCB     $06

CmdCircle:
LB238   CMPA    #$40
        BNE     LB23E
        JSR     <BasChrGet
LB23E   JSR     >LA8B0
        JSR     >LA740
        JSR     >LA6AB
        LDX     ,U
        STX     <GrCircleXCo
        LDX     2,U
        STX     <GrCircleYCo
        JSR     >VarCKComma L89AA
        JSR     >VarGet16Bit L8E83
        LDU     #BasRenumVal L00CF
        STX     ,U
        JSR     >LA6AE
        LDA     #$01
        STA     <GrPlotFlag
        JSR     >LA90F
        LDX     #SecVecSWI3 L0100
        JSR     <BasChrGetCurr
        BEQ     LB27A
        JSR     >VarCKComma L89AA
        JSR     >L8872
        LDA     <BasVarFPAcc1
        ADDA    #$08
        STA     <BasVarFPAcc1
        JSR     >L8E86
LB27A   LDA     <GrCurrPmode
        BITA    #$02
        BEQ     LB284
        TFR     X,D
        LEAX    D,X
LB284   STX     <BasRenumStart
        LDB     #$01
        STB     <GrPlotFlag
        STB     <$D8
        JSR     >LB37D
        PSHS    D
        JSR     >LB37D
        STD     <$D9
        PULS    D
LB298   PSHS    D
        LDX     <GrPixelNoX
        STX     <GrCurrX
        LDX     <GrPixelNoY
        STX     <GrCurrY
        LDU     #DB216
        ANDA    #$01
        BEQ     LB2AC
        NEGB
        ADDB    #$08
LB2AC   ASLB
        ASLB
        LEAU    B,U
        PSHS    U
        JSR     >LB342
        PULS    U
        LEAU    -2,U
        PSHS    X
        JSR     >LB342
        PULS    Y
        LDA     ,S
        ANDA    #$03
        BEQ     LB2CC
        CMPA    #$03
        BEQ     LB2CC
        EXG     X,Y
LB2CC   STX     <GrPixelNoX
        TFR     Y,X
        LDD     <BasRenumStart
        JSR     >LB350
        TFR     Y,D
        TSTA
        LBNE    BasFCError L8B8D
        STB     <GrPixelNoY
        TFR     U,D
        STA     <$C6
        LDA     ,S
        CMPA    #$02
        BCS     LB2F6
        CMPA    #$06
        BCC     LB2F6
        LDD     <GrCircleXCo
        SUBD    <GrPixelNoX
        BCC     LB303
        CLRA
        CLRB
        BRA     LB303

LB2F6   LDD     <GrCircleXCo
        ADDD    <GrPixelNoX
        BCS     LB301
        CMPD    <BasCloadMOffs
        BCS     LB303
LB301   LDD     <BasCloadMOffs
LB303   STD     <GrPixelNoX
        LDA     ,S
        CMPA    #$04
        BCS     LB315
        LDD     <GrCircleYCo
        SUBD    <GrPixelNoY
        BCC     LB322
        CLRA
        CLRB
        BRA     LB322

LB315   LDD     <GrCircleYCo
        ADDD    <GrPixelNoY
        BCS     LB320
        CMPD    <BasRenumStartLine
        BCS     LB322
LB320   LDD     <BasRenumStartLine
LB322   STD     <GrPixelNoY
        TST     <$D8
        BNE     LB32A
        BSR     LB37A
LB32A   PULS    D
        LSR     <$D8
        BCS     LB335
        CMPD    <$D9
        BEQ     LB341
LB335   INCB
        CMPB    #$08
        BNE     LB33E
        INCA
        CLRB
        ANDA    #$07
LB33E   JMP     >LB298

LB341   RTS

LB342   LDX     <BasRenumVal
        LDD     ,U
        BEQ     LB34F
        SUBD    #$0001
        BSR     LB350
        TFR     Y,X
LB34F   RTS

LB350   PSHS    D,X,Y,U
        CLR     4,S
        LDA     3,S
        MUL
        STD     6,S
        LDD     1,S
        MUL
        ADDB    6,S
        ADCA    #$00
        STD     5,S
        LDB     ,S
        LDA     3,S
        MUL
        ADDD    5,S
        STD     5,S
        BCC     LB36F
        INC     4,S
LB36F   LDA     ,S
        LDB     2,S
        MUL
        ADDD    4,S
        STD     4,S
        PULS    D,X,Y,U,PC

LB37A   JMP     >LA82F

LB37D   CLRB
        JSR     <BasChrGetCurr
        BEQ     LB393
        JSR     >VarCKComma L89AA
        JSR     >L8872
        LDA     <BasVarFPAcc1
        ADDA    #$06
        STA     <BasVarFPAcc1
        JSR     >L8E54
        ANDB    #$3F
LB393   TFR     B,A
        ANDB    #$07
        LSRA
        LSRA
        LSRA
        RTS

	ifne	dragon64ram

DoHWInit
LF39B   LDX     #DF40C				; Fill in some low mem vars
        LDU     #BasExecAddr 
        LDB     #$0E
        JSR     >UtilCopyBXtoU
	
        LDU     #SecVecIRQ 			; And some more
        LDB     #$0E
        JSR     >UtilCopyBXtoU
	
        LEAU    5,U				; yet more
        LDB     #$0B
        JSR     >UtilCopyBXtoU
	
        LDX     #BasSNError
        STX     3,U
        STX     8,U
        LDX     #VectDevOpen L015E
        LDD     #$394B
LF3C2   STA     ,X+
        DECB
        BNE     LF3C2
        JSR     >TextCls
        JSR     >L8424
        ANDCC   #$AF
        LDX     #BasSignonMess-1
        JSR     >TextOutString
        LDA     #$55
        STA     <WarmStartFlag
        STA     $011A
        BRA     LF3E1

DoSWInit:
LF3DE   JSR     >TextCls
LF3E1   CLR     <SndTimerPlay
        CLR     <SndTimerPlay+1
        LDA     >$FF03
        ORA     #$01
        STA     >$FF03
        CLR     <TextDevN
        JSR     >BasResetStack
        ANDCC   #$AF				; enable inturrupts
        JMP     >BasCmdMode			; go to basic command mode

	TST	>$FF23
	BMI	LF3FD
	RTI
	
LF3FD	JSR	>LF405

LF400   CLR     <WarmStartFlag
        JMP     >$C000

LF405	LDX	<Misc16BitScratch
LF407	LEAX	-1,X
	BNE	LF407
	RTS

DF40C   FDB     $CB8D			; Exec address
	INC	<$A7			; charget
	BNE	LF414
	INC	<BasAddrSigByte
LF414	LDA	>$0000
	JMP	>LBB26
	
	JMP	$FE18			; IRQ vector
        JMP	$F3F7			; FIRQ vector
	
        FDB     $0000			; Timer value
        FCB     $00			; unused
        FCB     $80,$4F,$C7,$52,$59	; Random number seeds
        
	FCB     $05			
		
        FCB     $4E			; no reserved words
        FDB     BasCommandWords		; word table
        FDB     BasCommandDisp		; dispatch address
        FCB     $22			; no of functions
        FDB     BasFunctionWords	; function table
        FDB     BasFunctionDisp			; Function dispatch
	
BasSignonMess:
	FCC	/(C) 1983 DRAGON DATA LTD /
        FCB     $0D
        FCC     /16K BASIC INTERPRETER 1.0      /
	FCB     $0D
MessByMicrosoft
        FCC     /(C) 1983 BY MICROSOFT/
        FCB     $0D,$0D
	FCB	$00
	
	else

LB39B   LDS     #BasBuffer L03D7
        LDA     #$37				; Enable FIRQ from PIA 
        STA     $FF23
	
        LDA     <WarmStartFlag			; Check warm start flag
        CMPA    #$55				; Warm start ?
	BNE     DoColdStart			; No, skip on
        
	LDX     <IndVecReset			; Get secondary reset vector
        LDA     ,X				; first byte of code
        CMPA    #$12				; is it a NOP ?
        BNE     DoColdStart			; No, do cold reset
        JMP     ,X				; else jump to secondary reset

;
; Called on system power on or reset, pointed to by reset vector.
;

SysReset:
LB3B4   LEAY    <LB39B,PCR			; Point to routine to call after hw init
        JMP     >BasicHWInit 			; Do hwinit


DoColdStart   
	LDX     #$0401				; Clear sys vars area $0000-$0400
LB3BD   CLR     ,--X
        LEAX    1,X
        BNE     LB3BD
	
        JSR     >TextCls 			; Clear text screen
        CLR     ,X+
        STX     <BasStartProg			; Set start of basic (asumes no graphics ram)
	
; Test and size ram
	
RamTestLoop   
	LDA     2,X				; Get a byte from ram				
        COMA					; change it
        STA     2,X				; store back
        CMPA    2,X				; did the store work, is it ram ?
        BNE     LB3D9				; nope, we have hit rom
	
        LEAX    1,X				; increment pointer
        COM     1,X				; restore ram contents (cleaver !)
        BRA     RamTestLoop			; loop again

LB3D9   STX     <AddrRamTop			; Save top of RAM (physical)				
        STX     <AddrFWareRamTop		; Save top of ram (cleared)
        STX     <BasVarStrTop			; Set top of string area
        LEAX    -200,X				; Set initial 200 byte string space
        STX     <AddrStack			; Set pointer to machine stack, below string area				
        TFR     X,S				; Point Stack there
	
        JSR     >BasicSWInit 			; Do basic software init
        LDX     #DB487				; Init Exec addr, charget etc
        LDU     #BasExecAddr 
        LDB     #$0E
        JSR     >UtilCopyBXtoU 

        LDU     #SecVecIRQ L010C		; Init FIRQ vector, basic stub etc
        LDB     #$1E
        JSR     >UtilCopyBXtoU 
        LDX     #BasSNError 
	
BasBootBasic:
LB400   STX     3,U				
        STX     8,U
	
        LDX     #VectDevOpen 			; Init RAM hooks
        LDD     #$394B				; make all $4b of them RTS :)
LB40A   STA     ,X+				; store RTS
        DECB					; decrement count
        BNE     LB40A				; loop if not all done
	
        STA     $02D9
        JSR     >BasNew 			; Perform a 'NEW'
        JSR     >L98E3
        LDX     #BasUsrVecNoDisk 		; Get address of USR table
        STX     <BasUSRTableAddr		; Set it
        LDU     #BasFCError 			; Set all USR vectors to ?FC Error initially
        LDB     #$0A				; 10 vectors
LB422   STU     ,X++				; Store address
        DECB					; dec counter
        BNE     LB422				; Loop again if not all done
	
        JSR     >LAA81
	
        LDA     $FF03				; Enable Timer IRQ ????
        ORA     #$01
        STA     $FF03
        
	LDX     #$444B				; Check for 'DK' flag for DOS cart
	CMPX    $C000
        LBEQ    $C002				; yes, found, jump to Dos init
	
        ANDCC   #$AF				; Enable IRQ and FIRQ
	
        LDX     #BasSignonMess-1 		; Display basic signon message
        JSR     >TextOutString 
        LDX     #WarmStart 			; Setup Warmstart vector 
        STX     <IndVecReset
        LDA     #$55
        STA     <WarmStartFlag			; Flag warm start
        BRA     LB466				; Enter basic command loop

WarmStart:
LB44F   NOP					; NOP, all warmstart routines must start with NOP.
        CLR     <SndTimerPlay			; Stop play timer
        CLR     <SndTimerPlay+1
	
        LDA     $FF03				; Enable IRQ
        ORA     #$01
        STA     $FF03
        
	CLR     <TextDevN			; Set device no back to screen/keyboard
        JSR     >BasResetStack 			; Reset basic stack
        ANDCC   #$AF				; Enable inturrupts
	
        JSR     >TextCls 			; clear screen
LB466   JMP     >BasCmdMode 			; Entter basic command loop


;
; FIRQ vector, used to aut-start cartrages
;
;LB469
FIRQVector   
	TST     $FF23				; FIRQ on PIA ?
        BMI     LB46F				; yes : do it
        RTI					; nope, return
	
LB46F   JSR     >LB480				; Delay
        JSR     >LB480				; Delay
        LEAY    <DoCartBoot,PCR			; Point to cart boot routine
        JMP     >BasicHWInit 			; Do hardware init, then boot cart

DoCartBoot   
	CLR     <WarmStartFlag			; Mark always cold start
        JMP     $C000				; Jump to cart

LB480   LDX     <Misc16BitScratch				; zero x
LB482   LEAX    -1,X				; Decrement X
        BNE     LB482				; loop again if not zero
        RTS
	
;
; The following are copied into low ram by the initialization routines
;

DB487
	ifne	dragon64
	FDB     DoBoot64			; Initial EXEC address
	else
	FDB     BasFCError			; Initial EXEC address
	endc
	
LB489   INC     <$A7				; Basic charget routine, copied into ram
        BNE     LB48F
        INC     <BasAddrSigByte
LB48F   LDA     >$0000
        JMP     >LBB26

LB495   JMP     >BasIRQVec L9D3D

LB498   JMP     >FIRQVector

        FDB     $0000				; Init timer value
        FCB     $00				; unused
        FCB     $80,$4F,$C7,$52,$59		; Random number seeds
        FCB     $00,$00,$00,$00,$00		; unused
	
	ifne	dragon64
	FCB	$05
	else
	FCB	$00
	endc
	
        FCB     $4E				; No of reserved words
	FDB	BasCommandWords			; Pointer to command word table
	FDB	BasCommandDisp			; Pointer to dispatch routine
        FCB     $22				; No of functions
	FDB	BasFunctionWords		; Pointer to function word table
	FDB	BasFunctionDisp			; Pointer to function dispatch table

BasSignonMess:
	FCC	/(C) 1982 DRAGON DATA LTD /
        FCB     $0D
        FCC     /16K BASIC INTERPRETER 1.0      /
	FCB     $0D
MessByMicrosoft
        FCC     /(C) 1982 BY MICROSOFT/
        FCB     $0D,$0D
	
	ifne	resettest
	FCC	/PHILL WAS HERE !/
	FCB	$0D
	endc

	FCB	$00

	endc

TextWaitKeyCurs2:
LB505   BSR     LB50A
        ANDA    #$7F
        RTS

LB50A   JSR     VectInChar L016A
        CLR     <CasEOFFlag
        TST     <TextDevN
        BEQ     LB538
        TST     <CasIOBuffSize
        BNE     LB51A
        COM     <CasEOFFlag
        RTS

LB51A   PSHS    B,X,Y,U
        LDX     <CasHeadBuffAddr
        LDA     ,X+
        PSHS    A
        STX     <CasHeadBuffAddr
        DEC     <CasIOBuffSize
        BNE     LB531
        LDA     <TextDevN
        CMPA    #$FD
        BEQ     LB533
        JSR     >LB867
LB531   PULS    D,X,Y,U,PC

LB533   JSR     >LA106
        BRA     LB531

LB538   PSHS    B,X
LB53A   JSR     >BasicCursorB L8009
        JSR     >BasicKbdIn L8006
        BEQ     LB53A
        LDB     #$60
        STB     [TextVDUCursAddr] L0088
        PULS    B,X,PC

TextOutChar:
LB54A   JSR     VectOutChar L0167
        PSHS    B
        LDB     <TextDevN
        CMPB    #$FD
        BNE     LB557
        PULS    B,PC

LB557   INCB
        PULS    B
        LBMI    BasicPrintOut L800F
        BNE     LB58F
        PSHS    D,X
        LDB     <CasStatus
        DECB
        BEQ     LB576
        LDB     <CasIOBuffSize
        INCB
        BNE     LB56E
        BSR     LB578
LB56E   LDX     <CasHeadBuffAddr
        STA     ,X+
        STX     <CasHeadBuffAddr
        INC     <CasIOBuffSize
LB576   PULS    D,X,PC

LB578   LDB     #$01
LB57A   STB     <CasBlockType
        LDX     #CasIOBuff L01DA
        STX     <CasIOBuffAddr
        LDB     <CasIOBuffSize
        STB     <CasBlockLen
        PSHS    A,Y,U
        JSR     >CasWriteBlock1 LB991
        PULS    A,Y,U
        JMP     >LB882

LB58F   JSR     >TextResetVDU LA93A
        JMP     >BasicScreenOut L800C

LB595   JSR     VectDevInit L0164
        PSHS    D,X
        CLR     <CasIOFlag
        LDA     <TextDevN
        BEQ     LB5A9
        INCA
        BEQ     LB5BA
        LDX     <TextPrnCommaW
        LDD     <TextPrnLineW
        BRA     LB5B2

LB5A9   LDB     <$89
        ANDB    #$1F
        LDX     #$1010
        LDA     #$20
LB5B2   STX     <TextVDUCommaW
        STB     <TextVDUCurrCol
        STA     <$6D
        PULS    D,X,PC

LB5BA   COM     <CasIOFlag
        LDX     #SecVecSWI3 L0100
        CLRA
        CLRB
        BRA     LB5B2

LB5C3   JSR     >TextCls LBA77
LB5C6   JSR     VectLineInputFile L0182
        CLR     <TextLastKey
        LDX     #$02DD
        LDB     #$01
LB5D0   JSR     >TextWaitKeyCurs2 LB505
BasInBuffFromX:
        TST     <CasEOFFlag
        BNE     LB602
        TST     <TextDevN
        BNE     LB5FE
        CMPA    #$0C
        BEQ     LB5C3
        CMPA    #$08
        BNE     LB5EA
        DECB
        BEQ     LB5C6
        LEAX    -1,X
        BRA     LB61E

LB5EA   CMPA    #$15
        BNE     LB5F8
LB5EE   DECB
        BEQ     LB5C6
        LDA     #$08
        JSR     >TextOutChar LB54A
        BRA     LB5EE

LB5F8   CMPA    #$03
        ORCC    #$01
        BEQ     LB603
LB5FE   CMPA    #$0D
        BNE     LB60F
LB602   CLRA
LB603   PSHS    CC
        JSR     >TextOutCRLF L90A1
        CLR     ,X
        LDX     #BasLinInpBuff L02DC
        PULS    CC,PC

LB60F   CMPA    #$20
        BCS     LB5D0
        CMPA    #$7B
        BCC     LB5D0
        CMPB    #$FA
        BCC     LB5D0
        STA     ,X+
        INCB
LB61E   JSR     >TextOutChar LB54A
        BRA     LB5D0

LB623   JSR     VectInputFile L016D
        LDA     <TextDevN
        BEQ     LB64B
        INCA
        BNE     LB639
        LDA     <CasStatus
        BNE     LB636
LB631   LDB     #$2E
        JMP     >SysErr L8344

LB636   DECA
        BEQ     LB64B
LB639   JMP     >BasFMError LB848

LB63C   JSR     VectOutputFile L0170
        LDA     <TextDevN
        INCA
        BNE     LB64B
        LDA     <CasStatus
        BEQ     LB631
        DECA
        BEQ     LB639
LB64B   RTS

LB64C   BEQ     LB65C
        JSR     >LB7D7
LB651   BSR     LB663
        JSR     <BasChrGetCurr
        BEQ     LB681
        JSR     >BasGetDevNo LB7D4
        BRA     LB651

LB65C   JSR     VectCloseAllFiles L0173
CasClosFiles:
LB65F   LDA     #$FF
        STA     <TextDevN
LB663   JSR     VectCloseFile L0176
        LDA     <TextDevN
        CLR     <TextDevN
        INCA
        BNE     LB681
        LDA     <CasStatus
        CMPA    #$02
        BNE     LB67F
        LDA     <CasIOBuffSize
        BEQ     LB67A
        JSR     >LB578
LB67A   LDB     #$FF
        JSR     >LB57A
LB67F   CLR     <CasStatus
LB681   RTS

LB682   CMPA    #$4D
        LBEQ    L9900
        JSR     >LB7AA
        JSR     <BasChrGetCurr
        BEQ     CasWriteBasic LB6A5
        JSR     >VarCKComma L89AA
        LDB     #$41
        JSR     >VarCKChar L89AC
        BNE     LB681
        CLRA
        JSR     >LB88E
        LDA     #$FF
        STA     <TextDevN
        CLRA
        JMP     >CmdList L8EAA

CasWriteBasic:
LB6A5   CLRA
        LDX     <Misc16BitScratch
        JSR     >LB891
        CLR     <CasStatus
        INC     <CasBlockType
        JSR     >CasWriteLeader L801B
        LDX     <BasStartProg
LB6B4   STX     <CasIOBuffAddr
        LDA     #$FF
        STA     <CasBlockLen
        LDD     <BasVarSimpleAddr
        SUBD    <CasIOBuffAddr
        BEQ     LB6CD
        CMPD    #$00FF
        BCC     LB6C8
        STB     <CasBlockLen
LB6C8   JSR     >CasBlockOut LB999
        BRA     LB6B4

LB6CD   NEG     <CasBlockType
        CLR     <CasBlockLen
        JMP     >LB994

LB6D4   CLR     <CasStatus
        CMPA    #$4D
        LBEQ    LA08E
        LEAS    2,S
        JSR     >LB7F7
        JSR     >LB87A
        TST     CasGapFlag L01E4
        BEQ     LB706
        LDA     CasASCIIFlag L01E3
        BEQ     LB70B
        JSR     >BasNew L8417
        LDA     #$FF
        STA     <TextDevN
        INC     <CasStatus
        JSR     >LB867
        JMP     >L837A

LB6FD   JSR     VectCloseFileCmd L0185
        JSR     >LB663
        JMP     >BasCmdMode L8371

LB706   LDA     CasFType L01E2
        BEQ     LB70E
LB70B   JMP     >BasFMError LB848

LB70E   JSR     >BasNew L8417
        JSR     >BasicCassOnRd L8021
        LDX     <BasStartProg
LB716   STX     <CasIOBuffAddr
        LDD     <CasIOBuffAddr
        INCA
        JSR     >L8335
        JSR     >CasBlockIn LB93E
        BNE     LB736
        LDA     <CasBlockType
        BEQ     LB736
        BPL     LB716
        STX     <BasVarSimpleAddr
        BSR     LB76D
        LDX     #MessOK-1
        JSR     >TextOutString L90E5
        JMP     >L83E7

LB736   JSR     >BasNew L8417
LB739   JMP     >LB84B

LB73C   LDX     <Misc16BitScratch
        JSR     <BasChrGetCurr
        BEQ     CasReadBin LB748
        JSR     >VarCKComma L89AA
        JSR     >VarGet16Bit L8E83
CasReadBin:
LB748   LDA     CasFType L01E2
        CMPA    #$02
        BNE     LB70B
        LDD     CasEntryAddr L01E5
        LEAU    D,X
        STU     <BasExecAddr
        LDD     CasLoadAddr L01E7
        LEAX    D,X
        STX     <CasIOBuffAddr
        JSR     >BasicCassOnRd L8021
LB760   JSR     >CasBlockIn LB93E
        BNE     LB739
        STX     <CasIOBuffAddr
        TST     <CasBlockType
        BEQ     LB739
        BPL     LB760
LB76D   JMP     >BasicCassOff L8018

LB770   BEQ     LB777
        JSR     >VarGet16Bit L8E83
        STX     <BasExecAddr
LB777   JMP     [BasExecAddr] L009D

LB77B   JSR     VectCheckKeys L017F
        LDA     <TextDevN
        INCA
        BEQ     LB7D3
        JMP     >BasPollKeyboard L851B

LB786   JSR     >L8B21
        SUBD    #$01FF
        LBHI    BasFCError L8B8D
        ADDD    #$05FF
        STD     <TextVDUCursAddr
        RTS

LB796   LDA     <TextLastKey
        BNE     LB79D
        JSR     >BasicKbdIn L8006
LB79D   CLR     <TextLastKey
        STA     <$53
        LBNE    L8DD5
        STA     <$56
        JMP     >L8DE1

LB7AA   LDX     #CasFNameLen L01D1
        CLR     ,X+
        LDA     #$20
LB7B1   STA     ,X+
        CMPX    #CasIOBuff L01DA
        BNE     LB7B1
        JSR     <BasChrGetCurr
        BEQ     LB7D3
        JSR     >VarGetStr L8887
        JSR     >BasGetStrLenAddr L8D9A
        LDU     #CasFNameLen L01D1
        STB     ,U+
        BEQ     LB7D3

	FCB	Skip2
LB7CA   LDB	#$08
	
UtilCopyBXtoU:
LB7CC   LDA     ,X+
        STA     ,U+
        DECB
        BNE     UtilCopyBXtoU LB7CC
LB7D3   RTS

BasGetDevNo:
LB7D4   JSR     >VarCKComma L89AA
LB7D7   CMPA    #$23
        BNE     LB7DD
        JSR     <BasChrGet
LB7DD   JSR     >L8872
LB7E0   JSR     >L8B2D
        ROLB
        ADCA    #$00
        BNE     LB851
        RORB
        STB     <TextDevN
        JSR     VectDevNo L0161
        BEQ     LB7F6
        BPL     LB851
        CMPB    #$FE
        BLT     LB851
LB7F6   RTS

LB7F7   BSR     LB7AA
LB7F9   JSR     <BasChrGetCurr
LB7FB   BEQ     LB7F6
        JMP     >BasSNError L89B4

LB800   JSR     VectCheckEOF L0188
        LDA     <TextDevN
        PSHS    A
        BSR     LB7E0
        JSR     >LB623
        CLRB
        LDA     <TextDevN
        BEQ     LB816
        TST     <CasIOBuffSize
        BNE     LB816
        COMB
LB816   PULS    A
        STA     <TextDevN
LB81A   SEX
        JMP     >VarAssign16Bit2 L8C37

LB81E   BSR     LB7F7
        BSR     LB87A
        JSR     >LB903
        BNE     LB84B
        RTS

LB828   JSR     VectDevOpen L015E
        JSR     >VarGetStr L8887
        JSR     >BasGetStrFirst L8DEA
        PSHS    B
        BSR     BasGetDevNo LB7D4
CmdOpenEntry:
        JSR     >VarCKComma L89AA
        BSR     LB7F7
        LDA     <TextDevN
        CLR     <TextDevN
        PULS    B
        CMPB    #$49
        BEQ     LB856
        CMPB    #$4F
        BEQ     LB88A
BasFMError:
LB848   LDB     #$2C

        FCB     $8C

LB84B   LDB     #$2A

        FCB	Skip2
LB84E   LDB	#$26

        FCB	Skip2
LB851   LDB	#$28
	
        JMP     >SysErr L8344

LB856   INCA
        BMI     BasFMError LB848
        BNE     LB889
        BSR     LB87A
        LDA     CasASCIIFlag L01E3
        ANDA    CasGapFlag L01E4
        BEQ     BasFMError LB848
        INC     <CasStatus
LB867   JSR     >CasReadBlock1 LB933
        BNE     LB84B
        TST     <CasBlockType
        BEQ     LB84B
        BMI     LB889
        LDA     <CasBlockLen
        BEQ     LB867
LB876   STA     <CasIOBuffSize
        BRA     LB884

LB87A   TST     <CasStatus
        BNE     LB84E
        BSR     CasFindFile LB8B3
        BNE     LB84B
LB882   CLR     <CasIOBuffSize
LB884   LDX     #CasIOBuff L01DA
        STX     <CasHeadBuffAddr
LB889   RTS

LB88A   INCA
        BNE     LB889
        INCA
LB88E   LDX     #$FFFF
LB891   TST     <CasStatus
        BNE     LB84E
        LDU     #CasIOBuff L01DA
        STU     <CasIOBuffAddr
        STA     8,U
        STX     9,U
        LDX     #CasFName L01D2
        JSR     >LB7CA
        CLR     <CasBlockType
        LDA     #$0F
        STA     <CasBlockLen
        JSR     >CasWriteBlock1 LB991
        LDA     #$02
        STA     <CasStatus
        BRA     LB882

CasFindFile:
LB8B3   LDX     #CasIOBuff L01DA
        STX     <CasIOBuffAddr
LB8B8   LDA     <BasCurrentLine
        INCA
        BNE     LB8C8
        JSR     >TextCls LBA77
        LDX     <TextVDUCursAddr
        LDB     #$53
        STB     ,X++
        STX     <TextVDUCursAddr
LB8C8   BSR     CasReadBlock1 LB933
        ORB     <CasBlockType
        
	ifne	dragon64
	BNE	LB8C8
	else
	BNE     LB902
	endc
	
        LDX     #CasIOBuff L01DA
        LDU     #CasFName L01D2
        LDB     #$08
        CLR     ,-S
LB8D8   LDA     ,X+
        LDY     <BasCurrentLine
        LEAY    1,Y
        BNE     LB8E6
        CLR     <TextDevN
        JSR     >TextOutChar LB54A
LB8E6   SUBA    ,U+
        ORA     ,S
        STA     ,S
        DECB
        BNE     LB8D8
        LDA     ,S+
        BEQ     LB8FD
        TST     -9,U
        BEQ     LB8FD
        BSR     LB903
        BNE     LB902
        BRA     LB8B8

LB8FD   LDA     #$46
        BSR     LB92A
        CLRA
LB902   RTS

LB903   TST     CasGapFlag L01E4
        BNE     LB911
        JSR     >BasicCassOnRd L8021
LB90B   BSR     CasBlockIn LB93E
        BSR     LB917
        BRA     LB90B

LB911   BSR     CasReadBlock1 LB933
        BSR     LB917
        BRA     LB911

LB917   BNE     LB91F
        LDA     <CasBlockType
        NEGA
        BMI     LB932
        DECA
LB91F   STA     <CasIOErrorCode
        LEAS    2,S
        BRA     LB938

LB925   LDA     $0400
        EORA    #$40
LB92A   LDB     <BasCurrentLine
        INCB
        BNE     LB932
        STA     $0400
LB932   RTS

CasReadBlock1:
LB933   JSR     >BasicCassOnRd L8021
        BSR     CasBlockIn LB93E
LB938   JSR     >BasicCassOff L8018
        LDB     <CasIOErrorCode
        RTS

CasBlockIn:
LB93E   ORCC    #$50
        BSR     LB925
        LDX     <CasIOBuffAddr
        CLRA
LB945   JSR     >BasicCassBitIn L8027
        RORA
        CMPA    #$3C
        BNE     LB945
        JSR     >BasicCassByIn L8024
        STA     <CasBlockType
        JSR     >BasicCassByIn L8024
        STA     <CasBlockLen
        ADDA    <CasBlockType
        STA     <CasCkSum
        LDA     <CasBlockLen
        STA     <CasIOErrorCode
        BEQ     LB972
LB961   JSR     >BasicCassByIn L8024
        STA     ,X
        CMPA    ,X+
        BNE     LB97C
        ADDA    <CasCkSum
        STA     <CasCkSum
        DEC     <CasIOErrorCode
        BNE     LB961
LB972   JSR     >BasicCassByIn L8024
        SUBA    <CasCkSum
        BEQ     LB97E
        LDA     #$01
        
	FCB	Skip2
LB97C   LDA	#$02

LB97E   STA     <CasIOErrorCode
        RTS

LB981   TFR     A,B
        JSR     <BasChrGet
        CMPB    #$C2
        BEQ     LB996
        CMPB    #$88
        JSR     >LB7FB
        JMP     >BasicCassOn L8015

CasWriteBlock1:
LB991   JSR     >CasWriteLeader L801B
LB994   BSR     CasBlockOut LB999
LB996   JMP     >BasicCassOff L8018

CasBlockOut:
LB999   ORCC    #$50
        LDB     <CasBlockLen
        STB     <CasIOErrorCode
        LDA     <CasBlockLen
        BEQ     LB9AA
        LDX     <CasIOBuffAddr
LB9A5   ADDA    ,X+
        DECB
        BNE     LB9A5
LB9AA   ADDA    <CasBlockType
        STA     <CasCkSum
        LDX     <CasIOBuffAddr
        BSR     LB9CD
        LDA     #$3C
        BSR     LB9CF
        LDA     <CasBlockType
        BSR     LB9CF
        LDA     <CasBlockLen
        BSR     LB9CF
        TSTA
        BEQ     LB9C9
LB9C1   LDA     ,X+
        BSR     LB9CF
        DEC     <CasIOErrorCode
        BNE     LB9C1
LB9C9   LDA     <CasCkSum
        BSR     LB9CF
LB9CD   LDA     #$55
LB9CF   JMP     >BasicCassByOut L801E

LB9D2   BSR     LBA13
        PSHS    X
        JSR     >VarGetComma8 L8E7E
        PULS    X
        CMPB    #$08
        BHI     LBA24
GrSetLRGPixel:
LB9DF   DECB
        BMI     LB9E7
        LDA     #$10
        MUL
        BRA     LB9EF

LB9E7   LDB     ,X
        BPL     LB9EE
        ANDB    #$70

        FCB	Skip1
LB9EE   CLRB

LB9EF   PSHS    B
        BSR     LBA5C
        LDA     ,X
        BMI     LB9F8
        CLRA
LB9F8   ANDA    #$0F
        ORA     <GrSetResetData
        ORA     ,S+
LB9FE   ORA     #$80
        STA     ,X
        RTS

LBA03   BSR     LBA13
        BSR     LBA5C
GrResetLRGPixel:
LBA07   CLRA
        LDB     ,X
        BPL     LB9FE
        COM     <GrSetResetData
        ANDB    <GrSetResetData
        STB     ,X
        RTS

LBA13   JSR     >VarCKOpBrac L89A7
LBA16   JSR     >VarGet8Bit L8E51
        CMPB    #$3F
        BHI     LBA24
        PSHS    B
        JSR     >VarGetComma8 L8E7E
        CMPB    #$1F
LBA24   BHI     LBA97
        PSHS    B
GrCalcPixelPos:
LBA28   LSRB
        LDA     #$20
        MUL
        LDX     #$0400
        LEAX    D,X
        LDB     1,S
        LSRB
        ABX
        PULS    D
        ANDA    #$01
        RORB
        ROLA
        LDB     #$10
LBA3D   LSRB
        DECA
        BPL     LBA3D
        STB     <GrSetResetData
        RTS

LBA44   BSR     LBA16
        LDB     #$FF
        LDA     ,X
        BPL     LBA59
        ANDA    <GrSetResetData
DBA4E   BEQ     LBA58
        LDB     ,X
        LSRB
        LSRB
        LSRB
        LSRB
        ANDB    #$07
LBA58   INCB
LBA59   JSR     >LB81A
LBA5C   JMP     >VarCKClBrac L89A4

LBA5F   JSR     VectAccessScreen L01A0
        BEQ     TextCls LBA77
        JSR     >VarGet8Bit L8E51
        CMPB    #$08
        BHI     LBA86
        TSTB
        BEQ     LBA74
        DECB
        LDA     #$10
        MUL
        ORB     #$0F
LBA74   ORB     #$80

        FCB     $8C

TextCls:
LBA77   LDB     #$60
TextClsChar:
LBA79   LDX     #$0400
        STX     <TextVDUCursAddr
LBA7E   STB     ,X+
        CMPX    #$05FF
        BLS     LBA7E
        RTS

LBA86   BSR     TextCls LBA77
        LDX     #MessByMicrosoft-1
        JMP     >TextOutString L90E5

LBA8E   JSR     >VarCKComma L89AA
LBA91   JSR     >VarGet8Bit L8E51
        TSTB
        BNE     LBAD3
LBA97   JMP     >BasFCError L8B8D

LBA9A   BSR     LBA91
        STB     <SndPitch
        BSR     LBA8E
	
SndBeep:
LBAA0   LDA     #$04
        MUL
        STD     <SndLength
        LDA     $FF03
        ORA     #$01
        STA     $FF03
	
	ifne	dragon64
	NOP
	CLRB
	else
        CLR     <BasDisArraySearch
	endc
	
        BSR     LBAF1
        BSR     SndEnable LBAC5
LBAB3   BSR     SysResetDA LBAD4
        LDA     #$FC
        BSR     SysWriteDA LBAD6
        BSR     SysResetDA LBAD4
        LDA     #$00
        BSR     SysWriteDA LBAD6
        LDX     <SndLength
        BNE     LBAB3
SndDisable:
LBAC3   CLRA
	FCB	Skip2			; CMPX    #D8608

SndEnable:
LBAC5   LDA	#$08
        STA     ,-S
        LDA     $FF23
        ANDA    #$F7
        ORA     ,S+
        STA     $FF23
LBAD3   RTS

SysResetDA:
LBAD4   
	ifne	dragon64
	LDA     #$7C
	else
	LDA     #$7E
	endc
	
SysWriteDA:
LBAD6   STA     $FF20
        LDA     <SndPitch
LBADB   INCA
        BNE     LBADB
        RTS

CmdAudio:
LBADF   TFR     A,B
        JSR     <BasChrGet
        CMPB    #$C2
        BEQ     SndDisable LBAC3
        SUBB    #$88
        JSR     >LB7FB
CasAudioOn:
LBAEC   INCB
SndDTOAOn:
        BSR     LBAF1
        BRA     SndEnable LBAC5

LBAF1   LDU     #$FF01
        BSR     LBAF6
LBAF6   LDA     ,U
        ANDA    #$F7
        ASRB
        BCC     LBAFF
        ORA     #$08
LBAFF   STA     ,U++
        RTS

LBB02   LDX     >SndLength L008D
        BEQ     LBB0C
        LEAX    -1,X
        STX     >SndLength L008D
LBB0C   RTI
CmdJoystk:
LBB0D   JSR     >L8E54
        CMPB    #$03
        LBHI    BasFCError L8B8D
        TSTB
        BNE     LBB1C
        JSR     >BasicJoyIn L8012
LBB1C   LDX     #BasJoyVal0 L015A
        LDB     <$53
        LDB     B,X
        JMP     >VarAssign8Bit L8C36

LBB26   CMPA    #$3A
        BCC     LBB34
        CMPA    #$20
        BNE     LBB30
        JMP     <BasChrGet

LBB30   SUBA    #$30
        SUBA    #$D0
LBB34   RTS

	ifeq	dragon64ram
        ifne	dragon64
	FILL	$00,7
	else
	FCC     /9999999n/
        FCB     $A4
        FCB     $39
        FCB     $39
	endc
	else
	endc
	
;
; Do initial hardware init, program PIAs, SAM & VDG.
;
	ifeq	dragon64ram
;LBB40
DoHWInit  
	LDD     #$0034			; Setup PIA0
        LDX     #$FF00
        STA     1,X
        STA     3,X
        STA     ,X
        COMA
        STA     2,X
        STB     1,X
        STB     3,X

        LDX     #$FF20			; Setup PIA1
        CLR     1,X
        CLR     3,X
        DECA
        STA     ,X
        LDA     #$F8
        STA     2,X
        STB     1,X
        STB     3,X
        CLR     ,X
        CLR     2,X
	
	ifne	dragon64
        LDD     #$0A98			; init ACIA
        STD     $FF06
        LDA     $FF04
	else
        LDA     2,X			; Read memory config bit from PIA
	endc
	
        LDX     #$FFC0			; Zero SAM bits for VDG mode & Display offset
        LDB     #$10
LBB70   STA     ,X++
        DECB
        BNE     LBB70
	
        STB     $FFC9			; Set display offse to $0400

	ifeq	dragon64
	
        BITA    #$04			; Check memory config Dragon 32 only
        BEQ     LBB81			; jump ahead if 64K rams
        STB     $FFDB			; Setup for 1 or 2 banks of 16K
	
LBB7F   BRA     LBB84			; skip 
	endc

LBB81   STB     $FFDD			; Setup for 1 bank of 64K (or half good 64K)
	
	ifne	dragon64
	BRA	LBB83

	
SysBoot64:				
LBB80   JMP     >DoBoot64			; Boot to all RAM mode, 64/Alpha

LBB83	clrb
	endc

LBB84   
	TFR     B,DP			; Set DP=0
        TFR     Y,PC			; chain routine ($B39B)

DoSWInit   
	LDX     #DBB9F			; Init some stuff in low ram
        LDU     #TextCursFalshCnt 
        LDB     #$0D
        BSR     LBB97
	
        ifne	dragon64
        JSR     >LBEFF
	else
	LDU     #TextPrnAutoCRLF L0148
	endc
	
        LDB     #$09
	
LBB97   LDA     ,X+
        STA     ,U+
        DECB
        BNE     LBB97
        RTS
	
;
; Data copied into ram by above
;
	

	ifne	dragon64
DBB9F   FCB     $32			; cursor flash count
        FDB     $0100			; No of leader bytes on cassette files
        FCB     $12			; Min cycle width 1200Hz
        FCB     $0A			; Min pulse width 1200Hz
        FCB     $12			; Max pulse width 1200Hz
        FDB     $DA5C			; Cassett motor delay
        FDB     $045E			; Keyboard debounce value
        FCB     $10			; Line printer comma width
        FCB     $74			; Line printer last comma field
        FCB     $84			; Line printer width
	
        FCB     $FF			; Auto line feed flag	
        FCB     $FF			; Alpha lock flag
        FCB     $01			; Printer EOL length
        FCB     $0D			; CR
        FCB     $0A			; LF
        FCB     $00,$00,$00,$00		; D.N.Smeed wasn't here !
	else
DBB9F   FCB     $32			; cursor flash count
        FDB     $0080			; No of leader bytes on cassette files
        FCB     $12			; Min cycle width 1200Hz
        FCB     $0A			; Min pulse width 1200Hz
        FCB     $12			; Max pulse width 1200Hz
        FDB     $DA5C			; Cassett motor delay
        FDB     $045E			; Keyboard debounce value
        FCB     $10			; Line printer comma width
        FCB     $74			; Line printer last comma field
        FCB     $84			; Line printer width
	
        FCB     $FF			; Auto line feed flag	
        FCB     $FF			; Alpha lock flag
        FCB     $01			; Printer EOL length
        FCB     $0D			; CR
        FCB     $0A			; LF
        FCC     / DNS/			; D.N.Smeed was here !
	endc

	endc
	
TextUpdateCurs:
LBBB5   DEC     <TextCursFalshCnt
        BNE     LBBC5
        LDA     #$32
        STA     <TextCursFalshCnt
        LDX     <TextVDUCursAddr
        LDA     ,X

	ifne 	dragon64ram
        CMPA    #$AF
	BEQ     LFAC7
	LDA     #$8F
LFAC7   EORA    #$20
	else
        EORA    #$40
	endc
	
        STA     ,X
LBBC5   LDX     #$045E
LBBC8   LEAX    -1,X
        BNE     LBBC8
        RTS


LBBCD   LDB     $FF00
        ORB     #$80
        TST     $FF02
        BMI     LBBD9
        ORB     #$40
LBBD9   RTS

LBBDA   LDB     #$7F
        STB     $FF02
        LDB     $FF00
        ANDB    #$40
        RTS

TextScanKbd:
LBBE5   PSHS    B,X
        BSR     LBBEC
	
	ifne	dragon64
	JMP     >LBEE0
	else
        TSTA
        PULS    B,X,PC
	endc
	
LBBEC   LEAS    -2,S
        LDX     #$0151
        CLR     $FF02
        LDB     $FF00
        ORB     #$80
        CMPB    ,X
        BEQ     LBC6F
        TFR     B,A
        COM     $FF02
        BSR     LBBCD
        CMPB    #$FF
        BNE     LBC6F
        STA     ,X+
        CLR     ,S
        LDB     #$FE
        STB     $FF02
LBC11   BSR     LBBCD
        STB     1,S
        EORB    ,X
        ANDB    ,X
        LDA     1,S
        STA     ,X+
        TSTB
        BNE     LBC2A
        INC     ,S
        COMA
        ROL     $FF02
        BCS     LBC11
        BRA     LBC6F

LBC2A   LDX     <TextKbdDelay
        BSR     LBBC8
        EXG     A,B
        BSR     LBBCD
        CMPB    1,S
        EXG     A,B
        BNE     LBC6F
        LDA     ,S
        SUBA    #$08
LBC3C   ADDA    #$08
        LSRB
        BCC     LBC3C
        TSTA
        BEQ     LBC76
        CMPA    #$0C
        BCS     LBC5F
        CMPA    #$11
        BCS     LBC74
        CMPA    #$2A
        BHI     LBC72
        ADDA    #$30
        BSR     LBBDA
        BEQ     LBC68
        TST     TextCapsLock L0149
        BNE     LBC68
        ORA     #$20
        BRA     LBC68

LBC5F   ADDA    #$30
        LBSR    LBBDA
        BNE     LBC68
        SUBA    #$10
LBC68   CMPA    #$12
        BNE     LBC70
        COM     TextCapsLock L0149
LBC6F   CLRA
LBC70   PULS    X,PC

LBC72   SUBA    #$1A
LBC74   SUBA    #$0B
LBC76   ASLA
        LBSR    LBBDA
        BNE     LBC7D
        INCA
LBC7D   LDX     #DBC84
        LDA     A,X
        BRA     LBC68

DBC84   FCB     $30
        FCB     $12
        FCB     $2C
        FCB     $3C
        FCB     $2D
        FCB     $3D
        FCB     $2E
        FCB     $3E
        FCB     $2F
        FCB     $3F
        FCB     $40
        FCB     $13
        FCB     $5E
        FCB     $5F
        FCB     $0A
        FCB     $5B
        FCB     $08
        FCB     $15
        FCB     $09
        FCB     $5D
        FCB     $20
        FCB     $20
        FCB     $0D
        FCB     $0D
        FCB     $0C
        FCB     $5C
        FCB     $03
        FCB     $03

TextClearLine:
LBCA0   LDA     #$60
        STA     ,X+
        TFR     X,D
        ANDB    #$1F
        BNE     TextClearLine LBCA0
        RTS

TextVDUOut:
LBCAB   PSHS    D,X
        LDX     <TextVDUCursAddr
        CMPA    #$08
        BNE     LBCBE
        CMPX    #$0400
        BEQ     LBCF3
        LDA     #$60
        STA     ,-X
        BRA     LBCDB

LBCBE   CMPA    #$0D
        BNE     LBCC6
        BSR     TextClearLine LBCA0
        BRA     LBCDB

LBCC6   CMPA    #$20
        BCS     LBCF3
        TSTA
        BMI     LBCD9
        CMPA    #$40
        BCS     LBCD7
        CMPA    #$60
        BCS     LBCD9
        ANDA    #$DF
LBCD7   EORA    #$40
LBCD9   STA     ,X+
LBCDB   STX     <TextVDUCursAddr
        CMPX    #$05FF
        BLS     LBCF3
        LDX     #$0400
LBCE5   LDD     $20,X
        STD     ,X++
        CMPX    #$05E0
        BCS     LBCE5
        STX     <TextVDUCursAddr
        BSR     TextClearLine LBCA0
LBCF3   PULS    D,X,PC

PrinterDirOut:
LBCF5   PSHS    B
	
	ifne	dragon64
        TST     TextPrnSelFlag L03FF
        JMP     >LBEC5
	else
LBCF7   LDB     $FF22
        RORB
        BCS     LBCF7
	endc
	
LBCFD   STA     $FF02
        LDB     #$02
        STB     $FF20
        CLR     $FF20
        PULS    B,PC

PrinterCRLF:
LBD0A   LDX     #TextPrnEOLCnt L014A
        LDB     ,X+
LBD0F   

	ifeq	dragon64ram
	TSTB
	endc
	
        BEQ     LBD19
        LDA     ,X+
        BSR     PrinterDirOut LBCF5

	ifne	dragon64
        JMP     >LBF0C
	else
        DECB
        BRA     LBD0F
	endc

LBD19   RTS

PrinterOut:
LBD1A   PSHS    D,X
        CMPA    #$0D
        BEQ     LBD33
        CMPA    #$20
        BCS     LBD26
        INC     <TextPrnCurrCol
LBD26   BSR     PrinterDirOut LBCF5
        LDB     <TextPrnCurrCol
        CMPB    <TextPrnLineW
        BCS     LBD3F
        TST     TextPrnAutoCRLF L0148
        BNE     LBD3D
LBD33   TST     <TextPrnCurrCol
        BNE     LBD3B
        LDA     #$20
        BSR     PrinterDirOut LBCF5
LBD3B   BSR     PrinterCRLF LBD0A
LBD3D   CLR     <TextPrnCurrCol
LBD3F   PULS    D,X,PC

SysSelJoystick:
LBD41   LDU     #$FF01
        BSR     LBD46
LBD46   LDA     ,U
        ANDA    #$F7
        RORB
        BCC     LBD4F
        ORA     #$08
LBD4F   STA     ,U++
        RTS

SysReadJoystick:
LBD52   LEAS    -3,S
        LDX     #VectDevOpen L015E
        LDB     #$03
LBD59   LDA     #$0A
        STD     1,S
        BSR     SysSelJoystick LBD41
LBD5F   LDD     #$4080
LBD62   STA     ,S
        STB     $FF20
        TST     $FF00
        BMI     LBD70
        SUBB    ,S
        BRA     LBD72

LBD70   ADDB    ,S
LBD72   LSRA
        CMPA    #$01
        BNE     LBD62
        LSRB
        LSRB
        CMPB    -1,X
        BEQ     LBD81
        DEC     1,S
        BNE     LBD5F
LBD81   STB     ,-X
        LDB     2,S
        DECB
        BPL     LBD59
        PULS    A,X,PC

LBD8A   INC     <CasTemp
        LDB     $FF20
        RORB
        RTS

LBD91   CLR     <CasTemp
        TST     <CasPhaseFlag
        BNE     LBD9E
LBD97   BSR     LBDA0
LBD99   BSR     LBD8A
        BCC     LBD99
        RTS

LBD9E   BSR     LBD99
LBDA0   BSR     LBD8A
        BCS     LBDA0
        RTS

CasBitIn:
LBDA5   BSR     LBD91
        LDB     <CasTemp
        DECB
        CMPB    <CasPartrt
        RTS

CasByteIn:
LBDAD   LDA     #$08
        STA     <CasBitCount
LBDB1   BSR     CasBitIn LBDA5
        RORA
        DEC     <CasBitCount
        BNE     LBDB1
        RTS

LBDB9   CLR     <CasTemp
        BSR     LBDA0
        BRA     LBDC3

LBDBF   CLR     <CasTemp
        BSR     LBD99
LBDC3   LDB     <CasTemp
        CMPB    <CasMax24
        BHI     LBDCC
        CMPB    <CasMax12
        RTS

LBDCC   CLR     <CasBitCount
        RTS

CasMotorOn:
LBDCF   LDA     $FF21
        ORA     #$08
        STA     $FF21
        LDX     <CasMotorDelay
        LBRA    LBBC8

CasMotorOff:
LBDDC   LDA     $FF21
        ANDA    #$F7
        STA     $FF21
        ANDCC   #$AF
        RTS

CasReadLeader:
LBDE7   ORCC    #$50
        BSR     CasMotorOn LBDCF
        CLR     <CasBitCount
LBDED   BSR     LBD97
LBDEF   BSR     LBDB9
        BHI     LBDFF
LBDF3   BSR     LBDBF
        BCS     LBE03
        INC     <CasBitCount
        LDA     <CasBitCount
        CMPA    #$60
        BRA     LBE0D

LBDFF   BSR     LBDBF
        BHI     LBDEF
LBE03   BSR     LBDB9
        BCS     LBDF3
        DEC     <CasBitCount
        LDA     <CasBitCount
        ADDA    #$60
LBE0D   BNE     LBDED
        STA     <CasPhaseFlag
        RTS

CasByteOut:
LBE12   PSHS    A
        LDB     #$01
LBE16   LDY     #DBE44
        LDA     <CasLastSine
        STA     $FF20
        BITB    ,S
        BNE     LBE30
LBE23   LDA     ,Y+
        CMPY    #LBE68
        BCC     LBE3D
        STA     $FF20
        BRA     LBE23

LBE30   LDA     ,Y++
        CMPY    #LBE68
        BCC     LBE3D
        STA     $FF20
        BRA     LBE30

LBE3D   STA     <CasLastSine
        ASLB
        BCC     LBE16
        PULS    A,PC

DBE44   FCB     $80
        FCB     $90
        FCB     $A8
        FCB     $B8
        FCB     $C8
        FCB     $D8
        FCB     $E8
        FCB     $F0
        FCB     $F8
        FCB     $F8
        FCB     $F8
        FCB     $F0
        FCB     $E8
        FCB     $D8
        FCB     $C8
        FCB     $B8
        FCB     $A8
        FCB     $90
        FCB     $78
        FCB     $68
        FCB     $50
        FCB     $40
        FCB     $30
        FCB     $20
        FCB     $10
        FCB     $08
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $08
        FCB     $10
        FCB     $20
        FCB     $30
        FCB     $40
        FCB     $50
        FCB     $68

LBE68   PSHS    B,Y
        ORCC    #$50
        LBSR    CasMotorOn LBDCF
        LDA     #$55
        LDX     <CasLeadCount
LBE73   BSR     CasByteOut LBE12
        LEAX    -1,X
        BNE     LBE73
        PULS    B,Y,PC

	ifne	dragon64
	
DoSerialIn   
	PSHS    CC,B
        ORCC    #$50
        LDA     #$08
        LDB     $FF06
        ORB     #$01
        STB     $FF06
        ANDB    #$FE
LBE8B   BITA    $FF05
        BEQ     LBE8B
        STB     $FF06
        LDA     $FF04
        PULS    CC,B,PC
	
DoSerialOut:
LBE98   PSHS    CC,B
        LDB     #$10
LBE9C   BITB    $FF05
        BEQ     LBE9C
        STA     $FF04
        PULS    CC,B,PC


DoSetBaud
	CMPB    #$07
        BCC     LBEBC
        LDX     #DBEBE
        ABX
        LDB     TextSerBaudRate LFF07
        ANDB    #$F0
        ORB     ,X
        STB     TextSerBaudRate LFF07
        ANDCC   #$FE
        BRA     LBEBD

LBEBC   COMB
LBEBD   RTS

DBEBE   FCB     $03
        FCB     $06
        FCB     $07
        FCB     $08
        FCB     $0A
        FCB     $0C
        FCB     $0E

LBEC5   BNE     LBEDB
LBEC7   LDB     $FF22
        RORB
        BCS     LBEC7
LBECD   CLR     $FF02
        LDB     $FF00
        ORB     #$80
        INCB
        BNE     LBECD
        JMP     >LBCFD	

LBEDB   PULS    B
        JMP     >LBE98

LBEE0   LDB     $0151
        INCB
        BNE     LBEE9
        STB     $011D
LBEE9   TSTA
        BEQ     LBEFD
        LDB     $011F
        CMPA    $011D
        BEQ     LBEF7
        ASLB
        ASLB
        ASLB
LBEF7   STB     $011E
        STA     $011D
LBEFD   PULS    B,X,PC

	ifeq	dragon64ram
LBEFF   CLR     TextPrnSelFlag L03FF
        CLR     TextSerEOLDelay L03FD
        CLR     $03FE
        LDU     #TextPrnAutoCRLF L0148
        RTS
	endc
	
	else
DoSerialIn   
	RTS
	
DoSerialOut   
	RTS

DoSetBaud   
	COMB
        RTS
	endc
	
	ifne	dragon64
LBF0C   DECB
        BEQ     LBF12
        JMP     >LBD0F

LBF12   LDY     TextSerEOLDelay L03FD
        BEQ     LBF1F
LBF18   JSR     >LBBC5
        LEAY    -1,Y
        BNE     LBF18
LBF1F   RTS

LBF20   LDB     $FF05
        BPL     LBF32
        ANDB    #$08
        BEQ     LBF31
        LDA     $FF06
        ANDA    #$FE
        STA     $FF06
LBF31   RTI
LBF32   LSR     $0151
        DEC     $011E
        BNE     LBF46
        LDA     #$FF
        LDX     #$0151
LBF3F   STA     ,X+
        CMPX    #BasJoyVal0 L015A
        BCS     LBF3F
LBF46   JMP     >BasIRQVec L9D3D

	ifeq	dragon64ram

DoBoot64
	NOP
        ORCC    #$50
        LDX     #LBF5A
        LDU     #CasIOBuff L01DA
        LDB     #$8F
        LBSR    LBB97
        JMP     CasIOBuff L01DA

LBF5A   LDA     $011A
        CMPA    #$55
        BNE     LBF77
        STA     $FFDF
        LDX     #$C000
        LDD     #$0000
LBF6A   ADDD    ,X++
        CMPX    #$FF00
        BCS     LBF6A
        CMPD    $011B
        BEQ     LBFE6
	
LBF77   STA     $FFDE
        LDA     $FF23
        ANDA    #$FB
        STA     $FF23
        LDB     $FF22
        ORB     #$04
        STB     $FF22
        ORA     #$04
        STA     $FF23
        LDA     $FF22
        ANDA    #$FB
        STA     $FF22
        LDU     >DBFF0
        CMPU    #$3634
        BEQ     LBFA8
        ORA     #$04
        STA     $FF22
        JMP     >BasFCError L8B8D
	
LBFA8   CLR     $011B
        CLR     $011C
        LDX     #BasicHWInit L8000
        LDY     #$C000
LBFB5   LDD     ,X++
        STA     $FFDF
        STD     ,Y++
        ADDD    $011B
        STD     $011B
        CMPY    #$FF00
        BCC     LBFCD
        STA     $FFDE
        BRA     LBFB5

LBFCD   LDX     #DBFF0
        STX     <AddrRamTop
        STX     <AddrFWareRamTop
        STX     <BasVarStrTop
        LEAX    $FF38,X
        STX     <AddrStack
        TFR     X,S
        LDX     #DoBoot64
        STX     <IndVecReset
        JMP     $C000


LBFE6   JMP     $C003
	endc

	else
	FDB     $0000
	endc
	
	ifne	dragon64ram
	FILL	$00,$FFF0-*			; Fill spare bytes with zeros
	else
	FILL	$00,$BFF0-*			; Fill spare bytes with zeros
	endc
	
DBFF0	
	ifne	dragon64
	FDB	$3634				; '64' 
	else
        FDB     $0000
	endc
        FDB     SecVecSWI3
        FDB     SecVecSWI2
        FDB     SecVecFIRQ
        FDB     SecVecIRQ
        FDB     SecVecSWI
        FDB     SecVecNMI
	
	ifne	dragon64ram
	FDB	$C000
	else
	ifne	resettest
	FDB	NewSysReset
	else
        FDB     SysReset
	endc
	endc
